<?php
/*
** @name: admin_img.php
**
** @description: controller for plan images administration
** @author: Quentin George <q.george@esia-sa.com>
** @date: 06/04/2022
*/

$path = $_SERVER['DOCUMENT_ROOT'];
include_once $path . '/plugin/plan/inc/common_admin_plan.inc.php';
include_once LIB_PLAN_IMG;

try
{
	if($action == 'admin')
	{
		// Get objects by types name to have all container & links objects in one array
		$objectsByTypes = $plan->getListObjectsByTypeName(TRUE, $path);
		// For link format source and target ids from old naming to new (old params saved in DB are like obj_type_id.obj_id.'object' new param is only the object id example: 33object become 3)
		$regex = '/\d(\d+)object/';
		foreach($objectsByTypes['link'] as $key => $obj)
		{
			if(!empty($obj['params']['service']))
			{
				$obj['params']['service'] = intval(explode(",", $obj['params']['service'])[0]);
			}
			if(!is_numeric($obj['params']['source']))
			{
				$srcMatches = array();
				preg_match($regex, $obj['params']['source'], $srcMatches);
				if(isset($srcMatches[1]))
				{
					$objectsByTypes['link'][$key]['params']['source'] = $srcMatches[1];
				}
			}
			if(!is_numeric($obj['params']['target']))
			{
				$targMatches = array();
				preg_match($regex, $obj['params']['target'], $targMatches);
				if(isset($targMatches[1]))
				{
					$objectsByTypes['link'][$key]['params']['target'] = $targMatches[1];
				}
			}
		}
		$smarty->assign('objectsByTypes', $objectsByTypes);

		// Get Icons menu list
		$icons = Model_plan::getIconsByGroups($dbconnection, $lang);
		$smarty->assign('icons', $icons);

		// Get background images list

		// From default plan dir
		$commonBgImages = array();
		$commonBgImages = getBgImgFilesFromDir($path, DATA_PLANS);
		// Remove default bg img from list
		unset($commonBgImages[array_search(DEFAULT_BG_FILE, $commonBgImages)]);
		// Sort
		sort($commonBgImages);
		$smarty->assign('commonBgImages', $commonBgImages);
		$smarty->assign('commonBgImgDir', DATA_PLANS);

		// From plan id dir
		$linkedBgImages = array();
		$planDataDir = DATA_PLANS . 'plan' . $plan->getId() . '/';
		$isPlanDir = is_dir($path . $planDataDir);
		if($isPlanDir)
		{
			$linkedBgImages = getBgImgFilesFromDir($path, $planDataDir);
			sort($linkedBgImages);
		}
		$smarty->assign('linkedBgImages', $linkedBgImages);
		$smarty->assign('linkdedBgImgDir', $planDataDir);

		// Get selected bg img
		$selectedImg = '';
		// If url & not default folder to be sure there is a file on the url
		$planParamUrl = $plan->getParams('url');
		if(!empty($planParamUrl) && $planParamUrl != DATA_PLANS)
		{
			$selectedImg = $planParamUrl;
		}
		$smarty->assign('selectedImg', $selectedImg);
		$smarty->assign('defaultBgImg', DEFAULT_BG_FILE);
	}
}
catch(\EsiaDbException | \Plan\PlanException | \Exception $e)
{
	$errors[] = $e->getMessage();
}
$smarty->assign('errors', $errors);

$smarty->display($templatePath . 'admin_img.html');


function getBgImgFilesFromDir($path, $dirPath)
{
	$list = array();
	$dir = opendir($path . $dirPath);
	for($i=0; $f=readdir($dir); $i++)
	{
		if(is_file($path . $dirPath . $f))
		{
			$fileExt = preg_replace("#(.+)\.(.+)#", "$2", $f);
			$fileExt = strtolower($fileExt);
			if(in_array($fileExt, VALID_BG_FILE_EXT))
			{
				$list[] = $f;
			}
		}
	}
	closedir($dir);

	return $list;
}
?>
