<?php
/*
** @name: dialog_plan.php
**
** @description: controller for all plans common dialog windows
** @author: Quentin George <q.george@esia-sa.com>
** @date: 06/04/2022
*/

$path = $_SERVER['DOCUMENT_ROOT'];
include_once $path . '/plugin/plan/inc/common_dialog_plan.inc.php';

// Text Object Min & Max styles Properties
define('OBJ_STYLES_LIMITS_MIN', array(
	'fontSize' => 10,
	'bgOpacity' => 0,
	'borderWidth' => 0,
	'borderRad' => 0,
	'padding' => 0
));
define('OBJ_STYLES_LIMITS_MAX', array(
	'fontSize' => 70,
	'bgOpacity' => 1,
	'borderWidth' => 10,
	'borderRad' => 200,
	'padding' => 100
));

try
{
	if($action == 'delPlan')
	{
		// Get plan user access
		$access = Model_plan::getPlanUserAccess($dbconnection, $planId, $user->getId());

		if(isset($_POST['confirmIdPlan']) && $_POST['confirmIdPlan'] == $planId)
		{
			/*
			** Validation form sended
			*/
			if(!$isUserPerms['admin'] || ($access != 'WRITE' && $access != 'CREATOR'))
			{
				throw new \Exception('Cannot delete this plan ! User privilege missing.');
			}
			// Authorize deletion only if the plan id from the confirmation form is the same than the requested plan id to delete
			Model_plan::deletePlan($dbconnection, $planId);

			// Delete data folder
			if(is_dir($path . DATA_PLANS . 'plan' . $planId))
			{
				rmdirRecursive($path . DATA_PLANS . 'plan' . $planId);
			}

			// Success message
			$_SESSION["plan_msg_success"] = strtoupper($_POST['name']) . ": " . $lang['PLAN_DELETE_SUCCESS'];

			// Back to folder parent (null = racine)
			$url = $foldersURL;
			if(!empty($_POST['folder_id']) && is_numeric($_POST['folder_id']))
			{
				$url .= "?id=" . $_POST['folder_id'];
			}

			echo json_encode(array('url'=> $url));
			exit(0);
		}
		else
		{
			/*
			** Display html content
			*/
			if(!$isUserPerms['admin'] || ($access != 'WRITE' && $access != 'CREATOR'))
			{
				$smarty->display('no_access.html');
				exit(0);
			}
			$smarty->assign('planName', $_POST['name']);
			$smarty->assign('folderId', $_POST['folder_id']);
			$linkedObjs = Model_plan::getObjectsLinkedOnPlan($dbconnection, $planId);
			$smarty->assign('linkedObjs', $linkedObjs);
		}
	}
	elseif($action == 'serviceGraph')
	{
		// Open node status pop-up when ctrl+right-click on object linked to node.
		$service = NULL;
		if(!empty($_POST['serviceId']) && is_numeric($_POST['serviceId']))
		{
			$service = Model_esia::getServices($dbconnection, NULL, intval($_POST['serviceId']))[0];
			$service['common_name'] = Model_esia::getCommonName($lang, $service['name'], $service['check_plugin_name']);
		}

		if(empty($service))
		{
			throw new \Exception('Failed to retreive service data.');
		}

		if(isset($_POST['isIframe']) && $_POST['isIframe'] == TRUE)
		{
			$smarty->assign('isIframe', TRUE);
		}

		$smarty->assign('service', $service);
	}
	elseif($action == 'updateObjText')
	{
		// Get plan user access
		$access = Model_plan::getPlanUserAccess($dbconnection, $planId, $user->getId());

		if(isset($_POST['confirmIdPlan']) && $_POST['confirmIdPlan'] == $planId)
		{
			/*
			** Validation form sended
			*/
			if(!$isUserPerms['admin'] || ($access != 'WRITE' && $access != 'CREATOR'))
			{
				throw new \Exception('Cannot update this plan object ! User privilege missing.');
			}

			$params = FALSE;
			if(isset($_POST['params']))
			{
				$params = json_decode($_POST['params'], TRUE); // False if not decoded
			}

			if(!empty($_POST['object_id']) && $params)
			{
				$params['style'] = checkObjTextMinMaxStylesVal($params['style']);

				if(is_numeric($_POST['object_id']))
				{
					// Update existing object
					Model_plan::updateObjectParamJSON($dbconnection, $_POST['object_id'], json_encode($params), array());
				}
				else
				{
					// Save new object
					$_POST['object_id'] = Model_plan::addObject($dbconnection, $planId, $planId.'-text', 'text', json_encode($params));
				}

				echo json_encode($_POST['object_id']); // We must return somthing or ajax promise doesn't goes into done fct
			}
			else
			{
				throw new \Exception('ERROR update : wrong object parameters type');
			}
			exit(0);
		}
		else
		{
			/*
			** Display html content
			*/
			if(!$isUserPerms['admin'] || ($access != 'WRITE' && $access != 'CREATOR'))
			{
				$smarty->display('no_access.html');
				exit(0);
			}

			$obj = $_POST['object'];
			$obj = json_decode($obj, TRUE);
			$smarty->assign('object', $obj);
			$plan = Model_plan::getPlans($dbconnection, $user->getId(), $planId)[$planId];
			$planController = $apache_path . explode('?', $plan['type_controller'])[0];
			$smarty->assign('planController', $planController);
			$limits = array(
				'min' => OBJ_STYLES_LIMITS_MIN,
				'max' => OBJ_STYLES_LIMITS_MAX
			);
			$smarty->assign('limits', $limits);
		}
	}
	elseif($action == 'updateObjService')
	{
		// Get plan user access
		$access = Model_plan::getPlanUserAccess($dbconnection, $planId, $user->getId());

		if(isset($_POST['confirmIdPlan']) && $_POST['confirmIdPlan'] == $planId)
		{
			/*
			** Validation form sended
			*/
			if(!$isUserPerms['admin'] || ($access != 'WRITE' && $access != 'CREATOR'))
			{
				throw new \Exception('Cannot update this plan object ! User privilege missing.');
			}

			$formData = FALSE;
			if(isset($_POST['formData']))
			{
				$formData = json_decode($_POST['formData'], TRUE); // False if not decoded
			}
			$params = FALSE;
			if(isset($_POST['params']))
			{
				$params = json_decode($_POST['params'], TRUE);
			}

			if(!empty($_POST['object_id']) && $formData && $params)
			{
				if(is_numeric($_POST['object_id']))
				{
					// Get original object
					$obj = Model_plan::instanceObjectPlans($dbconnection, $planId, array($_POST['object_id']), FALSE)[$_POST['object_id']];
					$obj = $obj->toArray();

					// Update only required params
					// Position
					if(isset($params['position']))
					{
						$obj['params']['position'] = $params['position'];
					}
					// Size
					if(isset($params['size']))
					{
						$obj['params']['size'] = $params['size'];
					}
				}
				else
				{
					$obj = array(
						'id' => $_POST['object_id'],
						'type_name' => 'service',
						'name' => $planId.'-service',
						'params' => $params, // Copy all given params (position, size, ...)
					);
				}

				// Update object for new parameters (keep position, size, ...)
				if(!empty($formData['sub_type']))
				{
					$obj['params']['sub_type'] = $formData['sub_type'];
				}

				// Styles
				$obj['params']['style'] = array(
					'title' => array(
						'fontSize' => checkObjSingleStyleVal($formData['titleFontSize'], 'fontSize'),
						'fontColor' => substr($formData['titleFontColor'], 1), // Remove # for colors values
						'bgColor' => substr($formData['titleBgColor'], 1),
						'bgOpacity' => checkObjSingleStyleVal($formData['titleBgOpacity'], 'bgOpacity'),
					),
					'content' => array(
						'fontSize' => checkObjSingleStyleVal($formData['contentFontSize'], 'fontSize'),
						'fontColor' => substr($formData['contentFontColor'], 1),
						'bgColor' => substr($formData['contentBgColor'], 1),
						'bgOpacity' => checkObjSingleStyleVal($formData['contentBgOpacity'], 'bgOpacity'),
					),
					'box' => array(
						'borderColor' => substr($formData['borderColor'], 1),
						'borderWidth' => checkObjSingleStyleVal($formData['borderWidth'], 'borderWidth'),
						'borderRadTopLeft' => checkObjSingleStyleVal($formData['topLeftRad'], 'borderRad'),
						'borderRadTopRight' => checkObjSingleStyleVal($formData['topRightRad'], 'borderRad'),
						'borderRadBottomRight' => checkObjSingleStyleVal($formData['bottRightRad'], 'borderRad'),
						'borderRadBottomLeft' => checkObjSingleStyleVal($formData['bottLeftRad'], 'borderRad'),
						'padTop' => checkObjSingleStyleVal($formData['topPad'], 'padding'),
						'padRight' => checkObjSingleStyleVal($formData['rightPad'], 'padding'),
						'padBottom' => checkObjSingleStyleVal($formData['bottPad'], 'padding'),
						'padLeft' => checkObjSingleStyleVal($formData['leftPad'], 'padding'),
					)
				);

				// Node
				if(is_numeric($formData['selectNode']))
				{
					$obj['params']['device_id'] = $formData['selectNode'];
				}
				else
				{
					$obj['params']['device_id'] = NULL;
				}

				// Delete old data values and services
				$obj['params']['values'] = array();
				$obj['params']['services'] = array();

				// Specific data for each sub Type (Value, State, Message)
				switch($formData['sub_type'])
				{
					case 'value':
						$obj = updObjServiceVal($obj, $formData);
						break;
					case 'state':
						$obj = updObjServiceState($obj, $formData);
						break;
					case 'message':
						$obj = updObjServiceMsg($obj, $formData);
						break;
				}

				if(is_numeric($_POST['object_id']))
				{
					// Update DB object
					Model_plan::updateObjectParamJSON($dbconnection, $_POST['object_id'], json_encode($obj['params']), array());
				}
				else
				{
					// Save new object
					$obj['id'] = Model_plan::addObject($dbconnection, $planId, $obj['name'], $obj['type_name'], json_encode($obj['params']));
				}

				echo json_encode($obj); // Return the new updated object
			}
			else
			{
				throw new \Exception('ERROR update : missing form data');
			}
			exit(0);
		}
		else
		{
			/*
			** Display html content
			*/
			if(!$isUserPerms['admin'] || ($access != 'WRITE' && $access != 'CREATOR'))
			{
				$smarty->display('no_access.html');
				exit(0);
			}

			// Object parameters
			$obj = array();
			if(is_numeric($_POST['object_id']))
			{
				$obj = Model_plan::instanceObjectPlans($dbconnection, $planId, array($_POST['object_id']), FALSE)[$_POST['object_id']];
				$obj = $obj->toArray();
			}
			elseif(isset($_POST['object']))
			{
				$obj = json_decode($_POST['object'], TRUE);
			}
			if(empty($obj))
			{
				throw new \Exception('Cannot update this plan object ! Missing object parameter');
			}
			// Update current obj position
			if(!empty($_POST['position']))
			{
				$obj['params']['position']['x'] = $_POST['position']['x'];
				$obj['params']['position']['y'] = $_POST['position']['y'];
			}
			$smarty->assign('object', $obj);

			// Nodes list
			$nodes = Model_esia::getNodeList($dbconnection, $user->getId(), NULL, Model_esia::NODE_INSTANCE);
			$list = array();
			foreach($nodes as $node)
			{
				$selected = FALSE;
				if($node->getId() == $obj['params']['device_id'])
				{
					$selected = TRUE;
				}
				$nodeGroup = $node->getGroup();
				if(is_NULL($nodeGroup)) {
					$nodeGroup = $lang['GROUPNODE_ROOT'];
				}
				$list[] = array(
					'id' => $node->getId(),
					'name' => $nodeGroup . ' > ' . $node->getName(),
					'selected' => $selected
				);
			}
			$smarty->assign('adm_list_node', $list);

			// Service values list
			$rrdList = array();
			$rrdOrderedList = array();
			// Do this only if there is alread values saved for this object. List will be updated anyway when node select change
			if(!empty($obj['params']['device_id']) && !empty($obj['params']['values']))
			{
				// Get a list of all values added on this object (service_id-value_name)
				$objSelectedVal = array();
				foreach($obj['params']['values'] as $val)
				{
					$objSelectedVal[$val['order']] = $val['id'] . '-' .  $val['name'];
				}

				// Fetch list of all values available on this node
				$rrdList = getNodeRrdList($dbconnection, $lang, $obj['params']['device_id'], $objSelectedVal);
				// Get all values applied on this object to display it into the sortable select
				foreach($rrdList as $rrd)
				{
					if($rrd['selected'])
					{
						$rrdOrderedList[$rrd['order']] = $rrd;
					}
				}
				// Sort by key because the keys = order number but keys can be in wrong order into the array
				ksort($rrdOrderedList);
			}
			$smarty->assign('rrdList', $rrdList);
			$smarty->assign('rrdOrderedList', $rrdOrderedList);

			// Services list
			$servicesList = array();
			$servicesOrderedList = array();
			// Do this only if there is alread service saved for this object. List will be updated anyway when node select change
			if(!empty($obj['params']['device_id']) && !empty($obj['params']['services']))
			{
				// Get a list of all services added on this object
				$servicesSelected = array();
				foreach($obj['params']['services'] as $serv)
				{
					$servicesSelected[$serv['order']] = $serv['id'];
				}

				// Fetch list of all services available on this node
				$servicesList = getNodeServicesList($dbconnection, $lang, $obj['params']['device_id']);
				$servicesSelectedOrder = array_flip($servicesSelected);
				foreach($servicesList as $key => $service)
				{
					if(in_array($service['id'], $servicesSelected))
					{
						$servicesList[$key]['selected'] = TRUE;
						$servicesList[$key]['order'] = $servicesSelectedOrder[$service['id']];
					}
					else
					{
						$servicesList[$key]['selected'] = FALSE;
						$servicesList[$key]['order'] = NULL;
					}
				}
				// Get all values applied on this object to display it into the sortable select
				foreach($servicesList as $service)
				{
					if($service['selected'])
					{
						$servicesOrderedList[$service['order']] = $service;
					}
				}
				// Sort by key because the keys = order number but keys can be in wrong order into the array
				ksort($servicesOrderedList);
			}
			$smarty->assign('servicesList', $servicesList);
			$smarty->assign('servicesOrderedList', $servicesOrderedList);

			// Preview & Styles
			$plan = Model_plan::getPlans($dbconnection, $user->getId(), $planId)[$planId];
			$planController = $apache_path . explode('?', $plan['type_controller'])[0];
			$smarty->assign('planController', $planController);
			$limits = array(
				'min' => OBJ_STYLES_LIMITS_MIN,
				'max' => OBJ_STYLES_LIMITS_MAX
			);
			$smarty->assign('limits', $limits);

			// Display fields selected. Set all to default and replace the one of object subtype from the object saved values
			$optsSelected = json_decode($_POST['defOpts'], TRUE);
			$optsSelected[$obj['params']['sub_type']] = $obj['params']['options'];
			$smarty->assign('optsSelected', $optsSelected);
		}
	}
}
catch(\EsiaDbException | \Plan\PlanException | \Exception $e)
{
	$errors[] = $e->getMessage();
}

if($isJSONResp == TRUE)
{
	echo json_encode(array('error' => $errors));
	exit(0);
}
else
{
	$smarty->assign('errors', $errors);
}

$smarty->display($templatePath . 'dialog_plan.html');


// Update object service specific data for sub Type Value
function updObjServiceVal($obj, $formData)
{
	// Values
	if(!empty($formData['input_order_service_val']))
	{
		$orderVal = json_decode($formData['input_order_service_val']);
		// Get all values for each service ID
		foreach($orderVal as $key => $rrd)
		{
			$rrdArray = explode("-", $rrd); // $rrdArray[0]: service id, $rrdArray[1]: value name
			$obj['params']['values'][] = array(
				'id' => $rrdArray[0],
				'name' => $rrdArray[1],
				'order' => $key,
			);
		}
	}

	// Options checkboxes
	$checkBoxes = array(
		'isDisplayValNode' => $formData['isDisplayValNode'],
		'isDisplayValStateColor' => $formData['isDisplayValStateColor'],
		'isDisplayValName' => $formData['isDisplayValName'],
	);
	$obj['params']['options'] = cleanCheckBoxesVals($checkBoxes);

	return $obj;
}

// Update object service specific data for sub Type State
function updObjServiceState($obj, $formData)
{
	// Services
	if(!empty($formData['input_order_service_state']))
	{
		$orderSrv = json_decode($formData['input_order_service_state']);
		// Get all services for each service ID
		foreach($orderSrv as $key => $service)
		{
			$obj['params']['services'][] = array(
				'id' => $service,
				'order' => $key,
			);
		}
	}

	// Options checkboxes
	$checkBoxes = array(
		'isDisplayStateNode' => $formData['isDisplayStateNode'],
	);
	$obj['params']['options'] = cleanCheckBoxesVals($checkBoxes);
	// Other options
	$obj['params']['options']['selDisplayStateStateColor'] = $formData['selDisplayStateStateColor'];
	$obj['params']['options']['selDisplayStateServData'] = $formData['selDisplayStateServData'];

	return $obj;
}

// Update object service specific data for sub Type Message
function updObjServiceMsg($obj, $formData)
{
	// Service
	if(!empty($formData['msgServices']))
	{
		$obj['params']['services'][] = array(
			'id' => $formData['msgServices'],
			'order' => 0,
		);
	}

	// Options
	$checkBoxes = array(
		'isDisplayMsgNode' => $formData['isDisplayMsgNode'],
		'isDisplayMsgName' => $formData['isDisplayMsgName'],
	);
	$obj['params']['options'] = cleanCheckBoxesVals($checkBoxes);

	return $obj;
}

// Check if a value is between min & max limits & return value between those limits
function checkObjSingleStyleVal($val, $minMaxKey)
{
	// Be sure than val is a number
	$val = floatval($val); // float for opacity

	if($val < OBJ_STYLES_LIMITS_MIN[$minMaxKey])
	{
		// If $val is < than min, set it equal to min
		$val = OBJ_STYLES_LIMITS_MIN[$minMaxKey];
	} elseif($val > OBJ_STYLES_LIMITS_MAX[$minMaxKey])
	{
		// If $val is > than max, set it equal to max
		$val = OBJ_STYLES_LIMITS_MAX[$minMaxKey];
	}

	return $val;
}

// Check if all styles values are between they're min max limits for obj type text
function checkObjTextMinMaxStylesVal($style)
{
	// Font-size
	$style['fontSize'] = checkObjSingleStyleVal($style['fontSize'], 'fontSize');

	// Background-opacity
	$style['bgOpacity'] = checkObjSingleStyleVal($style['bgOpacity'], 'bgOpacity');

	// Border width
	$style['borderWidth'] = checkObjSingleStyleVal($style['borderWidth'], 'borderWidth');

	// Border radius
	$style['borderRadTopLeft'] = checkObjSingleStyleVal($style['borderRadTopLeft'], 'borderRad');
	$style['borderRadTopRight'] = checkObjSingleStyleVal($style['borderRadTopRight'], 'borderRad');
	$style['borderRadBottomLeft'] = checkObjSingleStyleVal($style['borderRadBottomLeft'], 'borderRad');
	$style['borderRadBottomRight'] = checkObjSingleStyleVal($style['borderRadBottomRight'], 'borderRad');

	// Padding
	$style['padTop'] = checkObjSingleStyleVal($style['padTop'], 'padding');
	$style['padBottom'] = checkObjSingleStyleVal($style['padBottom'], 'padding');
	$style['padLeft'] = checkObjSingleStyleVal($style['padLeft'], 'padding');
	$style['padRight'] = checkObjSingleStyleVal($style['padRight'], 'padding');

	return $style;
}
?>
