<?php
/*
** @name: dialog_rack.php
**
** @description: controller for plan rack dialog windows
** @author: Quentin George <q.george@esia-sa.com>
** @date: 06/04/2022
*/

$path = $_SERVER['DOCUMENT_ROOT'];
include_once $path . '/plugin/plan/inc/common_dialog_plan.inc.php';
include_once LIB_PLAN_RACK;

// Node types wich can be linked for each rack item types (all=keyword for all node types)
define('RACK_ITEMS_NODE_TYPES', array(
	"server" => array("server", "virtual server", "cloud serveur", "backup server", "galaxy", "Hypervisor", "Virtual cluster", "Syslog Server", "flows server", "website"),
	"storage" => array("NAS", "SAN"),
	"switch" => array("switch"),
	"router" => array("firewall", "router", "access Point", "controlleur Wifi"),
	"firewall" => array("firewall", "router", "access Point", "controlleur Wifi"),
	"pdu" => array("PDU"),
	"ups" => array("UPS"),
	"sensor" => array("none"),
	"generic" => array("all")
));

try
{
	if($action == 'updateObjRackItem')
	{
		// Get plan user access
		$access = Model_plan::getPlanUserAccess($dbconnection, $planId, $user->getId());

		if(isset($_POST['confirmIdPlan']) && $_POST['confirmIdPlan'] == $planId)
		{
			/*
			** Validation form sended
			*/
			if(!$isUserPerms['admin'] || ($access != 'WRITE' && $access != 'CREATOR'))
			{
				throw new \Exception('Cannot update this plan object ! User privilege missing.');
			}

			// Node
			$objConf['device_id'] = NULL;
			$objConf['services'] = array();
			if(is_numeric($_POST['selectNode']))
			{
				$objConf['device_id'] = $_POST['selectNode'];
			}

			// Services
			foreach($_POST as $key => $value)
			{
				// If the input name begin by 'services-' it's a service select value. If -1 is in $value there is no service configured on this service state icon
				if((strpos($key, 'services-') === 0) && !in_array('-1', $value))
				{
					// Get the service object name and create services array by name and id. ex => (cpu: "139", ping: "145")
					$serviceName = explode('-', $key)[1];
					// Remove all other services IDs if select was "All" (value = -2)
					if(in_array('-2', $value))
					{
						$value = array("-2");
					}
					$objConf['services'][$serviceName] = $value;
				}
			}

			if(is_numeric($_POST['object_id']))
			{
				$obj['params']['config'] = $objConf;
				$obj['id'] = intval($_POST['object_id']);

				// Update DB object params config
				Model_plan::updateObjectParamJSON($dbconnection, $obj['id'], json_encode($objConf), array('config'));
			}
			elseif(isset($_POST['obj_params_old']))
			{
				// New object not saved in DB yet
				$obj['params'] = $_POST['obj_params_old'];

				// Check if rack item params are valid
				if(isValidRackItem($obj['params']['common']['item_type'], $obj['params']['common']['item_subtype'], $obj['params']['common']['u_size']))
				{
					$obj['params']['config'] = $objConf;
					$obj['type_name'] = 'rack_item';
					$obj['id'] = Model_plan::addObject($dbconnection, $planId, $planId.'-rack_item', $obj['type_name'], json_encode($obj['params']));
				}
				else
				{
					throw new \Exception($lang['PLAN_ERROR_RACK'] . ": " . $lang['PLAN_ERROR_INVALID_ITEM'] . " ID=" . $_POST['object_id'] . " (" . $obj['params']['common']['item_type'] . "-" . $obj['params']['common']['item_subtype'] . "-" . $obj['params']['common']['u_size'] . "U)");
				}
			}

			// Return new device name
			if(is_numeric($obj['params']['config']["device_id"]))
			{
				$obj['tmp_params']['device_name'] = Model_plan::getDeviceNameById($dbconnection, $obj['params']['config']["device_id"]);
			}

			echo json_encode($obj); // Return the new updated object config
			exit(0);
		}
		else
		{
			/*
			** Display html content
			*/
			if(!$isUserPerms['admin'] || ($access != 'WRITE' && $access != 'CREATOR'))
			{
				$smarty->display('no_access.html');
				exit(0);
			}
			$smarty->assign('planId', $planId);

			// Object parameters
			$objParams = array();
			if(is_numeric($_POST['objId']))
			{
				$objParams = Model_plan::getObjectParamJSON($dbconnection, $_POST['objId']);
			}
			elseif(isset($_POST['objParamsCommon']))
			{
				// New object not saved in DB yet
				$objParams['common'] = $_POST['objParamsCommon'];
			}

			// Check if rack item params are valid
			if(isValidRackItem($objParams['common']['item_type'], $objParams['common']['item_subtype'], $objParams['common']['u_size']))
			{
				// Icon preview title
				$langItemType = 'PLAN_ELT_' . strtoupper($objParams['common']['item_type']);
				$strItemSubType = '';
				if($objParams['common']['item_subtype'] !== 'none')
				{
					$strItemSubType = '-' . $objParams['common']['item_subtype'];

					switch($objParams['common']['item_type'])
					{
						case 'switch':
						case 'router':
						case 'firewall':
						case 'sensor':
							$strItemSubType .= $lang['PLAN_SW_PORTS'];
							break;
					}
				}
				$iconName = $lang[$langItemType] . $strItemSubType . '-' . $objParams['common']['u_size'] . 'U';
				$smarty->assign('iconName', $iconName);

				// Display states icons
				$displayStates = array();
				foreach(RACK_ITEMS_STATES_TO_DISPLAY[$objParams['common']['item_type']] as $state)
				{
					$displayStates[$state] = array(
						'name' => $state,
						'isMulti' => RACK_ITEMS_STATES_OPTS[$state]['isMulti'],
						'svg' => $state,
					);

					if(isset(RACK_ITEMS_STATES_OPTS[$state]['svg_variants']) && in_array($objParams['common']['item_subtype'], RACK_ITEMS_STATES_OPTS[$state]['svg_variants']))
					{
						$displayStates[$state]['svg'] .= '_' . $objParams['common']['item_subtype'];
					}
				}
				$smarty->assign('displayStates', $displayStates);

				// Nodes list
				$validNodeTypes = RACK_ITEMS_NODE_TYPES[$objParams['common']['item_type']];
				$nodes = Model_esia::getNodeList($dbconnection, $user->getId(), NULL, Model_esia::NODE_INSTANCE);
				$list = array();
				foreach($nodes as $node)
				{
					if($validNodeTypes[0] == 'all' || in_array($node->getType(), $validNodeTypes))
					{
						$selected = FALSE;
						if($node->getId() == $objParams['config']['device_id'])
						{
							$selected = TRUE;
						}
						$nodeGroup = $node->getGroup();
						if(is_null($nodeGroup)) {
							$nodeGroup = $lang['GROUPNODE_ROOT'];
						}
						$list[] = array(
							'id' => $node->getId(),
							'name' => $nodeGroup . ' > ' . $node->getName(),
							'selected' => $selected
						);
					}
				}
				$smarty->assign('adm_list_node', $list);

				// Services list
				$servicesList = array();
				if($objParams['config']['device_id'] !== NULL)
				{
					$servicesList = formatNodeServicesList($dbconnection, $lang, $objParams['config']['device_id']);
				}
				$smarty->assign('servicesList', $servicesList);
				$smarty->assign('confServices', $objParams['config']['services']);

				$smarty->assign('object', array('id' => $_POST['objId'], 'params' => $objParams));
			}
			else
			{
				throw new \Exception($lang['PLAN_ERROR_RACK'] . ": " . $lang['PLAN_ERROR_INVALID_ITEM'] . " ID=" . $_POST['objId'] . " (" . $objParams['common']['item_type'] . "-" . $objParams['common']['item_subtype'] . "-" . $objParams['common']['u_size'] . "U)");
			}
		}
	}
	elseif($action == 'emptyTrash')
	{
		// Get plan user access
		$access = Model_plan::getPlanUserAccess($dbconnection, $planId, $user->getId());

		if(isset($_POST['confirmIdPlan']) && $_POST['confirmIdPlan'] == $planId)
		{
			/*
			** Validation form sended
			*/
			if(!$isUserPerms['admin'] || ($access != 'WRITE' && $access != 'CREATOR'))
			{
				throw new \Exception('Cannot delete this plan ! User privilege missing.');
			}

			if(isset($_POST['delete_objects']) && !empty($_POST['delete_objects']))
			{
				Model_plan::deleteObjects($dbconnection, $planId, $_POST['delete_objects']);
			}
			else
			{
				throw new \Exception($lang['PLAN_ERROR_RACK_ITEMS_DEL'] . ": " . $lang['PLAN_ERROR_RACK_ITEMS_DEL_EMPTY']);
			}

			echo json_encode(array('success' => 'Trash successfully deleted'));
			exit(0);
		}
		else
		{
			/*
			** Display html content
			*/
			if(!$isUserPerms['admin'] || ($access != 'WRITE' && $access != 'CREATOR'))
			{
				$smarty->display('no_access.html');
				exit(0);
			}
			$smarty->assign('planId', $planId);
		}
	}
}
catch(\EsiaDbException | \Plan\PlanException | \Exception $e)
{
	$errors[] = $e->getMessage();
}

if($isJSONResp == TRUE)
{
	echo json_encode(array('error' => $errors));
	exit(0);
}
else
{
	$smarty->assign('errors', $errors);
}

$smarty->display($templatePath . 'dialog_rack.html');

?>
