<?php
/*
** @name: display_img.php
**
** @description: controller for plan img display ajax requests
** @author: Quentin George <q.george@esia-sa.com>
** @date: 06/04/2022
*/

$path = $_SERVER['DOCUMENT_ROOT'];
include_once $path . '/common.php';
include_once $path . '/Model_adm.php';
include_once $path . '/libs/convert.lib.php';
include_once $path . '/plugin/plan/plan.conf.php';
include_once MODEL_PLAN;
include_once LIB_PLAN;
include_once LIB_PLAN_IMG;

// Action
if(isset($_GET['action']))
{
	$action = $_GET['action'];
}
elseif(isset($_POST['action']))
{
	$action = $_POST['action'];
}

try
{
	if($action == 'delBgImg')
	{
		if(!empty($_POST['dir']) && !empty($_POST['img']))
		{
			// Check if the file we want to remove is in DATA_PLANS to avoid removing file anywhere else on the server
			// If strpos return 0 means str match and is at the string beginning
			if(strpos($_POST['dir'], $path . DATA_PLANS) != 0)
			{
				throw new \Exception('Wrong directory. You can only remove files from &laquo;' . $path . DATA_PLANS . '&raquo;');
			}

			if($_POST['img'] != DEFAULT_BG_FILE) // IMAGE BLANCHE PEUT PAS ETRE SUPPRIME
			{
				$returDelImg = unlink($path . $_POST['dir'] . "miniature/min_" . $_POST['img']);
				if($returDelImg == TRUE)
				{
					$returDelImg = unlink($path . $_POST['dir'] . $_POST['img']);
					if($returDelImg == TRUE)
					{
						echo json_encode(array('success' => $lang['PLAN_DELETE_IMAGE_TEXT']));
						exit(0);
					}
					else
					{
						throw new \Exception('File img not writable');
					}
				}
				else
				{
					throw new \Exception('File img miniature not writable');
				}
			}
			else
			{
				throw new \Exception('File img esia not writable');
			}
		}
	}
	elseif($action == 'addBgImg')
	{
		if($_FILES['bgFile']['name'] != NULL) // Si une nouvelle image
		{
			// Check upload php file errors
			checkUploadFilePhpErrors($_FILES['bgFile']['error']);

			// Check if user as write right on this plan
			if(!is_numeric($_GET['id']) || !is_numeric($_POST['imgUserId']))
			{
				throw new Exception('Plan ID or user ID missing');
			}
			$userAccess = Model_plan::getPlanUserAccess($dbconnection, $_GET['id'], $_POST['imgUserId']);
			if($userAccess != 'WRITE' && $userAccess != 'CREATOR')
			{
				throw new Exception($lang['PLAN_ADD_FAILED_USER_RIGHTS']);
			}

			// Check if image should be on shared plan images folder or not (common plan image folder or planid image folder)
			$planDataDir = DATA_PLANS;
			$isPlanLinked = FALSE;
			if(!empty($_POST['isPlanLinked']) && $_POST['isPlanLinked'] == 1)
			{
				$isPlanLinked = TRUE;
				// Check if data folder exist for this plan and create it if not
				$planDataDir = DATA_PLANS . 'plan' . $_GET['id'] . '/';
				$isPlanDir = is_dir($path . $planDataDir);
				if(!$isPlanDir)
				{
					$isPlanDir = @mkdir($path . $planDataDir, 0755); // Main folder for this plan
					$isPlanDir = @mkdir($path . $planDataDir . 'miniature', 0755); // Thumbnail folder

					if(!$isPlanDir)
					{
						// If any folder creation fail delete main folder. Thumbnail folder is not created at this point
						@rmdir($path . $planDataDir);

						throw new Exception('Unable to create data images folder');
					}
				}
			}

			// Clean file name to avoid unexpected chars like é,&,@,...
			$_FILES['bgFile']['name'] = strtolower(cleanUploadFileName($_FILES['bgFile']['name']));

			if(!file_exists($path . $planDataDir . $_FILES['bgFile']['name'])) // Si elle n'existe pas déjà
			{
				if(move_uploaded_file($_FILES['bgFile']['tmp_name'], $path . $planDataDir . $_FILES['bgFile']['name']) == TRUE) // Si le dwl s'est bien passé
				{
					$params['imageHeader'] = "min_" . $_FILES['bgFile']['name'];

					// Thumbnail creation
					$fileExt = strtolower(pathinfo($_FILES['bgFile']['name'], PATHINFO_EXTENSION));
					$thumbnail = $path . $planDataDir . "miniature/min_" . $_FILES['bgFile']['name'];
					$upload_image = $path . $planDataDir . $_FILES['bgFile']['name'];

					list($width,$height) = getimagesize($upload_image);
					$ratio = $width/$height;
					$NouvelleHauteur = 58;
					$NouvelleLargeur = (($width * (($NouvelleHauteur * 100)/$height))/100 );

					$thumb_create = imagecreateTRUEcolor($NouvelleLargeur,$NouvelleHauteur);

					$white = imagecolorallocate($thumb_create, 255, 255, 255);
					imagefill($thumb_create, 0, 0, $white);

					switch($fileExt)
					{
						case 'jpg':
							$source = imagecreatefromjpeg($upload_image);
							break;
						case 'jpeg':
							$source = imagecreatefromjpeg($upload_image);
							break;
						case 'png':
							$source = imagecreatefrompng($upload_image);
							break;
						case 'gif':
							$source = imagecreatefromgif($upload_image);
							break;
						case 'bmp':
							$source = imagecreatefrombmp($upload_image);
							break;
						default:
							$source = imagecreatefromjpeg($upload_image);
					}

					imagecopyresampled($thumb_create , $source, 0, 0, 0, 0, $NouvelleLargeur, $NouvelleHauteur, $width,$height);

					$size = min(imagesx($thumb_create), imagesy($thumb_create));
					$im2 = imagecrop($thumb_create, ['x' => 0, 'y' => 0, 'width' => $size, 'height' => $size]);

					switch($fileExt)
					{
						case 'jpg' || 'jpeg':
						//	imagejpeg($thumb_create,$upload_image);
							imagejpeg($im2,$thumbnail,100);
							break;
						case 'png':
						//	imagepng($thumb_create,$upload_image);
							imagepng($im2,$thumbnail,100);
							break;
						case 'gif':
						//	imagegif($thumb_create,$upload_image);
							imagegif($im2,$thumbnail,100);
							break;
						default:
						//	imagejpeg($thumb_create,$upload_image);
							imagejpeg($im2,$thumbnail,100);
							break;
					}
					imagedestroy($im2);

					echo json_encode(array('bgImgDir' => $planDataDir, 'bgImage' => $_FILES['bgFile']['name'], 'isPlanLinked' => $isPlanLinked));
					exit(0);
				}
				else
				{
					throw new Exception('File not downloaded');
				}
			}
			else
			{
				throw new Exception('File name already exists');
			}
		}
		else
		{
			throw new Exception('No file uploaded');
		}
	}
	elseif($action == 'getLinkServiceValues')
	{
		if(!empty($_GET['service_id']))
		{
			$values = NULL;
			$service = Model_esia::getServices($dbconnection, NULL, $_GET['service_id'])[0];
			if(preg_match("/SNMP_SW_INTERFACE_EASY|INTERFACE_EASY/", $service['check_plugin_name']) && $service['real_state'] != 3)
			{
				$precision = 1;
				//si on est en dessous du MB/s
				if($service['value']['in'] < 1000000)
				{
					$precision = 0;
				}
				$convertIn = Convert\bitsPerSecond($service['value']['in'],$precision);
				//si on est en dessous du MB/s
				if($service['value']['out'] < 1000000)
				{
					$precision = 0;
				}
				else 
				{
					$precision = 1;
				}
				$convertOut = Convert\bitsPerSecond($service['value']['out'],$precision);
				$values['in']['val'] = 'In: ' . $convertIn['value'] . ' ' . $convertIn['unity'];
				$values['out']['val'] = 'Out: ' . $convertOut['value'] . ' ' . $convertOut['unity'];
				$percent_in = $service['value']['inPercent'];
				$percent_out = $service['value']['outPercent'];

				$values['in']['color'] = setColor($service['value']['inPercent']);
				$values['out']['color'] = setColor($service['value']['outPercent']);
			}
			elseif($service['value']['latency'])
			{
				$values['middle']['val'] = $lang['PLAN_LATENCY'] . ': ' . $service['value']['latency'] . ' ms';
			}

			echo json_encode($values);
			exit(0);
		}
	}
	elseif($action == 'displayAdmIconsMenu')
	{
		if(empty($_GET['apache_path']) || empty($_GET['templatePath']) || empty($_GET['planId']))
		{
			throw new \Exception('Missing required variables to display adm menu icon');
		}

		// Get icons & groups
		$icons = Model_plan::getIconsByGroups($dbconnection, $lang);
		$smarty->assign('icons', $icons);

		// Assign required vars
		$smarty->assign('plan', array('id' => $_GET['planId']));
		$smarty->assign('apache_path', $_GET['apache_path']);

		$smarty->display($_GET['templatePath'] . 'inc/admin_img_icons_menu.inc.html');
	}
	elseif($action == 'updatePlanParams')
	{
		// Instance the plan
		$plan = Model_plan::instancePlan($dbconnection, $_GET['id'], $user->getId(), FALSE, FALSE);

		// Plan common data (name, folder, users share, resources, refresh)
		$plan = setPlanCommonData($dbconnection, $lang, $plan, $_POST);

		// Plan img params
		if(!empty($_POST['bgImage']))
		{
			$plan->setParam(array('url'), $_POST['bgImage']);
		}
		else
		{
			$plan->setParam(array('url'), '');
		}

		// Update the plan in DB
		Model_plan::updatePlan($dbconnection, $user->getId(), $_GET['id'], $plan->getName(), json_encode($plan->getParams()), $plan->getFolderId(), $plan->getAccessUsersList());

		echo json_encode(TRUE);
		exit(0);
	}
	elseif($action == 'updatePlanContainersAndLinks')
	{
		// Update or add objects type container
		if(!empty($_POST['container_objects']))
		{
			$containerObjects = json_decode($_POST['container_objects'], TRUE);
			$newObjContainerIds = array();
			foreach($containerObjects as $objData)
			{
				if(!empty($objData['id']))
				{
					if(is_numeric($objData['id']))
					{
						// The id is valid for DB it's not beginning by 'tmp-'. The object already exist in DB we just update it
						$obj = Model_plan::instanceObjectPlans($dbconnection, $_GET['id'], array($objData['id']), FALSE)[$objData['id']];
						$obj->setParam(array('position'), $objData['params']['position']);
						$obj->setParam(array('size'), $objData['params']['size']);
						Model_plan::updateObject($dbconnection, $objData['id'], $_GET['id'], $objData['name'], json_encode($obj->getParams()), $objData['icon_id'], NULL);
					}
					else
					{
						// It's a 'tmp-' id so it's a new object. And save it's new DB ID into $newObjContainerIds where key is the tmp id. Used later for links
						$newObjContainerIds[$objData['id']] = Model_plan::addObject($dbconnection, $_GET['id'], $objData['name'], 'container', json_encode($objData['params']), $objData['icon_id'], NULL);
					}
				}
				else
				{
					throw new \Exception($lang['PLAN_ERROR_OBJ_SAVE']);
				}
			}
		}

		// Update or add objects type link
		if(!empty($_POST['link_objects']))
		{
			$linkObjects = json_decode($_POST['link_objects'], TRUE);
			foreach($linkObjects as $objData)
			{
				if(!empty($objData['id']))
				{
					// Replace links data which contains obj container tmp id's by New DB id's
					if(array_key_exists($objData['source'], $newObjContainerIds))
					{
						$objData['source'] = $newObjContainerIds[$objData['source']];
					}
					if(array_key_exists($objData['target'], $newObjContainerIds))
					{
						$objData['target'] = $newObjContainerIds[$objData['target']];
					}
					// Set name
					$objData['name'] = $objData['source'] . 'object-' . $objData['target'] . 'object';

					// Other parameters
					if(empty($objData['options']['isDisableArrows']))
					{
						$objData['options']['isDisableArrows'] = FALSE;
					}
					if(empty($objData['options']['isDisableLabels']))
					{
						$objData['options']['isDisableLabels'] = FALSE;
					}

					if(is_numeric($objData['id']))
					{
						// The id is valid for DB it's not beginning by 'tmp-'. The object already exist in DB we just update it
						$obj = Model_plan::instanceObjectPlans($dbconnection, $_GET['id'], array($objData['id']), FALSE)[$objData['id']];
						$obj->setParam(array('service'), $objData['service']);
						$obj->setParam(array('source'), $objData['source']);
						$obj->setParam(array('target'), $objData['target']);
						$obj->setParam(array('points'), $objData['points']);
						$obj->setParam(array('isDisableArrows'), $objData['options']['isDisableArrows']);
						$obj->setParam(array('isDisableLabels'), $objData['options']['isDisableLabels']);
						Model_plan::updateObject($dbconnection, $objData['id'], $_GET['id'], $objData['name'], json_encode($obj->getParams()), NULL, NULL);
					}
					else
					{
						$objParams = array(
							'service' => $objData['service'],
							'source' => $objData['source'],
							'target' => $objData['target'],
							'points' => $objData['points'],
							'isDisableArrows' => $objData['options']['isDisableArrows'],
							'isDisableLabels' => $objData['options']['isDisableLabels']
						);
						// It's a 'tmp-' id so it's a new object.
						Model_plan::addObject($dbconnection, $_GET['id'], $objData['name'], 'link', json_encode($objParams), NULL, NULL);
					}
				}
				else
				{
					throw new \Exception($lang['PLAN_ERROR_OBJ_SAVE']);
				}
			}
		}

		echo json_encode(TRUE);
		exit(0);
	}
}
catch(\EsiaDbException | \Plan\PlanException | \Exception $e)
{
	echo json_encode(array('error' => $e->getMessage()));
}

exit(0);


function setColor($percent)
{
	$val = "#AB0A00";

	$percent = intval($percent);

	if($percent <= 10)
	{
		$val = "#3d73ff";
	}
	elseif($percent <= 20)
	{
		$val = "#0093CF";
	}
	elseif($percent <= 30)
	{
		$val = "#66B131";
	}
	elseif($percent <= 40)
	{
		$val = "#D5FE00";
	}
	elseif($percent <= 50)
	{
		$val = "#FFFF32";
	}
	elseif($percent <= 60)
	{
		$val = "#FFDD00";
	}
	elseif($percent <= 70)
	{
		$val = "#FE9900";
	}
	elseif($percent <= 80)
	{
		$val = "#FE5900";
	}
	elseif($percent <= 90)
	{
		$val = "#FE0000";
	}
	else
	{
		$val = "#AB0A00";
	}
	return $val;
}

?>
