<?php
/*
** @name: display_plan.php
**
** @description: controller for plan common display ajax requests
** @author: Quentin George <q.george@esia-sa.com>
** @date: 06/04/2022
*/

$path = $_SERVER['DOCUMENT_ROOT'];
include_once $path . '/common.php';
include_once $path . '/Model_adm.php';
include_once $path . '/plugin/plan/plan.conf.php';
include_once MODEL_PLAN;
include_once LIB_PLAN;
include_once LIB_ESIA_CONVERT;

define('CONV_SERVICE_VALUE_UNITS', array(
	'bytes' => 'B',
	'bytes/s' => 'Bps',
	'bps' => 'Bps',
	'second' => 's',
	'millisecond' => 'ms',
	'percent' => '%',
	'packet/s' => 'Pck/s',
	'packetps' => 'Pck/s',
	'temperature' => '°',
	'temp' => '°',
	'celsius' => '°',
	'connections' => '',
	'number' => ''
));

// Action
if(isset($_GET['action']))
{
	$action = $_GET['action'];
}
elseif(isset($_POST['action']))
{
	$action = $_POST['action'];
}

try
{
	if($action == 'addObjectOnPlan')
	{
		if(isset($_POST['newObj']) && !empty($_POST['newObj']))
		{
			$obj = $_POST['newObj'];
			$object = json_decode($obj);
			$object->name = str_replace(NAME_FORBIDDEN_CHARS, NAME_REPLACEMENT_CHAR, trim($object->name));
			if(isset($object->type_name))
			{
				$type = $object->type_name;
			}
			else
			{
				$type = $object->type;
			}
			$newObjId = Model_plan::addObject($dbconnection, $_REQUEST['currentPlan'], $object->name, $type, json_encode($object->params), $object->icon);
			$newObj = Model_plan::instanceObjectPlans($dbconnection, $_REQUEST['currentPlan'], array($newObjId), FALSE)[$newObjId];
		}

		echo json_encode($newObj->toArray());
		exit(0);
	}
	elseif($action == 'getObject')
	{
		$obj = Model_plan::instanceObjectPlans($dbconnection, $_GET['id'], array($_GET['objId']), FALSE)[$_GET['objId']];

		echo json_encode($obj->toArray());
		exit(0);
	}
	elseif($action == 'isPlanWrittable')
	{
		if(empty($_GET['id']))
		{
			throw new \Exception($lang['PLAN_ERROR_PLAN_PARAM'] . ": " . $lang['PLAN_ERROR_PLAN_ID']);
		}

		// Instance the plan
		$plan = Model_plan::instancePlan($dbconnection, $_GET['id'], $user->getId(), FALSE, FALSE);
		// Check if page can be displayed
		$isUserPerms = getEsiaPlanPerms($dbconnection, $user);
		if(!$isUserPerms['admin'] || $plan->getCurrentUserAccess() != 'WRITE')
		{
			throw new \Exception($lang['PLAN_USER_PERMS_DENIED'] . " (User: " . $user->getName() . " Plan: " . $plan.getName() . ")");
		}

		echo json_encode(TRUE);
		exit(0);
	}
	elseif($action == 'deleteObjects')
	{
		// Delete items
		if(!empty($_POST['delete_objects']))
		{
			$deleteObjectsId = json_decode($_POST['delete_objects'], TRUE);
			foreach($deleteObjectsId as $key => $id)
			{
				// Check if not tmp object in list (new object not saved in DB yet)
				if(!is_numeric($id))
				{
					unset($deleteObjectsId[$key]);
				}
			}
			if(!empty($deleteObjectsId))
			{
				Model_plan::deleteObjects($dbconnection, $_GET['id'], $deleteObjectsId);
			}
		}
		else
		{
			throw new \Exception($lang['PLAN_ERROR_OBJ_DEL']);
		}

		echo json_encode(TRUE);
		exit(0);
	}
	elseif($action == 'updateObjects')
	{
		if(!empty($_GET['id']) && !empty($_POST['objects']))
		{
			$objects = json_decode($_POST['objects'], TRUE);
			foreach($objects as $objData)
			{
				if(!empty($objData['id']))
				{
					// Format object data
					if(is_numeric($objData['id']))
					{
						// The id is valid for DB it's not beginning by 'tmp-'
						$obj = Model_plan::instanceObjectPlans($dbconnection, $_GET['id'], array($objData['id']), FALSE)[$objData['id']];

						// Update only new params
						foreach($objData['params'] as $param => $value)
						{
							$obj->setParam(array($param), $value);
						}
					}
					else
					{
						// It's a 'tmp-' id so it's a new object
						$obj = new \Plan\ObjectPlan();
						$obj->setTypeName($objData['type_name']);
						$obj->setParams($objData['params']);
					}

					if(!empty($objData['name']))
					{
						$obj->setName($objData['name']);
					}

					if(!empty($objData['icon_id']))
					{
						$obj->setIconId($objData['icon_id']);
					}

					if(!empty($objData['link_plan_id']))
					{
						$obj->setLinkPlanId($objData['link_plan_id']);
					}

					// Save data into DB
					if(is_numeric($objData['id']))
					{
						// The object already exist in DB we just update it
						Model_plan::updateObject($dbconnection, $objData['id'], $_GET['id'], $obj->getName(), json_encode($obj->getParams()), $obj->getIconId(), $obj->getLinkPlanId());
					}
					else
					{
						// Add new object
						Model_plan::addObject($dbconnection, $_GET['id'], $obj->getName(), $obj->getTypeName(), json_encode($obj->getParams()), $obj->getIconId(), $obj->getLinkPlanId());
					}
				}
				else
				{
					throw new \Exception($lang['PLAN_ERROR_OBJ_SAVE']);
				}
			}
		}

		echo json_encode(TRUE);
		exit(0);
	}
	elseif($action == 'getNodeServices')
	{
		$servicesList = getNodeServicesList($dbconnection, $lang, $_GET['nodeId']);

		echo json_encode($servicesList);
		exit(0);
	}
	elseif($action == 'details')
	{
		// Open node status pop-up when ctrl+right-click on object linked to node.
		$alerts = array();
		if(!empty($_GET['objId']))
		{
			$obj = Model_plan::instanceObjectPlans($dbconnection, $_GET['id'], array($_GET['objId']), FALSE)[$_GET['objId']];
			$alerts = Model_plan::getObjectAlerts($dbconnection, $user, $lang, $obj->getParams(), $obj->getTypeName(), $obj->getLinkPlanId());
		}

		if(!empty($alerts))
		{
			echo json_encode($alerts);
		}
		else
		{
			echo json_encode(null);
		}
		exit(0);
	}
	elseif($action == 'getObjPlanLink')
	{
		// If an object is linked to a plan get the controller path to this plan
		$planController = '';
		if(!empty($_GET['objId']))
		{
			$obj = Model_plan::instanceObjectPlans($dbconnection, $_GET['id'], array($_GET['objId']), FALSE)[$_GET['objId']];

			if(!empty($obj->getLinkPlanId()))
			{
				$objPlanType = Model_plan::getPlanTypes($dbconnection, NULL, $obj->getLinkPlanId())[0];

				echo json_encode(array(
					'controller' => $objPlanType['controller'] . '&id=' . $obj->getLinkPlanId()
				));
				exit(0);
			}
		}
	}
	elseif($action == 'getNodeServicesValNames')
	{
		$rrdList = getNodeRrdList($dbconnection, $lang, $_GET['nodeId']);

		echo json_encode($rrdList);
		exit(0);
	}
	elseif($action == 'getNodeServicesValues')
	{
		$result = array('values' => array());
		$services = Model_esia::getServices($dbconnection, $_GET['nodeId']);
		$result['device_name'] = $services[0]['device_name'];

		// Reformat $_GET['values'] by service_id key to be reusable into a loop
		$objServices = array();
		foreach($_GET['values'] as $val)
		{
			$objServices[$val['id']][] = array('name' => $val['name'], 'order' => $val['order']);
		}

		foreach($services as $service)
		{
			// Only for services saved on the object
			if(array_key_exists($service['id'], $objServices))
			{
				// Get service related informations
				$common_name = Model_esia::getCommonName($lang, $service['name'], $service['check_plugin_name']);
				$serviceRrdInfo = Model_esia::getRRDInformationByServiceId($dbconnection, $service['id']);
				// Set data for each value saved on the object
				foreach($objServices[$service['id']] as $valArr)
				{
					// Remove unexpected , char at the end or start of value (Can happen for some service regex)
					$value = preg_replace('/^,+|,+$/', '', $service['value'][$valArr['name']]);
					// Calc the original value to match with unit
					if (isset($serviceRrdInfo[$valArr['name']]['calc']))
					{
						$value = eval('return ' . $value . $serviceRrdInfo[$valArr['name']]['calc'] . ';');
					}
					// Get value unit
					$unit = trim($serviceRrdInfo[$valArr['name']]['unity']);
					// Convert unit to be more relevant
					if(isset(CONV_SERVICE_VALUE_UNITS[strtolower($unit)]))
					{
						$unit = CONV_SERVICE_VALUE_UNITS[strtolower($unit)];
					}
					// Convert to KB, MB, GB, ...
					if($unit == 'B' || $unit == 'Bps') {
						$cBArr = Convert\bitsPerSecond($value);
						$value = $cBArr['value'];
						$unit = $cBArr['unity'];
					}

					$result['values'][$valArr['order']] = array(
						'order' => $valArr['order'],
						'name' => $common_name . ' > ' . ucfirst($valArr['name']),
						'value' => $value,
						'unit' => $unit,
						'state' => $service['state']
					);
				}
			}
		}

		echo json_encode($result);
		exit(0);
	}
	elseif($action == 'getNodeServicesData')
	{
		$result = array('services' => array());
		$services = Model_esia::getServices($dbconnection, $_GET['nodeId']);
		$result['device_name'] = $services[0]['device_name'];

		// Reformat $_GET['services'] by service_id key to be reusable into a loop
		$objServices = array();
		foreach($_GET['services'] as $srv)
		{
			$objServices[$srv['id']] = array('order' => $srv['order']);
		}

		foreach($services as $service)
		{
			// Only for services saved on the object
			if(array_key_exists($service['id'], $objServices))
			{
				$common_name = Model_esia::getCommonName($lang, $service['name'], $service['check_plugin_name']);
				$order = $objServices[$service['id']]['order'];
				$message = preg_replace('/\|(.*)$/', '', $service['stdout']);

				$result['services'][$order] = array(
					'id' => $service['id'],
					'order' => $order,
					'name' => $common_name,
					'state' => $service['state'],
					'message' => $message
				);
			}
		}

		echo json_encode($result);
		exit(0);
	}
}
catch(\EsiaDbException | \Plan\PlanException | \Exception $e)
{
	echo json_encode(array('error' => $e->getMessage()));
}

exit(0);

?>
