<?php
/*
** @name: display_rack.php
**
** @description: controller for plan rack display ajax requests
** @author: Quentin George <q.george@esia-sa.com>
** @date: 06/04/2022
*/

$path = $_SERVER['DOCUMENT_ROOT'];
include_once $path . '/common.php';
include_once $path . '/Model_adm.php';
include_once $path . '/plugin/plan/plan.conf.php';
include_once MODEL_PLAN;
include_once LIB_PLAN;
include_once LIB_PLAN_RACK;

// Action
if(isset($_GET['action']))
{
	$action = $_GET['action'];
}
elseif(isset($_POST['action']))
{
	$action = $_POST['action'];
}

try
{
	if($action == 'getNodeServices')
	{
		$servicesList = formatNodeServicesList($dbconnection, $lang, $_GET['nodeId']);

		// Prepare auto-selection for each state tag
		$autoSelectServices = array();
		foreach(RACK_ITEMS_STATES_TO_DISPLAY[$_GET['itemType']] as $stateTag)
		{
			if(RACK_ITEMS_STATES_OPTS[$stateTag]['isMulti'])
			{
				// For a multi select select the value all services
				$autoSelectServices[$stateTag] = -2;
			}
			else
			{
				// For single select tak the first result service id of plugin search by tags
				$plugServices = Model_plan::getServicesByPluginTags($dbconnection, RACK_ITEMS_STATES_OPTS[$stateTag]['plugins_tags'], $_GET['nodeId']);
				$plugServices = array_shift($plugServices);
				$autoSelectServices[$stateTag] = $plugServices['id'];
			}
		}

		echo json_encode(array("services" => $servicesList, "autoSelect" => $autoSelectServices));
		exit(0);
	}
	elseif($action == 'getObjRackItemStates')
	{
		$statesList = array('nodeState' => 'ok', 'services' => array());

		// Get the global node state regardless of configured services
		$nodeState = Model_plan::getObjectState($dbconnection, $user->getId(), array('nodes' => $_POST['nodeId']));
		$statesList['nodeState'] = ALERTS_STATES[$nodeState];

		// Get states for tags icons
		foreach($_POST['services'] as $tag => $tagServicesIds)
		{
			$state = -1; // If no services behind set icon to not configured
			$validServicesIds = array();
			foreach($tagServicesIds as $tagServiceId)
			{
				if($tagServiceId == '-2')
				{
					// If serviceId -2 (All) Get all services which specified plugin tag and get global state for all of them
					$plugServices = Model_plan::getServicesByPluginTags($dbconnection, RACK_ITEMS_STATES_OPTS[$tag]['plugins_tags'], $_POST['nodeId']);
					if(!empty($plugServices))
					{
						$validServicesIds = array_merge($validServicesIds, array_keys($plugServices));
					}
				}
				else
				{
					$validServicesIds[] = $tagServiceId;
				}
			}

			if(!empty($validServicesIds))
			{
				$validServicesIds = array_unique($validServicesIds, SORT_NUMERIC);
				$stateStr = Model_plan::getObjectState($dbconnection, $user->getId(), array('services' => $validServicesIds));
				$state = ALERTS_STATES[$stateStr];
			}

			$statesList['services'][$tag] = $state;
		}

		echo json_encode($statesList);
		exit(0);
	}
	elseif($action == 'updatePlanParams')
	{
		// Instance the plan
		$plan = Model_plan::instancePlan($dbconnection, $_GET['id'], $user->getId(), FALSE, FALSE);

		// Plan common data (name, folder, users share, resources, refresh)
		$plan = setPlanCommonData($dbconnection, $lang, $plan, $_POST);

		// Plan Rack params
		$rackParams = array();
		if(isset($_POST['rack_params']) && !empty($_POST['rack_params']))
		{
			parse_str(urldecode($_POST['rack_params']), $rackParams);
		}
		if(!isset($rackParams['empty_u_hidden']))
		{
			$rackParams['empty_u_hidden'] = "";
		}
		if(isset($rackParams['u_size']) && isset($rackParams['zoom']))
		{
			$plan->setParam(array('rack'), $rackParams);
		}
		else
		{
			throw new \Exception($lang['PLAN_ERROR_RACK'] . ": " . $lang['PLAN_ERROR_RACK_PARAM']);
		}

		// Update the plan in DB
		Model_plan::updatePlan($dbconnection, $user->getId(), $_GET['id'], $plan->getName(), json_encode($plan->getParams()), $plan->getFolderId(), $plan->getAccessUsersList());


		echo json_encode(TRUE);
		exit(0);
	}
	elseif($action == 'updatePlanRackItems')
	{
		// Update or add objects type rack_item
		if(!empty($_POST['rack_objects']))
		{
			$rackObjectsParams = json_decode($_POST['rack_objects'], TRUE);
			foreach($rackObjectsParams as $objParam)
			{
				if(!empty($objParam['info']['id']))
				{
					// Check if rack item params are valid
					if(isValidRackItem($objParam['common']['item_type'], $objParam['common']['item_subtype'], $objParam['common']['u_size']))
					{
						if($objParam['info']['isNew'] == TRUE)
						{
							// New rack items
							$commonParams = array('common' => $objParam['common']);
							Model_plan::addObject($dbconnection, $_GET['id'], $_GET['id'].'-rack_item', 'rack_item', json_encode($commonParams));
						}
						else
						{
							// Update rack items
							Model_plan::updateObjectParamJSON($dbconnection, $objParam['info']['id'], json_encode($objParam['common']), array('common'));
						}
					}
					else
					{
						throw new \Exception($lang['PLAN_ERROR_RACK'] . ": " . $lang['PLAN_ERROR_INVALID_ITEM'] . " ID=" . $objParam['info']['id'] . " (" . $objParam['common']['item_type'] . "-" . $objParam['common']['item_subtype'] . "-" . $objParam['common']['u_size'] . "U)");
					}
				}
				else
				{
					throw new \Exception($lang['PLAN_ERROR_RACK'] . ": " . $lang['PLAN_ERROR_RACK_ITEMS']);
				}
			}
		}

		echo json_encode(TRUE);
		exit(0);
	}
}
catch(\EsiaDbException | \Plan\PlanException | \Exception $e)
{
	echo json_encode(array('error' => $e->getMessage()));
}

exit(0);

?>
