<?php
/*
** @name: folder_plan.php
**
** @description: controller for plan folders
** @author: Quentin George <q.george@esia-sa.com>
** @date: 30/03/2022
*/

// Include main files
$path = $_SERVER['DOCUMENT_ROOT'];
include_once $path . '/common.php';
include_once $path . '/Model_adm.php';
include_once $path . '/plugin/plan/plan.conf.php';
include_once MODEL_PLAN;

$pluginInfos = Model_esia::getWebPlugin($dbconnection, FALSE, 'WEBP_PLAN')[0];
$apache_path = $pluginInfos['apache_path'];
$templatePath = $path . $pluginInfos['apache_path'] . 'templates/';
$foldersURL = $pluginInfos['apache_path'] . 'folder_plan.php';

$isUserPerms = array(
	'view' => ($user->getDroits($dbconnection, "map_view") == 1),
	'admin' => ($user->getDroits($dbconnection, "map_administration") == 1),
	'create_folder' => ($user->getDroits($dbconnection, "map_create_folder") == 1),
	'create_plan' => ($user->getDroits($dbconnection, "map_create_plan") == 1),
);

// Action
if(isset($_GET['action']))
{
	$action = $_GET['action'];
}
elseif(isset($_POST['action']))
{
	$action = $_POST['action'];
}
else
{
	$action = 'show';
}

$errors = array();
if($action == 'show')
{
	// HTML display
	try
	{
		// Check if user can access to webp_plan in view mode at least
		if(!$isUserPerms['view'])
		{
			$smarty->display('no_access.html');
			exit(0);
		}

		// Assign usefull variables
		$smarty->assign('apache_path', $apache_path);
		$smarty->assign('action', $action);

		// Retrieve saved messages from another page in $_SESSION
		if(isset($_SESSION["plan_msg_success"]))
		{
			$smarty->assign('success', $_SESSION["plan_msg_success"]);
			unset($_SESSION["plan_msg_success"]);
		}

		$folderId = NULL;
		$folderName = '';
		$folderParent = NULL;
		$access = '';
		$folders = array();
		if(!empty($_GET['id']) && is_numeric($_GET['id']))
		{
			$folderId = intval($_GET['id']);
			$smarty->assign('folder_id', $folderId);
			$folders = Model_plan::getFoldersTree($dbconnection, $user->getId(), $folderId);
			$folderName = $folders[$folderId]['name'];
			$access = Model_plan::getFolderUserAccess($dbconnection, $folderId, $user->getId());
			$folderParent = $folders[$folderId]['parent'];
			$smarty->assign('folder_parent', $folderParent);
		}
		else
		{
			$smarty->assign('folder_id', 0);
			$folderName = $lang['PLAN_FOLDER_ROOT'];
		}
		$smarty->assign('folder_name', $folderName);

		// NAVBAR
		// Add Root folder
		$navbar->addElement($lang['WEBP_PLAN'], $foldersURL);
		// Add folders
		$navbarElements = array();
		$elemId = $folderId;
		while(isset($folders[$elemId]))
		{
			array_unshift($navbarElements, array('name' => $folders[$elemId]['name'], 'href' => $foldersURL . '?id=' . $elemId));
			$elemId = $folders[$elemId]['parent'];
		}
		foreach($navbarElements as $navElt)
		{
			$navbar->addElement($navElt['name'], $navElt['href']);
		}

		//variables utiles pour l'administration
		if($isUserPerms['admin'])
		{
			include_once LIB_PLAN;

			//actions permises sur le dossier courant
			$folderActionsAuthorized = array(
				'update' => false,
				'delete' => false,
				'create_folder' => false,
				'create_plan' => false,
			);
			if(empty($folderId))
			{
				//autorise la creation sur le dossier racine
				$folderActionsAuthorized['create_folder'] = $isUserPerms['create_folder'];
				$folderActionsAuthorized['create_plan'] = $isUserPerms['create_plan'];
			}
			elseif($access == 'CREATOR' || $access == 'WRITE')
			{
				$folderActionsAuthorized['update'] = true;
				$folderActionsAuthorized['delete'] = true;
				$folderActionsAuthorized['create_folder'] = $isUserPerms['create_folder'];
				$folderActionsAuthorized['create_plan'] = $isUserPerms['create_plan'];

				// Get data for update action
				//récupère la liste des dossiers dont l'utilisateur a les droits write pour le déplacement
				$foldersList = getMoveFoldersList($dbconnection, $user->getId(), $lang['PLAN_FOLDER_ROOT'], $folderParent, $folderId);
				$smarty->assign('list_folders', $foldersList);

				$list_user = Model_adm::searchUser($dbconnection, NULL);
				$smarty->assign('list_user', $list_user);
				$userAccessRight = Model_plan::getFolderAccessUsersList($dbconnection, $folderId);
				$smarty->assign('list_user_read_access', $userAccessRight['READ']);
				$smarty->assign('list_user_write_access', $userAccessRight['WRITE']);

				// Get data for delete actions
				// modifications sur le texte contenu dans les variables de langues
				$langDeleteWarn = sprintf($lang['PLAN_FOLDER_DELETE_WARN'], strtoupper($folderName));
				$smarty->assign('lang_delete_warn', $langDeleteWarn);
			}
			$smarty->assign('authorized_actions', $folderActionsAuthorized);

			if($folderActionsAuthorized['create_plan'])
			{
				// Get data for plan creation
				$planTypes = Model_plan::getPlanTypes($dbconnection);
				foreach($planTypes as $key => $planType)
				{
					//utilisé pour trouver la variable de langue correspondante au texte du type de plan
					$langTypeKey = "PLAN_TYPE_" . strtoupper($planType["name"]);
					if(isset($lang[$langTypeKey]))
					{
						$planTypes[$key]["lang"] = $lang[$langTypeKey];
					}
					else
					{
						$planTypes[$key]["lang"] = $lang["PLAN_TYPE_UNKNOWN"];
					}
				}
				// sort plan types by type name
				usort($planTypes, function($a, $b) {
					return strcmp($a['lang'], $b['lang']);
				});
				$smarty->assign('plan_types', $planTypes);
			}
		}
		// Recupération des éléments a afficher dans le dossier
		$content = array('folders' => array(), 'plans' => array());
		if(empty($folderId))
		{
			$folderId = 0; // 0 for Root folder
		}
		$folders = Model_plan::getFolders($dbconnection, $user->getId(), NULL, $folderId);
		$plans = Model_plan::getPlans($dbconnection, $user->getId(), NULL, $folderId);
		// Folders
		foreach($folders as $fold)
		{
			$content['folders'][$fold['id']] = array(
				'id' => $fold['id'],
				'name' => $fold['name'],
				'url' => $foldersURL . '?id=' . $fold['id']
			);
		}
		// Plans
		foreach($plans as $plan)
		{
			$langTypeKey = "PLAN_TYPE_" . strtoupper($plan['type_name']);
			if(!isset($lang[$langTypeKey]))
			{
				$langTypeKey = "PLAN_TYPE_UNKNOWN";
			}

			$content['plans'][$plan['id']] = array(
				'id' => $plan['id'],
				'name' => $plan['name'],
				'url' => $apache_path . $plan['type_controller'] . '&id=' . $plan['id'],
				'type' => $plan['type_name'],
				'type_lang' => $lang[$langTypeKey]
			);
		}

		$smarty->assign('content', $content);
	}
	catch(\EsiaDbException $e)
	{
		$errors[] = $e->getMessage();
	}
	$smarty->assign('errors', $errors);

	$smarty->display($templatePath . 'folder_plan.html');
}
else
{
	// JSON Response
	try
	{
		//si on veut déplacer un dossier ou plan par drag & drop
		if($action == 'move')
		{
			if(!empty($_POST['type']) && !empty($_POST['id']) && is_numeric($_POST['id']) && !empty($_POST['parent']) && is_numeric($_POST['parent']))
			{
				//on vérifie si l'user a accès en écriture sur l'élément a déplacé et celui de destination
				$parentAccess = Model_plan::getFolderUserAccess($dbconnection, $_POST['parent'], $user->getId());
				if($_POST['type'] == 'folder')
				{
					$itemAccess = Model_plan::getFolderUserAccess($dbconnection, $_POST['id'], $user->getId());
				}
				elseif($_POST['type'] == 'plan')
				{
					$itemAccess = Model_plan::getPlanUserAccess($dbconnection, $_POST['id'], $user->getId());
				}
				else
				{
					throw new \Exception('ERROR: Wrong item type');
				}

				//si il a les accès creator ou write, on peut déplacer
				if(($itemAccess == 'CREATOR' || $itemAccess == 'WRITE') && ($parentAccess == 'CREATOR' || $parentAccess == 'WRITE'))
				{
					if($_POST['type'] == 'folder')
					{
						Model_plan::moveFolder($dbconnection, intval($_POST['id']), intval($_POST['parent']));
					}
					elseif($_POST['type'] == 'plan')
					{
						Model_plan::movePlan($dbconnection, intval($_POST['id']), intval($_POST['parent']));
					}

					echo json_encode(array('success' => strtoupper($_POST['name']) . $lang['PLAN_MOVE_SUCCESS']));
					exit(0);
				}
				else
				{
					// Génére le message d'erreur pour les droits sur l'élément et/ou sur la cible
					$accessError = '';
					if($itemAccess != 'CREATOR' && $itemAccess != 'WRITE')
					{
						$accessError = sprintf($lang['PLAN_MOVE_FAILED_ITEM_RIGHTS'], strtoupper($_POST['name']));
					}
					if($parentAccess != 'CREATOR' && $parentAccess != 'WRITE')
					{
						if(!empty($accessError))
						{
							$accessError .= "<br/>";
						}
						$accessError .= $lang['PLAN_MOVE_FAILED_TARGET_RIGHTS'];
					}
					throw new \Exception($accessError);
				}
			}
			else
			{
				throw new \Exception($lang['PLAN_FOLDER_UPDATE_FAILED']);
			}
		}
		//si on veut créer un répertoire ou un plan
		elseif($action == 'create')
		{
			$parent = null;
			if(!empty($_POST['folder_id']) && is_numeric($_POST['folder_id']))
			{
				$parent = $_POST['folder_id'];
			}
			if(!empty($_POST['plan_name']) && !empty($_POST['plan_type']) && is_numeric($_POST['plan_type']))
			{
				if($_POST['plan_type'] == -1 && $isUserPerms['create_folder'])
				{
					//création d'un répertoire
					$folderId = Model_plan::addFolder($dbconnection, $_POST['plan_name'], $user->getId(), $parent);

					$newFold = array(
						'id' => $folderId ,
						'name' => $_POST['plan_name'],
						'type' => 'folder',
						'url' => $foldersURL . '?id=' . $folderId,
						'type_lang' => $lang["PLAN_FOLDER"]
					);

					echo json_encode(array(
						'success' => strtoupper($_POST['plan_name']) . ": " . $lang['PLAN_FOLDER_CREATE_SUCCESS'],
						'new'=> $newFold
					));
					exit(0);
				}
				elseif($isUserPerms['create_plan'])
				{
					//création d'un plan
					$planId = Model_plan::addPlan($dbconnection, $_POST['plan_name'], $_POST['plan_type'], $user->getId(), $parent);
					$planType = Model_plan::getPlanTypes($dbconnection, $_POST['plan_type'])[0];

					// Génère les infos pour créer le nouveau plan en JS
					$langTypeKey = "PLAN_TYPE_" . strtoupper($planType["name"]);
					if(!isset($lang[$langTypeKey]))
					{
						$langTypeKey = "PLAN_TYPE_UNKNOWN";
					}

					$newPlan = array(
						'id' => $planId,
						'name' => $_POST['plan_name'],
						'url' => $apache_path . $planType['controller'] . '&id=' . $planId,
						'type' => $planType["name"],
						'type_lang' => $lang[$langTypeKey]
					);

					echo json_encode(array(
						'success' => strtoupper($_POST['plan_name']) . ": " . $lang['PLAN_ADD_SUCCES'],
						'new'=> $newPlan
					));
					exit(0);
				}
				else
				{
					throw new \Exception($lang['PLAN_ADD_FAILED'] . '<br/>' . $lang['PLAN_ADD_FAILED_USER_RIGHTS']);
				}
			}
			else
			{
				throw new \Exception($lang['PLAN_ADD_FAILED'] . '<br/>' . $lang['PLAN_ADD_FAILED_MISSING_VAR']);
			}
		}
		//si on veut mettre à jour un répertoire
		elseif($action == 'update')
		{
			$formData = array();
			parse_str(urldecode($_POST['form_data']), $formData);
			if(!empty($formData['folder_name']) && isset($formData['folder_parent']) && !empty($formData['folder_id']) && is_numeric($formData['folder_id']))
			{
				// Check if read and write plan user access have been updated and send it to Model_plan::updateFolder. If no update user access will not be updated
				$folderNewAccess = array();
				if(serialize($formData['read_userplan']) != serialize($formData['old_read_userplan']))
				{
					$folderNewAccess['READ'] = $formData['read_userplan'];
				}
				if(serialize($formData['write_userplan']) != serialize($formData['old_write_userplan']))
				{
					$folderNewAccess['WRITE'] = $formData['write_userplan'];
				}

				Model_plan::updateFolder($dbconnection, $user->getId(), $formData['folder_id'], $formData['folder_name'], $formData['folder_parent'], $folderNewAccess);

				// Success message
				$_SESSION["plan_msg_success"] = strtoupper($formData['folder_name']) . ": " . $lang['PLAN_FOLDER_UPDATE_SUCCESS'];

				echo json_encode(array('url'=> $foldersURL . "?id=" . $formData['folder_id']));
				exit(0);
			}
			else
			{
				throw new \Exception($lang['PLAN_FOLDER_UPDATE_FAILED']);
			}
		}
		//si on veut supprimer un répertoire
		elseif($action == 'delete')
		{
			if(!empty($_POST['content_action']) && !empty($_POST['folder_id']) && is_numeric($_POST['folder_id']))
			{
				$isDelContent = FALSE;
				if($_POST['content_action'] == 'delete')
				{
					$isDelContent = TRUE;
				}
				Model_plan::deleteFolder($dbconnection, $_POST['folder_id'], $isDelContent);

				// Success message
				$_SESSION["plan_msg_success"] = strtoupper($_POST['folder_name']) . ": " . $lang['PLAN_FOLDER_DELETE_SUCCESS'];

				// Back to folder parent (null = racine)
				$url = $foldersURL;
				if(!empty($_POST['folder_parent']) && is_numeric($_POST['folder_parent']))
				{
					$url .= "?id=" . $_POST['folder_parent'];
				}

				echo json_encode(array('url'=> $url));
				exit(0);
			}
			else
			{
				throw new \Exception($lang['PLAN_FOLDER_DELETE_FAILED']);
			}
		}
	}
	catch(\EsiaDbException | \Exception $e)
	{
		echo json_encode(array('error' => $e->getMessage()));
	}
}
?>
