<?php
/*
** @name: common_admin_plan.inc.php
**
** @description: code included in all plan types controllers for admin mode (admin_)
** @author: Quentin George <q.george@esia-sa.com>
** @date: 22/12/2021
*/

$controlType = 'admin';
include_once $path . '/plugin/plan/inc/common_plan.inc.php';

// Retrieve saved messages from another page in $_SESSION
if(isset($_SESSION["plan_msg_success"]))
{
	$smarty->assign('success', $_SESSION["plan_msg_success"]);
	unset($_SESSION["plan_msg_success"]);
}

try
{
	if($action == 'admin')
	{
		// Get the plan with its objects but all states to ok
		$plan = Model_plan::instancePlan($dbconnection, $planId, $user->getId(), TRUE, FALSE, DEFAULT_ADM_STATES);

		// Check if page can be displayed
		if(!$isUserPerms['admin'] || $plan->getCurrentUserAccess() != 'WRITE')
		{
			$smarty->display('no_access.html');
			exit(0);
		}

		// Get users rights
		$usersListByAccess = Model_plan::getPlanAccessUsersList($dbconnection, $planId);
		$plan->setAccessUsersList($usersListByAccess['READ'], $usersListByAccess['WRITE'], $usersListByAccess['CREATOR']);

		// Assign plan & objects
		$smarty->assign('plan', $plan->toArray());
		$smarty->assign('objects', $plan->getListObjects(TRUE, $path));

		setPlanNavbar($dbconnection, $navbar, $lang['WEBP_PLAN'], $user->getId(), $foldersURL, $plan, $apache_path);

		// Get folder list wich user has write privilege to move plan into folder
		$moveFoldersList = getMoveFoldersList($dbconnection, $user->getId(), $lang['PLAN_FOLDER_ROOT'], $plan->getFolderId());
		$smarty->assign('moveFoldersList', $moveFoldersList);
		// Get all users list
		$usersList = Model_adm::searchUser($dbconnection, NULL);
		$smarty->assign('usersList', $usersList);
	}
}
catch(\Plan\PlanException $e)
{
	// If there is an issue on plan instanciation the plan does not exist
	$smarty->display('no_access.html');
	exit(0);
}
catch(\EsiaDbException | \Exception $e)
{
	$errors[] = $e->getMessage();
}

?>
