// Set plan img objects by types
function admSetPlanImgObjects(objectsByTypes)
{
	if(objectsByTypes.container)
	{
		setAllObjectsContainerOnPlan(objectsByTypes.container);
	}
	if(objectsByTypes.link)
	{
		setAllObjectsLinkOnPlan(objectsByTypes.link);
	}
	if(objectsByTypes.service)
	{
		objectsByTypes.service.forEach((obj, i) => {
			admSetObjectServiceOnPlan(obj);
		});
	}
	if(objectsByTypes.text)
	{
		objectsByTypes.text.forEach((obj, i) => {
			admSetObjectTextOnPlan(obj);
		});
	}
}

// Set plan img objects by types
function backgroundSwitch(elements)
{
	elements.on("change", function()
	{
		var value = $(this).val(),
			newSelectedId = $('input[name="bgImage"]:checked').attr("id");

		// Switch selected class
		$(".bg-choice-min.selected").removeClass("selected");
		$('label[for="' + newSelectedId + '"]').addClass("selected");

		// Switch background image on plan
		if(value != '')
		{
			$(".plan-container").css("background-image", "url(" + value + ")");
		}
		else
		{
			$(".plan-container").css("background-image", "");
		}
	});
}

// Remove background image
function deleteBgImage(delBtn)
{
	var container = delBtn.closest('.bg-choice-container');

	$.ajax({
		type: "POST",
		url: glob_apache_path + "display_img.php?action=delBgImg",
		data: {id: glob_plan.id, dir: container.attr('data-dir'), img: container.attr('data-img')},
		dataType: "json"
	}).done(function(data) {
		if (data.error) { // Error Message
			displayUserMessage(data.error, true);
		}
		else if (data.success) {
			displayUserMessage(data.success, false);
			container.remove();
		}
	});
}

// Set event for plan background image
function bgImageEvents()
{
	// Click on button will open browser file prompt by triggering click on input type=file
	$('#uploadImage').on('click', function(e)
	{
		$('#isPlanLinked').val(0);
		$('#bgFile').trigger('click');
	});
	$('#uploadImageLinked').on('click', function(e)
	{
		$('#isPlanLinked').val(1);
		$('#bgFile').trigger('click');
	});
	// On file change trigger submit form because we cannot retreive file data from an input only from the form submit
	$("#bgFile").change(function()
	{
		$('#uploadBgImg').submit();
	});
	// Send the file
	$("#uploadBgImg").submit(function(e)
	{
		e.preventDefault();

		$.ajax({
			type: "POST",
			url: $('#uploadBgImg').attr('action'),
			data: new FormData(this),
			processData: false,
			contentType: false,
			dataType: "json"
		}).done(function(data) {
			if (data.error) { // Error Message
				displayUserMessage(data.error, true);
			}
			else if (data) {
				// Add new image in DOM
				var bgChoice = $('#bgImgChoice .cloneBgChoice').clone(),
					randomId = 'bgChoice' + Date.now();

				bgChoice.removeClass('cloneBgChoice').removeClass('hidden').attr('data-dir', data.bgImgDir).attr('data-img', data.bgImage);
				bgChoice.find('.bg-choice-min').attr('for', randomId);
				bgChoice.find('.bg-choice').attr('id', randomId).val(data.bgImgDir + data.bgImage);
				bgChoice.find('.bg-choice-img').attr('src', data.bgImgDir + 'miniature/min_' + data.bgImage);

				if(data.isPlanLinked == true) {
					$('.planBgImgLinked').append(bgChoice);
				} else {
					$('.planBgImgCommon').append(bgChoice);
				}

				// Add selected & set events listeners
				var newChoice = $('#bgImgChoice').find('#' + randomId);
				$('input[name="bgImage"]:checked').prop('checked', false);
				newChoice.prop('checked', true);

				backgroundSwitch(newChoice);
				newChoice.trigger('change');

				bgChoice.find('.delBtn').on('click', function() {
					deleteBgImage($(this));
				});
			}
		});
	});
	// Delete image
	$('.bg-choice-min .delBtn').on('click', function()
	{
		deleteBgImage($(this));
	});
}

// Drag new plan object container type
function dragNewContainerObject()
{
	$('#adm_icons .imgMove').draggable({
		helper: "clone",
		containment: $('.plan-container')[0],
		cursor: "move",
		stop: function(e, ui) {
			const objParams = {
				class: 'planObj',
				name: '',
				position: {
					x: ui.offset.left - $('.canvasWidget').offset().left + $('.canvasWidget').scrollLeft(),
					y: ui.offset.top - $('.canvasWidget').offset().top + $('.canvasWidget').scrollTop(), // Cursor position from the top of the page - plan position from the top of the page + scroll position inside the plan (if scroll enable)
				},
				icon: {
					id: $(this).attr('data-icon-id'),
					path: $('img', this).attr('src')
				}
			};

			window.planImgApp.addContainerNodes([ objParams ]);
			window.planImgApp.repaint();
		}
	});
}

// Get common obj params from html data attributes
function getObjContainerCommonParams(objContainer) {
	return {
		id: objContainer.attr("data-id"),
		name: objContainer.attr("data-name"),
		icon_id: parseInt(objContainer.attr("data-icon-id")),
		params: {
			position: {
				z: parseInt(objContainer.css("z-index")),
				x: parseFloat(objContainer.closest(".node").css("left")),
				y: parseFloat(objContainer.closest(".node").css("top"))
			},
			size: {
				width: parseInt(objContainer.width()),
				height: parseInt(objContainer.height())
			}
		}
	};
}

// Save plan img parameters modifications. Return a promise
function updatePlanImgParams()
{
	savePlanSize();

	const formData = getFormValues('edit_infos');

	return asyncSaveData({
		url: glob_apache_path + "display_img.php?id=" + glob_plan.id,
		data: {
			action: "updatePlanParams",
			...formData,
		},
	});
}

// Save plan img objects container & links modifications. Return a promise
function updatePlanContainersAndLinks()
{
	let contObjects = [],
		linkObjects = [];

	// Get info about each obj container
	document.querySelectorAll('.objContainer').forEach(elt => {
		let obj = getObjContainerCommonParams($(elt));

		contObjects.push(obj);
	});

	// Get info about each obj link. Function created in reactJS app (plan/js/esia/plan_img.app.min.js)
	linkObjects = window.planImgApp.getServiceLinks();

	return asyncSaveData({
		url: glob_apache_path + "display_img.php?id=" + glob_plan.id,
		data: {
			action: "updatePlanContainersAndLinks",
			container_objects: JSON.stringify(contObjects),
			link_objects: JSON.stringify(linkObjects),
		},
	});
}

// Save plan img deleted objects. Return a promise
function deletePlanImgObjects()
{
	// { nodes: [ids], links: [ids] } => obj.nodes become containers, obj.links become links
	const { nodes: containers, links } = window.planImgApp.getDeleteItems();
	// Merge ids
	let deleteObjects = [...glob_plan.deleteObjects, ...containers, ...links];

	return asyncDeleteObjects(glob_apache_path, glob_plan.id, deleteObjects);
}

// Save plan image parameters, objects and plan modifications
async function updatePlanImg()
{
	$("#edit_infos").submit(async e => {
		e.preventDefault();

		$("body").addClass('loadCursor');

		try {
			// Check if the user has the rights to modify this plan (Will throw an exception if not)
			await isPlanWrittable();
			// If the user can, do all modification asynchronously
			await Promise.all([
				updatePlanImgParams(),
				updatePlanContainersAndLinks(),
				updatePlanTextObjects(),
				updatePlanServiceObjects(),
				deletePlanImgObjects(),
			]);

			// If no error set success message and reload the page
			sessionStorage.setItem('plan_msg_success', glob_lang.PLAN_UPD_OK);
			location.reload();
		} catch (error) {
			displayUserMessage(error.message ? error.message : error, true);
		} finally {
			$("body").removeClass('loadCursor');
		}
	});
}

// Open obj container configuration dialog
function configObjContainer(objId) {
	var url = glob_apache_path + "dialog_img.php?action=updateObjContainer&id=" + $("#currentPlan").data("id"),
		requestData = { objId: objId },
		dialogParams = { width: 1000, height: 800 };

	if(!$.isNumeric(objId)) {
		// Object not already saved in DB but somme params can be updated by the user
		requestData.objParamsCommon = JSON.stringify(getObjContainerCommonParams($('#' + objId + 'object')));
	}

	openDialogByAJAX(url, requestData, glob_lang.PLAN_OBJECT_CONF, 'dialog-info', dialogParams);

	// Send event to register unsaved changes
	$('#dialog-info').on('validation', function() {
		$('.plan-window').trigger('plan-changes');
	});
}

// Create object actions menu
function editContainerObj(container) {
	var objId = container.attr("data-id"),
		actions = $("<div>", {"class": "plan-obj-actions"}),
		closeButton = $("<button>", {"class": "obj-action close-action", "type": "button", "title": glob_lang.PLAN_CLOSE}),
		modifyButton = $("<button>", {"class": "obj-action modify-action", "type": "button", "title": glob_lang.PLAN_MODIFY}),
		delButton = $("<button>", {"class": "obj-action delete-action", "type": "button", "title": glob_lang.PLAN_DELETE}),
		resizeButton = $("<button>", {"class": "obj-action resize-action", "type": "button", "title": glob_lang.PLAN_RESIZE}),
		originalSize = { width: container.width(), height: container.height() };

	actions.append(modifyButton).append(resizeButton).append(delButton).append(closeButton);
	initObjectEditMenu(container, actions);

	// Close
	closeButton.on("click", function() {
		closeObjectEditMenu(container);
	});

	// Modify parameters
	modifyButton.on("click", function() {
		configObjContainer(objId);
		closeObjectEditMenu(container);
	});

	// Delete object
	delButton.on("click", function() {
		var acceptDelButton = $("<button>", {"class": "obj-action accept-action", "type": "button", "title": glob_lang.PLAN_DELETE_VALIDATE}),
			refuseDelButton = $("<button>", {"class": "obj-action refuse-action", "type": "button", "title": glob_lang.PLAN_DELETE_ABORT});

		acceptRefuseActionMenu(actions, acceptDelButton, refuseDelButton);

		// Validate object deletion
		acceptDelButton.on("click", function() {
			window.planImgApp.deleteNodes([ objId ]);
			window.planImgApp.repaint();
		});
	});

	const fctOnResizeRefused = () => {
		// Reset sizes
		container.css({
			"width": originalSize.width + 'px',
			"height": originalSize.height + 'px'
		});

		window.planImgApp.setNodeIsLocked(objId, false);
	};

	const fctOnResizeAccepted = (objId) => {
		const params = {
			id: objId,
			options: {
				size: {
					width: container.width(),
					height: container.height()
				}
			}
		};

		window.planImgApp.setNodeIsLocked(objId, false);
		window.planImgApp.updateContainerNodes([params]);
	};

	// Resize object
	resizeButton.on("click", function() {
		setResizeContainerObj($('#plan' + glob_plan.id), container, objId);
		actionResizeObject(objId, actions, container, fctOnResizeRefused, fctOnResizeAccepted);
	});
}

// Set container object resizable
function setResizeContainerObj(plan, container, objId) {
	window.planImgApp.setNodeIsLocked(objId, true);

	container.resizable({
		disabled: false,
		containment: plan,
		aspectRatio: true,
		minWidth: 20,
		minHeight: 20
	});
}

// Open object actions box
function setObjContainerEventsHandler(container) {
	// Open object edit menu on simple click
	onEltClickOrDrag(container, function(isDrag) {
		if(!isDrag && container.find(".plan-obj-infos").length == 0 && container.find(".newObjInfo").length == 0) {
			editContainerObj(container);
		}
	});

	// For new object check if name input position is inside the plan
	if(container.find(".newObjInfo").length > 0) {
		safePosNewObjInfos(container);
	}
}

// Open obj link configuration dialog
function configObjLink(objId) {
	var url = glob_apache_path + "dialog_img.php?action=updateLink&id=" + $("#currentPlan").data("id"),
		requestData = { link: JSON.stringify(window.planImgApp.getServiceLink(objId)) },
		dialogParams = { width: 1000, height: 640 };

	openDialogByAJAX(url, requestData, glob_lang.PLAN_OBJ_LINK_CONF, 'dialog-info', dialogParams);

	// Send event to register unsaved changes
	$('#dialog-info').on('validation', function() {
		$('.plan-window').trigger('plan-changes');
	});
}

// Create link actions menu
function editLinkObj(container) {
	var objId = container.attr("data-id"),
		actions = $("<div>", {"class": "plan-obj-actions"}),
		closeButton = $("<button>", {"class": "obj-action close-action", "type": "button", "title": glob_lang.PLAN_CLOSE}),
		modifyButton = $("<button>", {"class": "obj-action modify-action", "type": "button", "title": glob_lang.PLAN_MODIFY}),
		delButton = $("<button>", {"class": "obj-action delete-action", "type": "button", "title": glob_lang.PLAN_DELETE});

	actions.append(modifyButton).append(delButton).append(closeButton);
	initObjectEditMenu(container, actions);

	// Close
	closeButton.on("click", function() {
		closeObjectEditMenu(container);
	});

	// Modify parameters
	modifyButton.on("click", function() {
		configObjLink(objId);
		closeObjectEditMenu(container);
	});

	// Delete object
	delButton.on("click", function() {
		var acceptDelButton = $("<button>", {"class": "obj-action accept-action", "type": "button", "title": glob_lang.PLAN_DELETE_VALIDATE}),
			refuseDelButton = $("<button>", {"class": "obj-action refuse-action", "type": "button", "title": glob_lang.PLAN_DELETE_ABORT});

		acceptRefuseActionMenu(actions, acceptDelButton, refuseDelButton);

		// Validate object deletion
		acceptDelButton.on("click", function() {
			container.remove();
			window.planImgApp.deleteLinks([ objId ]);
			window.planImgApp.repaint();
		});
	});
}

// Open link actions box
function setLinkEventsHandler() {
	document.addEventListener('openLinkMenu', ({ detail }) => {
		var container = $("<div>", { class: "objLinkInfos planObj", 'data-id': detail.id });

		container.css({
			top: detail.center.y + 'px',
			left: detail.center.x + 'px'
		});

		$('#plan' + glob_plan.id).append(container);

		editLinkObj(container);

		container.on("editClose", function() {
			$(this).remove();
		});
	});
}

// Prevent dragging container object infos (name on new object creation)
function safePosNewObjInfos(container) {
	var menu = container.find(".newObjInfo"),
		plan = $(".plan-container"),
		maxPos = {
			top: plan.offset().top - menu.position().top,
			left: plan.offset().left - menu.position().left
		};

	planSafePosElt(menu, plan, maxPos, container.offset(), container.position());
	commonSafePosObjMenu(container, menu, plan, maxPos);
}

// Open icons configuration dialog
function openIconAdm() {
	$('.adm-icons-link').on("click", function(e) {
		e.preventDefault();

		const dialogParams = {
			close: function() {
				// Refresh admin mode adm icons menu to applpy new updates
				$.ajax({
					type: 'GET',
					url: glob_apache_path + "display_img.php?action=displayAdmIconsMenu",
					data: { apache_path: glob_apache_path, templatePath: glob_template_path, planId: glob_plan.id },
					dataType: 'html'
				})
				.done(function(data) {
					let err = '';
					// Retrieve error sended in JSON format
					try {
						err = JSON.parse(data).error;
					} catch(e) {}

					if(err == '') {
						// No error refresh menu icons
						let menu = $('.plan-container #adm_icons');

						menu.draggable('destroy');
						menu.empty();
						menu.append($(data).html());
					} else {
						// Display error message
						displayUserMessage(glob_lang.PLAN_ERROR_REFRESH_ADM_ICONS, true);
					}
				});
			},
		};

		openDialogByAJAX($(this).attr('href'), {}, glob_lang.PLAN_ADMIN_ICON, 'dialog-info', dialogParams, {}, false);
	});
}

// On plan changes set var to true to know if we have unsaved changes
function handlePlanImgHasUnsavedChanges() {
	// glob_plan_has_changes is reset to false on page load. A save action reload the page

	// Global plan informations changes
	$("#edit_infos input, #edit_infos select").change(function() {
		glob_plan_has_changes = true;
	});
	// On objects drag
	onEltClickOrDrag($('.plan-window'), function(isDrag, e) {
		if(isDrag) {
			if($(e.target).hasClass('objContainer')) {
				// On objContainer drag
				glob_plan_has_changes = true;
			} else if(e.target?.className?.baseVal?.includes('point')) {
				// On link point drag
				glob_plan_has_changes = true;
			}
		}
	});
	// On object accept action (add name, resize, delete)
	$('.plan-window').on('click', function(e) {
		if($(e.target).hasClass('accept-action')) {
			glob_plan_has_changes = true;
		}
	});
	// Custom plan changes event to trigger (save object modifications, ...)
	$('.plan-window').on('plan-changes', function() {
		glob_plan_has_changes = true;
	});
	// Create new link
	$('.plan-window').on('mouseup', function(e) {
		if($(e.target).hasClass('objContainer-port')) {
			glob_plan_has_changes = true;
		}
	});

	// On plan admin leave check if there is unsaved changes
	$("#planLeave").on("click", function(e) {
		// If there is unsaved changes
		if(glob_plan_has_changes) {
			e.preventDefault();

			var url = glob_apache_path + "dialog_rack.php?action=unsavedPlanDialog&id=" + $("#currentPlan").data("id"),
				dialogParams = {
					title: glob_lang.PLAN_CAUTION,
					resizable: false,
					draggable: false,
					width: 300,
					maxHeight: false,
					position: {
						my: "right top",
						at: "right bottom",
						of: $(".plan-main-title"),
					}
				};

			openDialogByAJAX(url, {}, '', 'dialog-info', dialogParams);

			$('#dialog-info').one("validation", function() {
				window.location = $("#planLeave").attr('href');
			});
		}
	});
}

// Initialize event listeners appenning on plan img app
function initPlanImgAppAdmEvents()
{
	$('#plan' + glob_plan.id + " .plan-img-app")[0].addEventListener('nodeUpdated', ({ detail: node }) => {
		var container = $('#' + node.options.dbid + 'object');

		addTooltipTitle(container);
		setObjContainerEventsHandler(container);
	});

	setLinkEventsHandler();
}

// Set some admin mode behaviour
function initPlanImgAdm()
{
	setPlanSize();
	bgImageEvents();
	backgroundSwitch($('input[name="bgImage"]'));
	updatePlanImg();
	setPlanResizable();
	handlePlanImgHasUnsavedChanges();
	addTooltipTitle($('.plan-tool'));
}
