// Set an object on the rack and set modification options related to admin mode
function admSetObjectRackItem(obj, deviceName='') {
	setObjectRackItemCommon(obj, "planObj", deviceName);

	var container = $("#" + obj.id + "object");

	dragObjRackItem(container);
	initObjRackItemMenu(container);
}

// Get the u position on rack when an object is droppend on it
function getObjRackUPosOnDrop(positionTop) {
	var minDraggPosTop = positionTop - 15, // Gap value must be at least half of rack-u height to match even when snap is bug on scroll
		maxDraggPosTop = positionTop + 15,
		uPos = 0;

	// For each rack-u search the one which is the closest of postion top
	$(".rack-u").each(function() {
		if ($(this).position().top > minDraggPosTop && $(this).position().top < maxDraggPosTop) {
			uPos = $(this).attr('data-u');
			return false; // Break the each loop
		}
	});

	return uPos;
}

// Calc draggable snap tolerance value (px) depending on the zoom value
function setSnapTolerance() {
	return (glob_plan.params.rack.zoom / 5); // 20px for zoom 100%
}

// Drag an object rack_item
function dragObjRackItem(object) {
	object.draggable({
		containment: ".rack-objects",
		snap: ".snapContainer",
		snapMode: "inner",
		snapTolerance: setSnapTolerance(),
		revert: "invalid",
		cursor: "move",
		appendTo: ".rack-objects",
		helper: "original",
		start: function(e, ui) {
			// On start we have to fix a height on the helper or it will be 0 because it's fixed only on the menuItems
			var rackUHeight = $(".rack-u-1").outerHeight(), // Take height on the first rack U
				uSize = $(this).attr("data-u-size");

			$(ui.helper).css({
				height: (rackUHeight * uSize) + "px",
			});
		},
		stop: function(e, ui) {
			setRackEmtyUSeparators();
		},
	});
}

// Get common obj params from html data attributes
function getObjRackItemCommonParams(objId) {
	var obj = $('#' + objId + 'object');

	return {
		item_type: obj.attr('data-item-type'),
		item_subtype: obj.attr('data-item-subtype'),
		u_size: obj.attr('data-u-size'),
		u_position: obj.attr('data-u'),
	};
}

// Open obj rack item configuration dialog
function configObjRackItem(objId) {
	var url = glob_apache_path + "dialog_rack.php?action=updateObjRackItem&id=" + $("#currentPlan").data("id"),
		requestData = { objId: objId },
		dialogParams = { height: 800 };

	if(!$.isNumeric(objId)) {
		// Object not already saved in DB but somme params can be updated by the user
		requestData.objParamsCommon = getObjRackItemCommonParams(objId);
	}

	openDialogByAJAX(url, requestData, glob_lang.PLAN_RACK_ITEM_CONF, 'dialog-info', dialogParams);
}

// Open object acions box
function openObjRackItemMenu(container) {
	var planId = $("#currentPlan").data("id"),
		objId = container.data("id"),
		actions = $("<div>", {"class": "plan-obj-actions"}),
		closeButton = $("<button>", {"class": "obj-action close-action", "type": "button", "title": glob_lang.PLAN_CLOSE}),
		modifyButton = $("<button>", {"class": "obj-action modify-action", "type": "button", "title": glob_lang.PLAN_MODIFY}),
		delButton = $("<button>", {"class": "obj-action delete-action", "type": "button", "title": glob_lang.PLAN_TO_TRASH});

	if (!const_rack_items_types_not_node.includes(container.data("item-type"))) {
		actions.append(modifyButton);
	}
	actions.append(delButton).append(closeButton);
	initObjectEditMenu(container, actions);

	// Close
	closeButton.on("click", function() {
		closeObjectEditMenu(container);
	});

	// Modify parameters
	modifyButton.on("click", function() {
		if ($.isNumeric(planId)) {
			configObjRackItem(objId);
			closeObjectEditMenu(container);
		}
	});

	// Delete object
	delButton.on("click", function() {
		closeObjectEditMenu(container);
		moveRackItemToTrash(objId);
	});
}

// Initialize rack item obj menu
function initObjRackItemMenu(container) {
	container.on("click", function() {
		if (container.find(".plan-obj-infos").length == 0) {
			openObjRackItemMenu(container);
		}
	});
}

// Set obj id on rack-u data-link-id and remove obj id from previous rack-u. Return objects id if other objects was setted on thoses new rack-u
function setObjRackULinkedId(objId, objSize, uPos) {
	var busyObjIds = []; // Severall obj can be replaced by a bigger one
	// Reset previous obj U link
	$(".rack-u[data-link-id='" + objId + "']").attr('data-link-id', 0);

	while (objSize > 0) {
		var rackU = $('.rack-u-' + uPos),
			rackULinkedId = rackU.attr('data-link-id');

		// Check if another object was linked on this U
		if (rackULinkedId != 0) {
			var isInArray = false,
				i = 0;

			for (; i < busyObjIds.length; i++) {
				if (rackULinkedId == busyObjIds[i]) {
					isInArray = true;
					break;
				}
			}

			if (!isInArray) {
				busyObjIds.push(rackULinkedId);
			}
		}

		// Set new obj U link
		rackU.attr('data-link-id', objId);

		objSize--;
		uPos--;
	}

	return busyObjIds;
}

// Move an rack object to the trash
function moveRackItemToTrash(objId) {
	var replObj = $("#" + objId + "object"),
		menuItem = $("<li>", {"class": "menuItem"});

	replObj.attr('data-u', 0);
	replObj.css({
		"top": 0,
		"bottom": 0,
		"left": 0,
		"right": 0,
	});

	replObj.removeClass("planObj");
	replObj.addClass("trashObj");
	menuItem.append(replObj);
	$(".trash-content").append(menuItem);

	handleIsTrashDisabled();

	// Remove old rack u link. The replaced obj can be bigger than the new object so empty places must be reset to 0
	$(".rack-u[data-link-id='" + objId + "']").attr('data-link-id', 0);
	setRackEmtyUSeparators();

	replObj.draggable("option", "helper", "clone");
	replObj.draggable("option", "containment", "#plan" + glob_plan.id);
}

// Drop an object rack_item on the rack
function dropObjInRack() {
	$(".rack-container .rack-objects").droppable({
		tolerance: "intersect",
		accept: ".objRackItem, .rackItemsSelected",
		drop: function(e, ui) {
			var obj = $(ui.draggable);

			// Prevent to use this function in case of multi-select because it use it's own function multiDragPlanRackItems
			if (obj.hasClass("planObj-selected")) {
				return;
			}

			var objId = $(ui.draggable).attr('data-id'),
				objSize = $(ui.draggable).attr('data-u-size'),
				uPos = getObjRackUPosOnDrop(ui.position.top),
				replacedObjIds = [];

			if (obj.hasClass("menuObj")) {
				// New obj dragged from menu
				obj = obj.clone();
				obj.empty();
				setObjRackItemSVG(obj);

				obj.removeClass("menuObj");
				obj.addClass("planObj newObj");
				objId = getRandomObjId();
				obj.attr('id', objId + "object");
				obj.attr('data-id', objId);
				$('.rack-objects').append(obj);

				dragObjRackItem(obj);
				obj.tooltipster({theme: 'tooltipster-borderless'});
				initObjRackItemMenu(obj);
			} else if (obj.hasClass("trashObj")) {
				// Object dragged from the trash
				var menuItem = obj.parent(".menuItem");

				obj.removeClass("trashObj");
				obj.addClass("planObj");
				$('.rack-objects').append(obj);
				menuItem.remove();
				handleIsTrashDisabled();

				obj.draggable("option", "helper", "original");
				obj.draggable("option", "containment", ".rack-objects");
			}

			// Check if postion as been correctly retrieved
			if (uPos > 0) {
				obj.attr('data-u', uPos);
				// Set position and size to be sure we match the U
				setRackItemOnU(obj, uPos, objSize);

				// Set obj id on rack u data
				replacedObjIds = setObjRackULinkedId(objId, objSize, uPos);
			} else {
				// If no U position found move the object to trash
				replacedObjIds[0] = objId;
			}

			// If rack u is not empty
			if (replacedObjIds.length > 0) {
				for (var i = 0; i < replacedObjIds.length; i++) {
					moveRackItemToTrash(replacedObjIds[i]);
				}
			}
		},
	});
}

// Drag an object rack_item from the menu
function dragMenuRackItem() {
	$('.menuItem .objRackItem').each(function() {
		dragObjRackItem($(this));
		$(this).draggable("option", "helper", "clone");
		$(this).draggable("option", "containment", "#plan" + glob_plan.id);
	});
}

// Show/hide trash content
function showHideTrashContent() {
	$(".navigation .toggleSubMenu-single .subMenuTitle").click(function(e) {
		var toggleSubMenu = $(this).parent();

		if (!toggleSubMenu.hasClass("isDisabled")) {
			$(".subMenu-single", toggleSubMenu).slideToggle("normal", function() {
				toggleSubMenu.toggleClass("open");
			});
		}
	});
}

// Be sure that values are between min max limits
function validRackLimits(uSize=null, zoom=null) {
	var validValues = {
		u_size: uSize,
		zoom: zoom,
	};

	if (uSize != null) {
		if (uSize < const_rack_limits.uSizeMin) {
			validValues.u_size = const_rack_limits.uSizeMin;
		} else if (uSize > const_rack_limits.uSizeMax) {
			validValues.u_size = const_rack_limits.uSizeMax;
		}
	}

	if (zoom != null) {
		if (zoom < const_rack_limits.zoomMin) {
			validValues.zoom = const_rack_limits.zoomMin;
		} else if (zoom > const_rack_limits.zoomMax) {
			validValues.zoom = const_rack_limits.zoomMax;
		}
	}

	return validValues;
}

// Add uSize number of rack U or additional U on an existing rack
function addRackU(uSize, uStartId=1) {
	var uParts = $(".rack-u-parts"),
		rackU = $(".rack-u-1", uParts).clone().removeClass("rack-u-1").attr("data-link-id", 0);

	while (uStartId <= uSize) {
		var newU = rackU.clone().addClass("rack-u-" + uStartId).attr("data-u", uStartId);

		uParts.prepend(newU);

		uStartId++;
	}
}

// Remove uSize number of rack U on an existing rack
function removeRackU(newUSize, currentUSize) {
	while (newUSize < currentUSize) {
		var rackU = $(".rack-u-" + currentUSize),
			linkedId = rackU?.attr('data-link-id') || "0";

		// If an object is on the rack U put it to the trash
		if (linkedId != 0) {
			moveRackItemToTrash(linkedId);
		}
		rackU.remove();

		currentUSize--;
	}
}

// Modify rack parameters
function handleRackParameters() {
	// Change rack size
	$("#inputUSize").on("change", function() {
		validValues = validRackLimits(parseInt($(this).val()), null);
		glob_plan.params.rack.u_size = parseInt(glob_plan.params.rack.u_size); // Safety because the first value retreived from the controller may not be an integer
		// Update field in case of the value was not between limits
		$(this).val(validValues.u_size);
		// Apply new U elements on rack if u size higher or smaller thant before
		if (validValues.u_size > glob_plan.params.rack.u_size) {
			addRackU(validValues.u_size, glob_plan.params.rack.u_size + 1);
		} else if (validValues.u_size < glob_plan.params.rack.u_size) {
			removeRackU(validValues.u_size, glob_plan.params.rack.u_size);
		}
		// Save in global rack var
		glob_plan.params.rack.u_size = validValues.u_size;
		// Trigger contentupdated to update rack items objects
		$(".rack-content").trigger("contentupdated");
		setRackEmtyUSeparators();
	});
	// Change rack zoom
	$("#inputZoom").on("change", function() {
		validValues = validRackLimits(null, parseInt($(this).val()));
		// Update field in case of the value was not between limits
		$(this).val(validValues.zoom);
		// Apply new zoom
		$(".rack-zoom").css("width", validValues.zoom + "%");
		// Save in global rack var
		glob_plan.params.rack.zoom = validValues.zoom;
		// Change rack-u font size
		if (validValues.zoom < 80) {
			$(".rack-u-parts").css("font-size", "10px");
		} else {
			$(".rack-u-parts").css("font-size", "12px");
		}
		// Update snapTolerance
		$(".objRackItem").draggable( "option", "snapTolerance", setSnapTolerance());
		$(".rackItemsSelected").draggable( "option", "snapTolerance", setSnapTolerance());

		// Trigger contentupdated to update rack items objects
		$(".rack-content").trigger("contentupdated");
	});
	// Show/Hide empty U
	$("#isEmptyUHidden").on("change", function() {
		if ($(this).is(":checked") == true) {
			$(".rack-content").addClass("empty-u-hidden");
		} else {
			$(".rack-content").removeClass("empty-u-hidden");
		}
		setRackEmtyUSeparators();
		// Trigger contentupdated to update rack items objects
		$(".rack-content").trigger("contentupdated");
	});
	// On update modify objects
	$(".rack-content").on("contentupdated", function() {
		// Update all rack items objects position and size
		$('.planObj.objRackItem').each(function() {
			setRackItemOnU($(this), $(this).attr("data-u"), $(this).attr("data-u-size"));
		});
	});
}

// Wait for the user to valid trash content deletion before save the plan
function validEmptyTrashOnSavePromise() {
	return new Promise(function(resolve) {
		var deleteObj = [];

		if($(".trash-count").attr("data-nbr") > 0) {
			// On save empty trash confirm dialog
			let url = glob_apache_path + "dialog_rack.php?action=emptyTrashOnSave&id=" + $("#currentPlan").data("id"),
				dialogParams = {
					title: glob_lang.PLAN_CAUTION,
					resizable: false,
					draggable: false,
					width: 300,
					maxHeight: false,
					position: {
						my: "right top",
						at: "right bottom",
						of: $(".plan-main-title"),
					}
				};

			openDialogByAJAX(url, {}, '', 'dialog-info', dialogParams);

			// We listen a custom event send on the dialog because we cannot create a listener on it's content now because it's setted by an ajax promise and does not exist yet
			$('#dialog-info').one("validation", function() {
				// User choose to valid trash empty
				deleteObj = getTrashObjWithDbId();
				$(this).dialog("close");

				resolve(deleteObj);
			});
		} else {
			// Trash is empty do nothing
			resolve(deleteObj);
		}
	});
}

// Save plan rack parameters modifications. Return a promise
function updatePlanRackParams()
{
	savePlanSize();
	$("#rack_params").val($("#edit_rack").serialize());

	const formData = getFormValues('edit_infos');

	return asyncSaveData({
		url: glob_apache_path + "display_rack.php?id=" + glob_plan.id,
		data: {
			action: "updatePlanParams",
			...formData,
		},
	});
}

// Save plan rack objects rack_item modifications. Return a promise
async function updatePlanRackItemsObjects()
{
	let rackObjects = [];

	// Get info about each obj rack item on the rack only
	document.querySelectorAll('.objRackItem.planObj').forEach(elt => {
		const obj = {
			info: {
				id: $(elt).attr("data-id"),
				isNew: ($(elt).hasClass("newObj")) ? true : false,
			},
			common: {
				item_type: $(elt).attr("data-item-type"),
				item_subtype: $(elt).attr("data-item-subtype"),
				u_size: $(elt).attr("data-u-size"),
				u_position: $(elt).attr("data-u"),
			},
		};

		rackObjects.push(obj);
	});

	return asyncSaveData({
		url: glob_apache_path + "display_rack.php?id=" + glob_plan.id,
		data: {
			action: "updatePlanRackItems",
			rack_objects: JSON.stringify(rackObjects),
		},
	});
}

// Save plan rack deleted objects. Return a promise
function deletePlanRackObjects(deleteObjects)
{
	deleteObjects.push(...glob_plan.deleteObjects);

	return asyncDeleteObjects(glob_apache_path, glob_plan.id, deleteObjects);
}

// Save plan rack parameters, objects and plan modifications
async function updatePlanRack()
{
	$("#edit_infos").submit(async e => {
		e.preventDefault();

		$("body").addClass('loadCursor');

		try {
			// Check if the user has the rights to modify this plan (Will throw an exception if not)
			await isPlanWrittable();
			// Wait for the user to valid trash content deletion before save the plan
			const deleteObjects = await validEmptyTrashOnSavePromise();
			// If the user can, do all modification asynchronously
			await Promise.all([
				updatePlanRackParams(),
				updatePlanRackItemsObjects(),
				updatePlanTextObjects(),
				updatePlanServiceObjects(),
				deletePlanRackObjects(deleteObjects),
			]);

			// If no error set success message and reload the page
			sessionStorage.setItem('plan_msg_success', glob_lang.PLAN_UPD_OK);
			location.reload();
		} catch (error) {
			displayUserMessage(error.message ? error.message : error, true);
		} finally {
			$("body").removeClass('loadCursor');
		}
	});
}

// On plan changes set var to true to know if we have unsaved changes
function handlePlanHasUnsavedChanges() {
	// glob_plan_has_changes is reset to false on page load. A save action reload the page

	// Global plan informations changes
	$("#edit_infos input, #edit_infos select").change(function() {
		glob_plan_has_changes = true;
	});
	// Rack parameters changes
	$("#edit_rack input").change(function() {
		glob_plan_has_changes = true;
	});
	// Drop obj on the rack
	$(".rack-container .rack-objects").on("drop", function() {
		glob_plan_has_changes = true;
	});
	// Custom rack changes event to trigger
	$(".rack-container").on("rack-changes", function() {
		glob_plan_has_changes = true;
	});

	// On plan admin leave check if there is unsaved changes
	$("#planLeave").on("click", function(e) {
		// If something in the trash there is changes
		if ($(".trash-count").attr("data-nbr") > 0) {
			glob_plan_has_changes = true;
		}

		// If there is unsaved changes
		if (glob_plan_has_changes) {
			e.preventDefault();

			var url = glob_apache_path + "dialog_rack.php?action=unsavedPlanDialog&id=" + $("#currentPlan").data("id"),
				dialogParams = {
					title: glob_lang.PLAN_CAUTION,
					resizable: false,
					draggable: false,
					width: 300,
					maxHeight: false,
					position: {
						my: "right top",
						at: "right bottom",
						of: $(".plan-main-title"),
					}
				};

			openDialogByAJAX(url, {}, '', 'dialog-info', dialogParams);

			$('#dialog-info').one("validation", function() {
				window.location = $("#planLeave").attr('href');
			});
		}
	});
}

// Open empty trash dialog
function openEmptyTrashDialog() {
	$(".trash-action-empty").on("click", function(e) {
		// Prevent event to bubble up to its parent which cause toggle trash menu
		e.stopPropagation();
		e.preventDefault();

		var url = glob_apache_path + "dialog_rack.php?action=emptyTrash&id=" + $("#currentPlan").data("id"),
			dialogParams = {
				title: glob_lang.PLAN_TRASH,
				resizable: false,
				draggable: false,
				width: 300,
				maxHeight: false,
				position: {
					my: "right top",
					at: "right bottom",
					of: $(".trash .subMenuTitle"),
				}
			};

		openDialogByAJAX(url, {}, '', 'dialog-info', dialogParams);
	});
}

// Set some admin mode behaviour
function initPlanRackAdm() {
	setPlanSize();
	dropObjInRack();
	initPlanIconsMenu();
	dragMenuRackItem();
	showHideTrashContent();
	handleRackParameters();
	updatePlanRack();
	handlePlanHasUnsavedChanges();
	openEmptyTrashDialog();
	setRackEmtyUSeparators();
	setPlanResizable();
	addTooltipTitle($('.plan-tool'));
}

// Drag multiples plan Rack Items objects together
function multiDragPlanRackItems() {
	// Create a container for each rack-items selected and drag this container instead with all rack-items inside
	$("#plan" + glob_plan.id).on("newMultiSelect", function() {
		var newSelected = $(".rack-objects > .objRackItem.planObj-selected"),
			selected = $(".objRackItem.planObj-selected"),
			uLimits = {max: 1, min: const_rack_limits.uSizeMax}, // Revert min & max U positions values to be sure first object erase it
			selectionHeight = $(".rack-u-1").outerHeight(), // Initialize value with the height of one U
			selectedContainerTop = 0;

		// Retrieve first and last U of selection in order to calc container height & position
		selected.each(function(index, element) {
			var eltULimits = {max: parseInt($(element).attr("data-u")), min: parseInt($(element).attr("data-u")) - (parseInt($(element).attr("data-u-size")) - 1)};

			if (eltULimits.max > uLimits.max) {
				uLimits.max = eltULimits.max;
			}

			if (eltULimits.min < uLimits.min) {
				uLimits.min = eltULimits.min;
			}

			$(element).draggable("disable");
		});

		// Create container & add items inside
		if (newSelected.length > 0) {
			if ($(".rack-objects > .rackItemsSelected").length == 0) {
				// First selection
				var selectedContainer = $("<div>", {"class": "rackItemsSelected planObj-selected common-multi-drag-disabled"});

				$(".rack-objects").append(selectedContainer);
			}
			// Add elements to selection
			$(".rack-objects .rackItemsSelected").append(newSelected);
		}

		// Set container height & position
		selectionHeight = selectionHeight * ((uLimits.max - uLimits.min) + 1);
		selectedContainerTop = $(".rack-u-" + uLimits.max).position().top;

		$(".rackItemsSelected").css({
			height: selectionHeight + "px",
			top: selectedContainerTop + "px",
		});

		// Re-calc elements positions inside container. Because they are no longer relatively positioned from rack but from this container wich is also positioned inside the rack
		newSelected.each(function(index, element) {
			$(element).css({
				top: $(element).position().top - selectedContainerTop + "px",
			});
		});
		// Update position on previously selected elements wich are already inside selected container because it's size change when adding new elements to selection
		selected.each(function(index, element) {
			setRackItemOnU($(element), $(element).attr('data-u'), $(element).attr('data-u-size'), selectedContainerTop);
		});

		// Set selection container dragg
		$(".rackItemsSelected").draggable({
			containment: ".rack-objects",
			snap: ".snapContainer",
			snapMode: "inner",
			snapTolerance: setSnapTolerance(),
			revert: "invalid",
			cursor: "move",
			appendTo: ".rack-objects",
			create: function() {
				// Re-enable multi dragging to add this select container to the multi dragg elements list
				enableMultiDragOnSelectedObj();
			},
			stop: function(e, ui) {
				// Set new positions on rack
				var selectedContainerTop = $(".rackItemsSelected").position().top;

				$(".objRackItem.planObj-selected").each(function(index, element) {
					var obj = $(element),
						objId = obj.attr('data-id'),
						objSize = obj.attr('data-u-size'),
						uPos = getObjRackUPosOnDrop(obj.position().top + selectedContainerTop),
						replacedObjIds = [];

					// Check if postion as been correctly retrieved
					if (uPos > 0) {
						obj.attr('data-u', uPos);

						// Set obj id on rack u data
						replacedObjIds = setObjRackULinkedId(objId, objSize, uPos);

						for (var i = 0; i < replacedObjIds.length; i++) {
							// Do not send to trash objects which are currently dragged together because they have been moved after checking if they where on the same u
							if (!$("#" + replacedObjIds[i] + "object").hasClass("planObj-selected")) {
								moveRackItemToTrash(replacedObjIds[i]);
							}
						}
					} else {
						// If no U position found move the object to trash & remove it from selected
						obj.removeClass("planObj-selected");
						moveRackItemToTrash(objId);
					}

					// Update position on U with selected container as relative parent
					setRackItemOnU($(element), uPos, objSize, selectedContainerTop);
				});

				setRackEmtyUSeparators();

				// Send event to register unsaved changes
				$(".rack-container").trigger("rack-changes");
			},
		});

		// Self handle dragg positions not handled by common drag.multiDrag event
		$(".rackItemsSelected").on("multidrag:drag", function(e, ui, draggedOffset) {
			var parent = $(this).parent(),
				bottomLimit = parent.height() - $(this).outerHeight(),
				newPosTop = ui.originalPosition.top + draggedOffset.top + parent.scrollTop();

			// Check values to keep all selected elements in parent limits
			if (newPosTop < 0) {
				newPosTop = 0;
			} else if (newPosTop > bottomLimit) {
				newPosTop = bottomLimit;
			}

			$(this).css("top", newPosTop + "px");
		});

		// Set multi drag events to correctly work when draggin another object type with rack items in the selection
		$(".rackItemsSelected").on("multidrag:stop", function(e, ui) {
			var stop = $(this).draggable("option", "stop");

			if (stop !== null) {
				// Apply is used to pass element context as this
				stop.apply($(this), [e, ui]);
			}
		});
	});

	// Reset some values & destroy selection container when multi selection end
	$("#plan" + glob_plan.id).on("removeMultiSelect", function() {
		var multiSelected = $(".rackItemsSelected .planObj");

		$(".rack-objects").append(multiSelected);
		multiSelected.each(function(index, element) {
			// Reset objects real U position after multi select container removall because now relative parent is rack-objects container
			setRackItemOnU($(element), $(element).attr('data-u'), $(element).attr('data-u-size'));

			$(element).draggable("enable");
		});
		$(".rackItemsSelected").remove();
	});
}
