// Set object rackItem states icons
function setObjRackItemStates(objId, config) {
	if (config?.device_id !== undefined && config?.services !== undefined) {
		$.ajax({
			type: "POST",
			url: glob_apache_path + "display_rack.php",
			data: {action: "getObjRackItemStates", nodeId: config.device_id, services: config.services},
			dataType: "json"
		})
		.done(function(data) {
			if (data.error) {
				// Error Message
				displayUserMessage(data.error, true);
			} else {
				var objectImg = $("#" + objId + "object > svg");

				$(".elt-state-global", objectImg).css("fill", PLAN_SERVICE_STATE_COLORS[data.nodeState] || PLAN_SERVICE_STATE_COLORS[3]);

				$.each(data.services, function(tag, state) {
					// Selector ".elt-state-" + tag for use tag with class like elt-state-ping. Selector .elt-" + tag + " use.rack-elt-state" for group of use tags like g of class elt-disks or fans
					let elt = $(".elt-state-" + tag + ", .elt-" + tag + " use.rack-elt-state", objectImg);
					if(state == -1) {
						// No services behind remove fill color to set icon to not configured
						elt.css("fill", "");
					}
					else {
						elt.css("fill", PLAN_SERVICE_STATE_COLORS[state] || PLAN_SERVICE_STATE_COLORS[3]);
					}
				});
			}
		});
	}
}

// Open node state table
function openRackItemNodeStateTable(objId, objParams) {
	if (objParams?.config?.device_id !== undefined) {
		var langSubType = glob_lang.PLAN_RACK_ITEM,
			objOnRackPos = "U" + objParams?.common?.u_position;

		// Get rack item type name by lang
		if (objParams?.common?.item_type !== undefined) {
			langSubType = "PLAN_ELT_" + objParams?.common?.item_type.toUpperCase();
		}

		// Calc rack item U postion limits
		if (objParams?.common?.u_size > 1) {
			objOnRackPos += " > U" + (parseInt(objParams?.common?.u_position) - parseInt(objParams?.common?.u_size) + 1);
		}

		getObjectsDetails(objId, glob_lang[langSubType] + " - " + objOnRackPos);
	}
}

// Set an object on the rack
function setObjectRackItem(obj, deviceName='') {
	setObjectRackItemCommon(obj, "planObjView", deviceName);

	var container = $("#" + obj.id + "object");

	setObjRackItemStates(obj.id, obj.params?.config);

	// Click actions
	container.on("click", function(e) {
		if (e.ctrlKey) {
			// Open node state table on ctrl + click
			openRackItemNodeStateTable(obj.id, obj.params);
		} else {
			// Avoid re-zoom if the document click is on this
			e.stopPropagation();

			// Re-init some stuff if other obj previously zoomed
			$(".zoomedObj-details").remove();
			$(document).off(".unzoomObjEvent");

			// Create show details button
			if ((obj.params?.config?.device_id !== undefined) && !$(this).hasClass("zoomedObj")) {
				var detailsButton = $("<button>", {
						"class": "zoomedObj-details",
						"type": "button",
						"title": glob_lang.PLAN_SHOW_DETAILS,
					});

				container.append(detailsButton);
				detailsButton.on("click", function() {
					// Open node state table on ctrl + click
					openRackItemNodeStateTable(obj.id, obj.params);
				});

				addTooltipTitle(detailsButton);
			}

			// Zoom on normal click
			$(".zoomedObj").not(this).removeClass("zoomedObj");
			$(this).toggleClass("zoomedObj");

			// Close on click any where or keyboard press
			$(document).on("click.unzoomObjEvent keydown.unzoomObjEvent", function(e) {
				// Escape press or simple click (not ctrl click)
				if (e.keyCode == undefined || e.keyCode == 27) {
					$(".zoomedObj").removeClass("zoomedObj");
					$(".zoomedObj-details").remove();
					// Remove event handler when condition match
					$(document).off(".unzoomObjEvent");
				}
			});
		}
	});
}

// Set rack view mode behaviour
function initPlanRackView() {
	setPlanSize();
	setRackEmtyUSeparators();
}
