// Init plan container object configuration
function initContainerObjConf(obj, planId, admLists) {
	updateElemInObject(admLists);
	onDelObjEltLinked();
	updObjContainerEvent(obj, planId);
}

// Add new service selector in obj conf
function addElementServiceInObj(node_options, number) {
	var label = glob_lang.DASHBOARD_RRD_SELECT_SERVICE,
		inputElt = $("<div>", {class: 'inputElem', html:
			'<label for="element-' + number + '">' + label + '</label>' +
			'<div class="selectSrvBox">' +
				'<div class="wrapperSelectSrvNode">' +
					'<select id="element-' + number + '" class="selectSrvNode" name="selectSrvNode' + number + '" data-placeholder="' + glob_lang.DASHBOARD_RRD_SELECT_NODE + '">' +
						'<option value=""></option>' +
						node_options +
					'</select>' +
				'</div>' +
				'<div class="wrapperSelectService">' +
					'<select class="selectService" name="service[' + number + ']" data-placeholder="' + glob_lang.DASHBOARD_RRD_SELECT_SERVICE + '" disabled="disabled">' +
						'<option value=""></option>' +
					'</select>' +
				'</div>' +
			'</div>'
		});

	$('#elems').append(inputElt);
	$(".selectSrvNode", inputElt).chosen({no_results_text: glob_lang.SEARCH_NONE, search_contains: true});
	$(".selectService", inputElt).chosen({no_results_text: glob_lang.SEARCH_NONE, search_contains: true});

	$(".selectSrvNode", inputElt).change(function() {
		var nodeId = $(this).val().split('-')[1];

		$(".selectService", inputElt).empty().prop('disabled', false);
		updSelectService(nodeId, $(".selectService", inputElt));
		$(".selectService", inputElt).trigger("chosen:updated");
	});
}

// Handle add node, group or service
function updateElemInObject(admLists)
{
	var node_options = "";
	var group_options = "";
	var number = 0;

	$.each( admLists.nodes, function( key, value ) {
	  node_options += '<option value="node-'+value.id+'">'+value.name+'</option>';
	});
	$.each( admLists.groups, function( key, value ) {
		group_options += '<option value="groupNode-'+value.id+'">'+value.name+'</option>';
	});

	$("input[name='type']").click(function() {
		var options = '',
			label = '',
			placeHolder = '';

		switch($(this).attr('id')) {
			case 'groupNode':
				options = group_options;
				label = glob_lang.DASHBOARD_SELECT_ONE_GROUP_NODE;
				placeHolder = glob_lang.DASHBOARD_SELECT_ONE_GROUP_NODE;
				break;
			case 'node':
				options = node_options;
				label = glob_lang.DASHBOARD_RRD_SELECT_NODE;
				placeHolder = glob_lang.DASHBOARD_RRD_SELECT_NODE;
				break;
			case 'service':
				addElementServiceInObj(node_options, number);
				break;
		}

		// Service element is add by addElementServiceInObj()
		if($(this).attr('id') != 'service') {
			var html = '<div class="inputElem">' +
							'<label for="element-' + number + '">' + label + '</label>' +
							'<select id="element-' + number + '" class="chosen" name="element[' + number + '][id]" data-placeholder="' + placeHolder + '">' +
								'<option value=""></option>' +
								options +
							'</select>' +
						'</div>';

			$('#elems').append(html);
			$(".chosen").chosen({search_contains:true});
		}

		number++;
	});
}

// Remove nodes, groups, linked on btn delete click
function onDelObjEltLinked() {
	$('.delElemObject').on("click", function()
	{
		var elts = {}
			inputVal = $('#delete_elements').val(),
			type = $(this).attr('data-type');

		// Get previous saved elts
		if(inputVal != "") {
			elts = JSON.parse(inputVal);
		}
		if(elts[type] == undefined) {
			elts[type] = [];
		}

		// Add new elt
		elts[type].push($(this).attr('data-id'));
		$('#delete_elements').val(JSON.stringify(elts));
		// Deleted html
		$(this).closest('.elemObject').remove();
	});
}

// Handle saving new object configuration parameters
function updObjContainerEvent(obj, planId) {
	$("#formUpdObjContainer").submit(function(e) {
		e.preventDefault();

		var formData = getFormValues('formUpdObjContainer'),
			objParams = {};

		if(!$.isNumeric(obj.id)) {
			objParams = JSON.stringify(obj.params);
		}

		$.ajax({
			type: "POST",
			url: $(this).attr("action"),
			data: { isJSONResp: true, ...formData, obj_params_old: objParams },
			dataType: "json"
		})
		.done(function(data) {
			if (data.error) {
				// Error Message
				displayUserMessage(data.error, true, 'dialog-messages');
			} else {
				const objParams = {
					id: data.old_id,
					options: {
						id: data.id,
						name: data.name,
						icon: {
							id: data.icon_id,
							path: data.icon_file.path
						}
					}
				};

				window.planImgApp.updateContainerNodes([ objParams ]);
				window.planImgApp.repaint();

				// Close dialog window
				$("#dialog-info").dialog("close");
			}
		});
	});
}

// Init plan object link configuration
function initLinkConf(obj, planId, nodesList) {
	$("#formUpdLink .selectNode").chosen({no_results_text: glob_lang.SEARCH_NONE, search_contains: true, allow_single_deselect: true});
	$("#formUpdLink .selectService").chosen({no_results_text: glob_lang.SEARCH_NONE, search_contains: true, allow_single_deselect: true});

	selectServicesEvents();
	reverseLinkEvents();
	updObjLinkEvent(obj, planId);
}

// Handle node & service selection change
function selectServicesEvents()
{
	$("#formUpdLink .selectNode").change(function() {
		var nodeId = $(this).val();

		if(nodeId != "") {
			$("#formUpdLink .selectService").empty().prop('disabled', false);
			updSelectService(nodeId, $("#formUpdLink .selectService"));
		} else {
			$("#formUpdLink .selectService").empty().prop('disabled', true);
		}
		$("#formUpdLink .selectService").trigger("chosen:updated");
	});
}

// Handle link reverse action
function reverseLinkEvents()
{
	$("#formUpdLink .link-reverse").on("click", function() {
		var src = {
				id: $('#source').val(),
				name: $('.link-src').text()
			},
			tar = {
				id: $('#target').val(),
				name: $('.link-tar').text()
			};

		$('#source').val(tar.id);
		$('#target').val(src.id);
		$('.link-src').text(tar.name);
		$('.link-tar').text(src.name);
	});
}

// Handle saving new object configuration parameters
function updObjLinkEvent(obj, planId) {
	$("#formUpdLink").submit(function(e) {
		e.preventDefault();

		var formData = getFormValues('formUpdLink'),
			objParams = JSON.stringify(obj);

		$.ajax({
			type: "POST",
			url: $(this).attr("action"),
			data: { isJSONResp: true, ...formData, obj_params_old: objParams },
			dataType: "json"
		})
		.done(function(data) {
			if (data.error) {
				// Error Message
				displayUserMessage(data.error, true, 'dialog-messages');
			} else {
				const objParams = {
					id: data.id,
					options: {
						service: data?.service || '',
						source: data.source,
						target: data.target,
						points: data?.points || [],
						isDisableArrows: data.isDisableArrows || false,
						isDisableLabels: data.isDisableLabels || false
					}
				};

				window.planImgApp.updateServiceLinks([ objParams ]);
				window.planImgApp.repaint();

				// Close dialog window
				$("#dialog-info").dialog("close");
			}
		});
	});
}

// Before sending new logo check if all required fields are filled & send form as AJAX request
function addAdmIcon(formHtmlId) {
	// Set green border on input files when an image is selected
	$('input[type="file"]', '#' + formHtmlId).change(function() {
		$(this).parent().find('label').css("border-color", "#83BC00");
	});

	$('#' + formHtmlId).submit(function(e) {
		let isError = false;

		// Prevent form submit to check inputs first
		e.preventDefault();

		// Check all file inputs and prevent submit form if some are missing
		$('input[type="file"]', '#' + formHtmlId).each(function() {
			if($(this).get(0).files.length === 0) {
				$(this).parent().find('label').css("border-color", "#DA002E");
				isError = true;
			} else {
				$(this).parent().find('label').css("border-color", "#83BC00");
			}
		});

		// If no errors on inputs send the form as AJAX request
		if(!isError) {
			$.ajax({
				type: "POST",
				url: $(this).attr("action") + '&isJSONResp=true',
				data: new FormData(this),
				processData: false,
				contentType: false,
				dataType: "json"
			})
			.done(function(data) {
				if(data.id) {
					const iconsGroupList = $('.iconMenuList > ul');
					const groupContainerSelector = 'li[data-group-id="' + data.group + '"]';
					let groupContainer = $(groupContainerSelector, iconsGroupList);
					const iconItem = $("<li>", { class: 'imgMove', title: data.name, 'data-icon-id': data.id, html:
						'<img src="' + data.path + data.name + '_ok.' + data.type + '">'
					});

					if(groupContainer.length == 0) {
						// Icon group not listed yet
						groupContainer = $("<li>", { class: 'toggleSubMenu', 'data-group-id': data.group, 'data-group-name': data.group_name, html:
							'<span class="subMenuTitle">' + data.group_name + '</span>' +
							'<ul class="subMenu">' +
							'</ul>'
						});

						iconsGroupList.append(groupContainer);
					}

					// Selector must be rewrite from iconsGroupList to reactualize children
					$(groupContainerSelector + ' .subMenu', iconsGroupList).append(iconItem);

					// Clean form
					$('#' + formHtmlId)[0].reset();
					$('#' + formHtmlId + ' .chooseIcons label').css("border-color", "");

					// Re init click menu icon events
					toggleIconsMenu('.iconMenuList', groupContainerSelector);
					handleIconsSelectActions();

					// Success message
					displayUserMessage(glob_lang.PLAN_UPL_ICON_SUCCESS, false, 'dialog-messages');
				}
				else if (data.error) {
					// Error Message
					displayUserMessage(data.error, true, 'dialog-messages');
				}
			})
			.fail(function(jqXHR, errMsg) {
				displayUserMessage("AJAX Request failed: " + errMsg, true, 'dialog-messages');
			});
		}
	});
}

// Open admIcon dialog on add group page
function openAdmIconAddGroup() {
	$('.addIconGroup').on("click", function() {
		$.ajax({
			type: "GET",
			url: glob_apache_path + "dialog_img.php?action=addIconGroup&id=" + glob_plan.id,
			data: {},
			dataType: "html"
		})
		.done(function(data) {
			if(data) {
				$('#dialog-info').empty();
				$('#dialog-info').append(data);
			}
		});
	});
}

// Open admIcon dialog on update group page
function openAdmIconUpdGroup() {
	$('.updIconGroup').on("click", function() {
		$.ajax({
			type: "GET",
			url: glob_apache_path + "dialog_img.php?action=updIconGroup&id=" + glob_plan.id + "&grpId=" + $(this).attr('data-group-id'),
			data: {},
			dataType: "html"
		})
		.done(function(data) {
			if(data) {
				$('#dialog-info').empty();
				$('#dialog-info').append(data);
			}
		});
	});
}

// Add a new icon group
function addIconGroup() {
	$("#formIconGroup").submit(function(e) {
		e.preventDefault();

		var formData = getFormValues('formIconGroup');

		$.ajax({
			type: "POST",
			url: $(this).attr("action"),
			data: { ...formData },
			dataType: "html"
		})
		.done(function(data) {
			if(data) {
				$('#dialog-info').empty();
				$('#dialog-info').append(data);
			}
		});
	});
}

// Update an icon group
function updIconGroup() {
	$("#formIconGroup").submit(function(e) {
		e.preventDefault();

		var formData = getFormValues('formIconGroup');

		$.ajax({
			type: "POST",
			url: $(this).attr("action"),
			data: { isJSONResp: true, ...formData },
			dataType: "json"
		})
		.done(function(data) {
			if (data.error) {
				// Error Message
				displayUserMessage(data.error, true, 'dialog-messages');
			} else if (data.success) {
				displayUserMessage(data.success, false, 'dialog-messages');
			}
		});
	});
}

// On admIcon dialog go back to main window
function goBackToAdmIcon(savedMessage={ msg: '', isError: true }) {
	$.ajax({
		type: "GET",
		url: glob_apache_path + "dialog_img.php?action=iconAdm&id=" + glob_plan.id,
		data: {},
		dataType: "html"
	})
	.done(function(data) {
		if(data) {
			$('#dialog-info').empty();
			$('#dialog-info').append(data);
			// Display message after page refresh
			if(savedMessage.msg != '') {
				displayUserMessage(savedMessage.msg, savedMessage.isError, 'dialog-messages');
			}
		}
	});
}

// On admIcon dialog go back to main window on cancel btn click
function cancelGroupAdm(savedMessage={}) {
	$('.btnBack').on("click", function() {
		goBackToAdmIcon();
	});
}

// Open admIcon dialog on delete group confirmation page
function openAdmIconDelGroup() {
	$('.delIconGroup').on("click", function() {
		$.ajax({
			type: "GET",
			url: glob_apache_path + "dialog_img.php?action=delIconGroup&id=" + glob_plan.id + "&grpId=" + $(this).attr('data-group-id'),
			data: {},
			dataType: "html"
		})
		.done(function(data) {
			if(data) {
				$('#dialog-info').empty();
				$('#dialog-info').append(data);
			}
		});
	});
}

// Delete an icon group
function delIconGroup() {
	$("#formDelIconGroup").submit(function(e) {
		e.preventDefault();

		var formData = getFormValues('formDelIconGroup');

		$.ajax({
			type: "POST",
			url: $(this).attr("action"),
			data: { isJSONResp: true, ...formData },
			dataType: "json"
		})
		.done(function(data) {
			if (data.error) {
				// Error Message
				displayUserMessage(data.error, true, 'dialog-messages');
			} else if (data.msg) {
				goBackToAdmIcon({ msg: data.msg, isError: data.isError });
			}
		});
	});
}

// Open admIcon dialog on update icon page
function openAdmIconUpdIcon(iconId) {
	if(iconId != '') {
		$.ajax({
			type: "GET",
			url: glob_apache_path + "dialog_img.php?action=updIcon&id=" + glob_plan.id + "&icnId=" + iconId,
			data: {},
			dataType: "html"
		})
		.done(function(data) {
			if(data) {
				$('#dialog-info').empty();
				$('#dialog-info').append(data);
			}
		});
	}
}

// Open admIcon dialog on delete icon confirm page
function openAdmIconDelIcon(iconId) {
	if(iconId != '') {
		$.ajax({
			type: "GET",
			url: glob_apache_path + "dialog_img.php?action=delIcon&id=" + glob_plan.id + "&icnId=" + iconId,
			data: {},
			dataType: "html"
		})
		.done(function(data) {
			if(data) {
				$('#dialog-info').empty();
				$('#dialog-info').append(data);
			}
		});
	}
}

// Set styles for selected icon and activate modify & delete buttons
function handleIconsSelectActions() {
	const iconAdmList = $('.iconAdm .menuIcons');
	const menuIconsActions = $('.menuIconsActions', iconAdmList);

	// Reset for reapplying events
	$('.imgMove.active', iconAdmList).removeClass('active');

	$('.imgMove', iconAdmList)
	.off('.admIconSelect') // Reset event in case it was already set on somme elements and need to be reaply on all of them
	.on('click.admIconSelect', function() {
		// Handle active class for styles
		$('.imgMove.active', iconAdmList).not(this).removeClass('active');
		$(this).toggleClass('active');
		// Display the modify & delete buttons if something is selected
		if($(this).hasClass('active')) {
			menuIconsActions.show();
		} else {
			menuIconsActions.hide();
		}
		// Disable the modify & delete buttons for the ESIA group which contain default icons and cannot be modify or delete
		if($(this).closest('.toggleSubMenu').attr('data-group-name') == 'ESIA') {
			$('button', menuIconsActions).prop('disabled', true).attr('title', glob_lang.PLAN_ESIA_ICON_INFO);
		} else if($('button', menuIconsActions).prop('disabled') == true) {
			$('button', menuIconsActions).prop('disabled', false);
			$('.btnIconActionUpd', menuIconsActions).attr('title', glob_lang.MODIF);
			$('.btnIconActionDel', menuIconsActions).attr('title', glob_lang.DELETE);
		}
	});

	// Reset icon selection on menu group open close action
	$('.toggleSubMenu .subMenuTitle', iconAdmList)
	.off('.admIconSelect')
	.on('click.admIconSelect', function() {
		$('.imgMove.active', iconAdmList).removeClass('active');
		menuIconsActions.hide();
	});

	// Actions on modify btn click
	$('.btnIconActionUpd', menuIconsActions)
	.off('.admIconSelect')
	.on('click.admIconSelect', function() {
		const iconId = $('.imgMove.active', iconAdmList).attr('data-icon-id') || '';
		openAdmIconUpdIcon(iconId);
	});

	// Actions on delete btn click
	$('.btnIconActionDel', menuIconsActions)
	.off('.admIconSelect')
	.on('click.admIconSelect', function() {
		const iconId = $('.imgMove.active', iconAdmList).attr('data-icon-id') || '';
		openAdmIconDelIcon(iconId);
	});
}

// Update an icon
function updIcon() {
	$("#formIconUpd").submit(function(e) {
		e.preventDefault();

		var formData = getFormValues('formIconUpd');

		$.ajax({
			type: "POST",
			url: $(this).attr("action"),
			data: { isJSONResp: true, ...formData },
			dataType: "json"
		})
		.done(function(data) {
			if (data.error) {
				// Error Message
				displayUserMessage(data.error, true, 'dialog-messages');
			} else if (data.success) {
				displayUserMessage(data.success, false, 'dialog-messages');
			}
		});
	});
}

// Delete an icon group
function delIcon() {
	$("#formDelIcon").submit(function(e) {
		e.preventDefault();

		var formData = getFormValues('formDelIcon');

		$.ajax({
			type: "POST",
			url: $(this).attr("action"),
			data: { isJSONResp: true, ...formData },
			dataType: "json"
		})
		.done(function(data) {
			if (data.error) {
				// Error Message
				displayUserMessage(data.error, true, 'dialog-messages');
			} else if (data.msg) {
				goBackToAdmIcon({ msg: data.msg, isError: data.isError });
			}
		});
	});
}
