var PLAN_VALID_SERVICE_STATE = [0, 1, 2, 3, 4];

var PLAN_SERVICE_STATE_COLORS = [
	"#83BC00",
	"#FFCB1F",
	"#DA002E",
	"#575759",
	"#3263C5",
];

// Plan menu items toggler
function toggleIconsMenu(menuSelector, openedAtInitSelector='.toggleSubMenu:first-child') {
	const menu = $(menuSelector);

	// On initialization
	// Hide all sub menus
	$('.subMenu', menu).hide();
	// Reset for reapplying events
	$('.toggleSubMenu.open', menu).removeClass("open");
	// Open first sub menu
	$(openedAtInitSelector, menu).addClass('open').find('.subMenu').show();

	// Toggle on click
	$('.toggleSubMenu .subMenuTitle', menu)
	.off('.toggleIconMenu') // Reset event in case we have added somme elements in the menu and need to reapply events
	.on("click.toggleIconMenu", function() {
		const toggleSubMenu = $(this).closest('.toggleSubMenu');
		const isOpened = toggleSubMenu.hasClass('open'); // Save opened state before closing every sub menu

		// Close all opened sub menu
		$('.toggleSubMenu.open .subMenu', menu).slideUp("normal", function() {
			$(this).closest('.toggleSubMenu').removeClass("open");
		});

		if(!isOpened) {
			// If clicked menu was closed open it
			$('.subMenu', toggleSubMenu).slideDown("normal", function() {
				$(this).closest('.toggleSubMenu').addClass("open");
			});
		}
	});
}

// Initialize actions on icons menu
function initPlanIconsMenu()
{
	toggleIconsMenu('#adm_icons');

	// Set height to not be higher than plan-container
	let maxContentHeight = $('#plan' + glob_plan.id).height() - ($('#adm_icons').outerHeight() - $('#adm_icons .navigation').height());
	maxContentHeight -= 50;
	$('#adm_icons .navigation').css('max-height', maxContentHeight + 'px');

	// Set position to plan window right for not beign hidden by scroll
	const planWindowWidth = $('#plan-' + glob_plan.id + ' .plan-window').width();
	if($('#plan' + glob_plan.id).width() > planWindowWidth) {
		const initialPosX = (planWindowWidth - $('#adm_icons').width()) - 12; // -12 for right gap
		$('#adm_icons').css('left', initialPosX + 'px');
	}

	$('#adm_icons').draggable({
		containment: '.plan-window',
		handle: '.tableview_title',
		cursor: 'move'
	});
}

// Get object plan linked url
function getObjectPlanLink(id)
{
	$.ajax({
		type: "GET",
		url: glob_apache_path + "display_plan.php?action=getObjPlanLink",
		data: {id: glob_plan.id, objId: id},
		dataType: "json"
	}).done(function(data) {
		if(data.error) { // Error Message
			displayUserMessage(data.error, true);
		}
		else if(data.controller) {
			if(glob_action == 'dash_plan') {
				window.parent.location.href = glob_apache_path + data.controller;
			} else {
				window.location.href = glob_apache_path + data.controller;
			}
		}
	});
}

// Get plan object details alerts for table
function getObjectsDetails(id, title='')
{
	var dialog = $('#dialog-info');

	if(glob_action == 'dash_plan') {
		dialog = window.parent.$('#dialog-info-' + glob_plan.id);
	}

	dialog.dialog({
		autoOpen: false,
		resizable: false,
		modal: true,
		title: title,
		width: 1000,
		height: 'auto',
		close: function() {
			// On dialog closure remove dialog content to re-create new one on next openning
			dialog.dialog('destroy');
			dialog.children().remove();
		},
	});

	$.ajax({
		type: "GET",
		url: glob_apache_path + "display_plan.php?action=details",
		data: {id: glob_plan.id, objId: id},
		dataType: "json"
	}).done(function(data) {
		if(data?.error) { // Error Message
			displayUserMessage(data.error, true);
		}
		else if(data != null) {
			setObjDetailsContent(dialog, data);
			dialog.dialog("open");
		}
	});
}

function setObjDetailsContent(parent, alerts)
{
	var container = $("<div>", {}),
		table = $("<table>", {class: "tab", html:
			"<tr>" +
				"<th></th>" +
				"<th>" + glob_lang.NAME + "</th>" +
				"<th>" + glob_lang.ADM_PLUGIN_TYPE + "</th>" +
				"<th>" + glob_lang.ALERT_MENU + "</th>" +
			"</tr>"
		});

	table.css("font-size", "14px");

	// Groups
	if(alerts?.groups) {
		Object.entries(alerts.groups).forEach((entry) => {
			var group = entry[1];

			var row = $("<tr>", {html:
					'<td style="width:22px;"></td>' +
					'<td><a href="' + group.page_control + '"\>' + group.name + '</td>' +
					'<td>' + glob_lang.ADM_GRP + '</td>'
				}),
				tdState = $("<td>", {});

			switch(group.state)
			{
				case 'critical':
					tdState.addClass('EventCritical');
					tdState.html(glob_lang.LGD_CRI);
					break;
				case 'warning':
					tdState.addClass('EventWarning');
					tdState.html(glob_lang.LGD_WARN);
					break;
				case 'ok':
					tdState.addClass('EventOK');
					tdState.html(glob_lang.LGD_OK);
					break;
				default:
					tdState.addClass('EventUnknown');
					tdState.html(glob_lang.LGD_UNKNOWN);
			}

			row.append(tdState);
			table.append(row);
		});
	}

	// Nodes
	if(alerts?.nodes) {
		Object.entries(alerts.nodes).forEach((entry) => {
			var node = entry[1];

			var row = $("<tr>", {html:
					'<td class="cell-expand" style="width:22px;"></td>' +
					'<td><a href="' + node.page_control + '"\>' + node.name + '</td>' +
					'<td>' + glob_lang.NODE + '</td>'
				}),
				tdState = $("<td>", {});

			switch(node.state)
			{
				case 'critical':
					tdState.addClass('EventCritical');
					tdState.html(glob_lang.LGD_CRI);
					break;
				case 'warning':
					tdState.addClass('EventWarning');
					tdState.html(glob_lang.LGD_WARN);
					break;
				case 'ok':
					tdState.addClass('EventOK');
					tdState.html(glob_lang.LGD_OK);
					break;
				default:
					tdState.addClass('EventUnknown');
					tdState.html(glob_lang.LGD_UNKNOWN);
			}

			row.append(tdState);
			table.append(row);

			if(node.errors.length > 0) {
				var expandBtn = $("<a>", {href: "#", html:
						'<img id="bouton_node' + node.id + '" src="/theme/default-image/esia_plus_button.png"/>'
					}),
					expandRow = $("<tr>", {id: 'node' + node.id, html:
						'<td colspan="4">' +
						'<table class="expandErrTable" style="width:100%;">' +
							'<tr>' +
								'<th>' + glob_lang.SERVICES + '</th>' +
								'<th>' + glob_lang.MESSAGES + '</th>' +
								'<th>' + glob_lang.DATE_START + '</th>' +
							'</tr>' +
						'</table>'
					});

				expandRow.css('display', 'none');
				$('.cell-expand', row).append(expandBtn);

				node.errors.forEach((error) => {
					var rowErr = $("<tr>", {html:
							'<td>' + error.message + '</td>' +
							'<td>' + error.time + '</td>'
						}),
						tdErr = $("<td>", {html: error.service});

					switch(error.type)
					{
						case 'critical':
							tdErr.addClass('EventCritical');
							break;
						case 'warning':
							tdErr.addClass('EventWarning');
							break;
						default:
							tdErr.addClass('EventUnknown');
					}

					rowErr.prepend(tdErr);
					$('.expandErrTable', expandRow).append(rowErr);
				});

				table.append(expandRow);

				expandBtn.on("click", function(e) {
					e.preventDefault();
					toggleDetailsContent($(this), 'node' + node.id);
				});
			}
		});
	}

	// Services
	if(alerts?.services) {
		Object.entries(alerts.services).forEach((entry) => {
			var service = entry[1];

			var row = $("<tr>", {html:
					'<td class="cell-expand" style="width:22px;"></td>' +
					'<td><a href="' + service.page_control + '"\>' + service.device_name + " > " + service.service + '</td>' +
					'<td>' + glob_lang.SERVICE + '</td>'
				}),
				tdState = $("<td>", {});

			switch(service.state)
			{
				case 'critical':
					tdState.addClass('EventCritical');
					tdState.html(glob_lang.LGD_CRI);
					break;
				case 'warning':
					tdState.addClass('EventWarning');
					tdState.html(glob_lang.LGD_WARN);
					break;
				case 'ok':
					tdState.addClass('EventOK');
					tdState.html(glob_lang.LGD_OK);
					break;
				default:
					tdState.addClass('EventUnknown');
					tdState.html(glob_lang.LGD_UNKNOWN);
			}

			row.append(tdState);
			table.append(row);

			if(service.state != 'ok') {
				var expandBtn = $("<a>", {href: "#", html:
						'<img id="bouton_service' + service.id + '" src="/theme/default-image/esia_plus_button.png"/>'
					}),
					expandRow = $("<tr>", {id: 'service' + service.id, html:
						'<td colspan="4">' +
						'<table class="expandErrTable" style="width:100%;">' +
							'<tr>' +
								'<th>' + glob_lang.SERVICES + '</th>' +
								'<th>' + glob_lang.MESSAGES + '</th>' +
								'<th>' + glob_lang.DATE_START + '</th>' +
							'</tr>' +
						'</table>'
					}),
					rowErr = $("<tr>", {html:
						'<td>' + service.message + '</td>' +
						'<td>' + service.time + '</td>'
					}),
					tdErr = $("<td>", {html: service.service});

				expandRow.css('display', 'none');
				$('.cell-expand', row).append(expandBtn);

				switch(service.state)
				{
					case 'critical':
						tdErr.addClass('EventCritical');
						break;
					case 'warning':
						tdErr.addClass('EventWarning');
						break;
					default:
						tdErr.addClass('EventUnknown');
				}

				rowErr.prepend(tdErr);
				$('.expandErrTable', expandRow).append(rowErr);

				table.append(expandRow);

				expandBtn.on("click", function(e) {
					e.preventDefault();
					toggleDetailsContent($(this), 'service' + service.id);
				});
			}
		});
	}

	container.append(table);
	parent.append(container);
}

// Use to open details row. Be sure than selectors can be used in dash_plan where the dialog is outside the iframe containing this plan.
function toggleDetailsContent(togglerBtn, rowId)
{
	if(togglerBtn.hasClass('show-details')) {
		togglerBtn.removeClass('show-details');
		togglerBtn.closest('.tab').find('#' + rowId).css('display', 'none');
		$('img', togglerBtn).attr('src', '/theme/default-image/esia_plus_button.png');
	}
	else {
		togglerBtn.addClass('show-details');
		togglerBtn.closest('.tab').find('#' + rowId).css('display', 'table-row');
		$('img', togglerBtn).attr('src', '/theme/default-image/esia_moins_button.png');
	}
}

// Be sure that an elt can't be dragged outside it's wrapper
function planSafePosElt(elt, wrapper, maxPos, dragOffset, dragPos) {
	// Take care of wrapper scroll for max positions
	var scrolledMaxTop = maxPos.top - wrapper.scrollTop(),
		scrolledMaxLeft = maxPos.left - wrapper.scrollLeft();

	// Top
	if(dragOffset.top < scrolledMaxTop) {
		elt.css("top", -dragPos.top);
	} else {
		elt.css("top", "");
	}

	// Left
	if(dragOffset.left < scrolledMaxLeft) {
		elt.css("left", -dragPos.left);
	} else {
		elt.css("left", "");
	}
}

// Don't let edit menu get out of the plan for not jquery ui draggable objects
function commonSafePosObjMenu(draggableElt, menu, plan, maxPos) {
	// Check position when dragging the object
	let isDragging = false;
	let initPos = {
		top: draggableElt.offset().top - plan.offset().top,
		left: draggableElt.offset().left - plan.offset().left
	};

	// On init check position in case of the menu is already outside the plan
	planSafePosElt(menu, plan, maxPos, draggableElt.offset(), initPos);
	// Check position when dragging the object
	draggableElt.on('mousemove.dragMenu', function() {
		if(isDragging == true) {
			let position = {
				top: draggableElt.offset().top - plan.offset().top,
				left: draggableElt.offset().left - plan.offset().left
			};

			planSafePosElt(menu, plan, maxPos, draggableElt.offset(), position);
		}
	}).on('mouseup.dragMenu', function() {
		isDragging = false;
	})
	.on('mousedown.dragMenu', function() {
		isDragging = true;
	});
}

// Don't let edit menu get out of the plan for jquery ui draggable objects
function uiDraggableSafePosObjMenu(draggableElt, menu, plan, maxPos) {
	// On init check position in case of the menu is already outside the plan
	planSafePosElt(menu, plan, maxPos, draggableElt.offset(), draggableElt.position());
	// Check position when dragging the object
	draggableElt.on("drag", function(e, ui) {
		planSafePosElt(menu, plan, maxPos, ui.offset, ui.position);
	});
}

// Don't let edit menu get out of the plan
function safePosObjectEditMenu(menu) {
	var draggableElt = menu.closest(".planObj"),
		plan = $(".plan-container"),
		maxPos = {
			top: plan.offset().top - menu.position().top,
			left: plan.offset().left - menu.position().left
		};

	// Set event for dragging
	if(draggableElt.hasClass('ui-draggable')) {
		uiDraggableSafePosObjMenu(draggableElt, menu, plan, maxPos);
	} else {
		commonSafePosObjMenu(draggableElt, menu, plan, maxPos);
	}
}

// Used to set obj-hover class to a draggable plan object when the draggable handler is child of draggable object and not itself. It allow to set styles on the object when its child handler is hover
function objHoverDraggableHandleChild(planObj) {
	$(".ui-draggable-handle", planObj).mouseenter(function(e) {
		planObj.addClass("obj-hover");
	}).mouseleave(function() {
		planObj.removeClass("obj-hover");
	});
}

// Annimation to fade an user message
function fadeUserMessage(item) {
	item.delay(4000).fadeOut(500);
}

// Display error or success messages
function displayUserMessage(message, isError=true, idHtml="messages") {
	var container = $("<div>", {"class": "messageUtilisateurs"});

	if(isError) {
		container.addClass("EventCritical");
	} else {
		container.addClass("EventOK");
	}

	if($.isArray(message)) {
		$.each(message, function() {
			container.append("<p>" + this + "</p>");
		});
	} else {
		container.append("<p>" + message + "</p>");
	}

	// Display new message during 5s
	$("#" + idHtml).append(container);
	fadeUserMessage(container);
}

// Original method to close object edit menu
function closeObjectEditMenu(container) {
	$(".plan-obj-infos, .infos", container).remove();
	container.removeClass("edit-mode");
	container.trigger('editClose');
	// Re-enable tooltip on main object
	if(container.hasClass('tooltipstered')) {
		container.tooltipster('enable');
	}
	$('.tooltipstered', container).tooltipster('enable');
}

// Initialize object edit mode menu
function initObjectEditMenu(container, actionsMenu) {
	if($(".plan-ctrl-event").length == 0) { // Don't open menu if ctrl is pressed
		var wrapper = $("<div>", {"class": "plan-obj-infos"});

		// Close all others edit menu
		$('.plan-obj-infos, .planObj .infos').each(function() {
			var container = $(this).closest(".planObj");
			// Only if not in resize mode (validation is required on resize events)
			if(!container.hasClass("ui-resizable")) {
				closeObjectEditMenu(container);
			}
		});
		// Reset click var value on plan images objects types container (.planObj .infos)
		if(typeof click !== 'undefined') {
			click = 0;
		}

		wrapper.append(actionsMenu);
		container.append(wrapper);

		container.addClass("edit-mode");

		// Prevent to re-open edit menu actions on action button click
		$(".obj-action", actionsMenu).on("click", function(e) {
			e.stopPropagation();
		});

		// Prevent object edit menu to get out of the plan and be overflow
		safePosObjectEditMenu(wrapper);

		// Disable tooltip on main object to not hide the menu behind
		if(container.hasClass('tooltipstered')) {
			container.tooltipster('disable');
		}
		$('.tooltipstered', container).tooltipster('disable');
		// Add menu button tooltip
		addTooltipTitle($('.obj-action', actionsMenu));
	}
}

// Handle an action menu with accept or refuse button
function acceptRefuseActionMenu(actionsMenu, acceptButton, refuseButton) {
	actionsMenu.children().each(function() {
		$(this).hide();
	});
	actionsMenu.append(acceptButton).append(refuseButton);

	// Cancel object deletion
	refuseButton.on("click", function() {
		acceptButton.remove();
		refuseButton.remove();
		actionsMenu.children().each(function() {
			$(this).show();
		});
	});

	addTooltipTitle($('.obj-action', actionsMenu));
}

// Open dialog window to configure object
function openConfObjDialog(url, title, width=1280, height=800) {
	var dialog = $("#dialog-info"),
		dialogIframe = $("<iframe>", {"src": url});

	// Create object parameter dialog
	dialog.dialog({
		resizable: true,
		modal: true,
		autoOpen: true,
		width: width,
		height: height,
		title: title,
		close: function() {
			// On dialog closure remove dialog content to re-create new one on next opening
			$(this).dialog("destroy");
			$(this).children().remove();
		}
	});
	dialogIframe.css({
		"width": "100%",
		"height": "100%"
	});
	dialog.append(dialogIframe);
}

// Add title tooltip styles
function addTooltipTitle(elements) {
	// Not for dashboards mode
	if($('.plan-main').attr('data-action') != 'dash_plan' && $('.plan-main')?.attr('data-tooltip') != 'false') {
		elements.each(function() {
			// Title must not be empty to not create empty tooltip bubble
			if($(this).attr('title') != undefined && $(this).attr('title') != '') {
				$(this).tooltipster({theme: 'tooltipster-borderless'});
			}
		});
	}
}

// Check if an element is inside limits position without overflow. Positions must be calculated relativelly to #plan
// elt => jQuery element
// limits => object topLeft.x & topLeft.y are top left corner positions and bottomRight.x & bottomRight.y are bottom right positions
function isPlanEltFullyInsideLimits(elt, limits) {
	// Use offset for objects wich are not directly children of #plan
	var eltPos = {
			topLeft: {
				x: elt.offset().left - $(".plan-container").offset().left,
				y: elt.offset().top - $(".plan-container").offset().top,
			},
			bottomRight: {
				x: 0,
				y: 0,
			},
		};

	eltPos.bottomRight.x = eltPos.topLeft.x + elt.width();
	eltPos.bottomRight.y = eltPos.topLeft.y + elt.height();

	var isPosInsideLimits = function(pos, limits) {
		if(pos.x >= limits.topLeft.x && pos.x <= limits.bottomRight.x && pos.y >= limits.topLeft.y && pos.y <= limits.bottomRight.y) {
			return true;
		} else {
			return false;
		}
	};

	if(isPosInsideLimits(eltPos.topLeft, limits) && isPosInsideLimits(eltPos.bottomRight, limits)) {
		// Elt top left corner && elt bottom rigth corner are inside limits all the rectangle is inside
		return true;
	} else {
		return false;
	}
}

// Plan objects multi selection (add or remove) by mouse area and by simple click
function planMultiSelectObjects() {
	var plan = $(".plan-container"),
		selectFrame = $('#mouseAreaSelection'),
		pos = {x1: 0, x2: 0, y1: 0, y2: 0},
		isSelectValid = false,
		isCtrlMaintained = false;

	// Re-calc the selection frame position and size
	var reCalc = function(selectFrame, pos) {
		var x3 = Math.min(pos.x1, pos.x2),
			x4 = Math.max(pos.x1, pos.x2),
			y3 = Math.min(pos.y1, pos.y2),
			y4 = Math.max(pos.y1, pos.y2);

		selectFrame.css({
			width: x4 - x3 + 'px',
			height: y4 - y3 + 'px',
			top: y3 + 'px',
			left: x3 + 'px',
		});
	};

	// Init the selection frame
	$(document).on("mousedown", function(e) {
		if(!$(e.target).hasClass("planObj-selected") && !e.ctrlKey) {
			// If it's a simple click on an object (not a maintained click for drag), remove all objects selected classes
			disableMultiDragOnSelectedObj();
		}

		// Start selection only if click on the plan and nothing else
		if(e.target.id == "plan") {
			isSelectValid = true;
		} else {
			isSelectValid = false;
		}

		if(isSelectValid) {
			// Get initial mouse position on plan
			pos.x1 = (e.pageX - plan.position().left) + plan[0].scrollLeft;
			pos.y1 = (e.pageY - plan.position().top) + plan[0].scrollTop;
			// Avoid weard blink selection started from beginin because x2, y2 are not still setted when select frame is show
			pos.x2 = pos.x1;
			pos.y2 = pos.y1;

			reCalc(selectFrame, pos);
			selectFrame.css("display", "block"); // Show the selection frame
		}
	});

	// Resize the selection frame on mouse move
	$(document).on("mousemove", function(e) {
		if(isSelectValid) {
			// Math.max(0, is used to prevent negative values because select area can't get out of the plan
			pos.x2 = Math.max(0, (e.pageX - plan.position().left)) + plan[0].scrollLeft;
			pos.y2 = Math.max(0, (e.pageY - plan.position().top)) + plan[0].scrollTop;
			reCalc(selectFrame, pos);
		}
	});

	// Handle the end of selection
	$(document).on("mouseup", function(e) {
		if(isSelectValid) {
			var framePos = {
					topLeft: {
						x: selectFrame.position().left,
						y: selectFrame.position().top,
					},
					bottomRight: {
						x: selectFrame.position().left + selectFrame.width(),
						y: selectFrame.position().top + selectFrame.height(),
					},
				};

			// Check if every plan objects are completly inside the selection frame and add selected class to them
			$('.planObj').each(function() {
				if(isPlanEltFullyInsideLimits($(this), framePos)) {
					if(e.ctrlKey) {
						// If ctrl is maintained we can select or unselect
						$(this).toggleClass("planObj-selected");
					} else {
						$(this).addClass("planObj-selected");
					}
				}
			});

			if($(".planObj-selected").length > 0) {
				plan.trigger("newMultiSelect");
			}

			isSelectValid = false;
		}

		selectFrame.css("display", ""); // Hide the selection frame
	});

	// Handle objects selection on simple click
	$(".planObj").on("click", function(e) {
		if(e.ctrlKey) {
			// If ctrl is maintained we can select or unselect on simple click
			$(this).toggleClass("planObj-selected");
			plan.trigger("newMultiSelect");
		} else if($(this).hasClass("planObj-selected")) {
			// Remove selection on click to plan obj selected to avoid openening unusable menu because event pointers are disabled on .planObj-selected children
			disableMultiDragOnSelectedObj();
		}
	});

	// Add/remove specific class to plan to know if ctrl key is pressed or not even without the event.ctrl variable
	$(document).on("keydown", function(e) {
		if(e.ctrlKey) {
			isCtrlMaintained = true;
			plan.addClass("plan-ctrl-event");
		} else if(e.keyCode == 27) {
			disableMultiDragOnSelectedObj();
		}
	}).on("keyup", function() {
		if(isCtrlMaintained) {
			isCtrlMaintained = false;
			plan.removeClass("plan-ctrl-event");
		}
	});

	// Set multidrag events
	plan.on("newMultiSelect", function() {
		enableMultiDragOnSelectedObj();
	});
}

// Remove all selected classes & reinitialise all events on older selected objects
function disableMultiDragOnSelectedObj() {
	$(".planObj-selected").removeClass("planObj-selected");
	$(".planObj.ui-draggable").off("dragstart.multiDrag").off("drag.multiDrag").off("dragstop.multiDrag");
	$(".plan-container").trigger("removeMultiSelect");
}

// Drag multi objects together if selected
function enableMultiDragOnSelectedObj() {
	var isMultiDragging = false;

	// Reinitialise all events on older selecte objects
	$(".ui-draggable").off("dragstart.multiDrag").off("drag.multiDrag").off("dragstop.multiDrag");

	// Set events on new selected objects
	$(".ui-draggable.planObj-selected").not(".ui-draggable-disabled").on("dragstart.multiDrag", function(e) {
		// Prevent events retriggering themselves
		if(!isMultiDragging) {
			var othersSelected = $(".planObj-selected").not($(this)).not(".ui-draggable-disabled"), // All selected object except the one which is currently dragging
				othersUi = [],
				startOffset = $(this).offset(); // Get initial position of user mouse dragging obj (the one in the selection the user click on)

			isMultiDragging = true;

			othersSelected.each(function(index, element) {
				var originalPos = {
					top: $(element).position().top + $(element).parent().scrollTop(),
					left: $(element).position().left + $(element).parent().scrollLeft(),
				};

				// Recreate correct ui for each elements
				othersUi[index] = {
					helper: $(element).draggable("widget"),
					originalOffset: $(element).offset(), // Not a jquery ui parameter but used in this function
					offset: $(element).offset(),
					originalPosition: originalPos,
					position: originalPos,
				};

				// Send start event on others elements
				$(element).trigger("multidrag:start", [othersUi[index]]);
			});

			// Initialize drag event
			$(this).on("drag.multiDrag", function(e) {
				// Get position gap of currently dragged element from it's started position
				var draggedOffset = {
						top: $(this).offset().top - startOffset.top,
						left: $(this).offset().left - startOffset.left,
					};

				// For others obj update positions from they're offset start + gap off current dragged obj
				othersSelected.each(function(index, element) {
					// Don't drag disabled elements
					if(!$(element).hasClass("ui-draggable-disabled") && !$(element).hasClass("common-multi-drag-disabled")) {
						var parent = $(element).parent(),
							posLimits = {
								right: parent[0].scrollWidth - $(element).outerWidth(),
								bottom: parent[0].scrollHeight - $(element).outerHeight(),
							},
							newPos = {
								top: (othersUi[index].originalOffset.top - parent.offset().top) + draggedOffset.top + parent.scrollTop(),
								left: (othersUi[index].originalOffset.left - parent.offset().left) + draggedOffset.left + parent.scrollLeft(),
							};

						// Check values to keep all selected elements in parent limits
						if(newPos.top < 0) {
							newPos.top = 0;
						} else if(newPos.top > posLimits.bottom) {
							newPos.top = posLimits.bottom;
						}

						if(newPos.left < 0) {
							newPos.left = 0;
						} else if(newPos.left > posLimits.right) {
							newPos.left = posLimits.right;
						}

						$(element).css({top: newPos.top + "px", left: newPos.left + "px"});

						// Update ui offset & position
						othersUi[index].offset = $(element).offset();
						othersUi[index].position = newPos;
					}

					// Send dragging event on others elements
					$(element).trigger("multidrag:drag", [othersUi[index], draggedOffset]);
				});
			});

			$(this).on("dragstop.multiDrag", function(e) {
				othersSelected.each(function(index, element) {
					// Send stop event on others elements
					$(element).trigger("multidrag:stop", [othersUi[index]]);
				});

				// Reinitialize isMultiDragging to allow restart a new multi drag
				isMultiDragging = false;
				// Remove events to reinitialise them on new multi drag
				$(this).off("drag.multiDrag").off("dragstop.multiDrag");
			});
		}
	});
}

// Return all inputs, select, ... of an html form as a data object (like used in $.ajax data: {}).
function getFormValues(idHtml) {
	return $('#' + idHtml).serializeArray().reduce(function(o, input) {
		if(input.name.slice(-2) === '[]') {
			// If input name ends by [] its val should be an array
			// Remove [] from the name
			input.name = input.name.slice(0, -2);
			if(!o[input.name]) {
				// If the array key doesn't exist create it and initialize the first array value
				o[input.name] = [input.value];
			} else {
				// The array key already exist push the new value into the array
				o[input.name].push(input.value);
			}
		} else {
			o[input.name] = input.value;
		}

		return o;
	}, {});
}

// Open dialog window to configure object by ajax method instead of iframe
// url is required but any dialog or ajax configuration parameter can be overhide by passing it into dialogParams or ajaxParams. Example to change dialog width only => dialogParams = {width: 800}
function openDialogByAJAX(url, requestData={}, title='', dialog=$('#dialog-info'), dialogParams={}, ajaxParams={}, isBottomActions=true) {
	if(typeof dialog == 'string') {
		dialog = $('#' + dialog);
	}

	dialog.addClass('ajaxDialog');
	if(!isBottomActions) {
		dialog.addClass('noBottomActions');
	}

	const setMediaMaxWidth = () => {
		// Set html attribute data-media-maxwidth with a value to be used in css as selector by window size
		// Sizes must be declared by ascending order
		const sizes = dialog.dialog('option').mediaSizes;
		let mediaMaxWidth = 0;

		// Loop on each sizes by ascending order if one is smaller than the width provided set mediaMaxWidth as size compared and break the loop (if width is smaller than 320 it is not neccessary to compare if it smaller than 420).
		for (let i = 0; i < sizes.length; i++) {
			if(dialog.dialog('widget').width() < sizes[i]) {
				mediaMaxWidth = sizes[i];
				break;
			}
		}

		dialog.attr('data-media-maxwidth', mediaMaxWidth);
	}

	const triggerResize = () => {
		dialog.dialog("option").resize();
	}

	dialog.dialog({
		resizable: true,
		modal: true,
		title: title,
		width: 1280,
		height: 'auto',
		maxHeight: 800,
		mediaSizes: [], // Sizes must be declared by ascending order (see setMediaMaxWidth() function)
		...dialogParams,
		open: () => {
			setMediaMaxWidth();

			$(window).on('resize', triggerResize);

			if(typeof dialogParams.open === 'function') {
				dialogParams.open();
			}
		},
		close: () => {
			// On dialog closure remove dialog content to re-create new one on next openning
			dialog.dialog('destroy');
			dialog.children().remove();
			dialog.off("validation");
			dialog.removeClass('ajaxDialog');
			$(window).off('resize', triggerResize);

			if(typeof dialogParams.close === 'function') {
				dialogParams.close();
			}
		},
		resize: () => {
			setMediaMaxWidth();

			if(typeof dialogParams.resize === 'function') {
				dialogParams.resize();
			}
		},
		autoOpen: false,
	});

	// Ajax request to get dialog html content
	$.ajax({
		type: 'POST',
		url: url,
		data: requestData,
		dataType: 'html',
		...ajaxParams
	})
	.done(function(data) {
		if(data) {
			dialog.append(data);
			dialog.find('.btnBack').on('click', function() {
				dialog.dialog('close');
			});
			$('#btnValidDialog').on("click", function() {
				dialog.trigger('validation');
			});

			// Set tooltip on everything which has title attribute inside dialog
			addTooltipTitle($('[title]', dialog));

			dialog.dialog('open');
		}
	});
}

// Set delete plan dialog
function openDelPlanDialog() {
	$("#deletePlan").on("click", function(e) {
		e.preventDefault();

		var dialogParams = {
				resizable: false,
				title: glob_lang.DELETE + " - " + glob_plan.name,
				draggable: false,
				width: 800
			},
			requestData = {
				name: glob_plan.name,
				folder_id: glob_plan.folder_id
			};

		openDialogByAJAX($(this).attr('href'), requestData, '', 'dialog-info', dialogParams);
	});
}

// Get services on a node. Use it as promise => getNodeServices(nodeId).then(function(services) {});
function getNodeServices(nodeId, displayMsgId="messages") {
	return $.ajax({
		type: "GET",
		url: glob_apache_path + "display_plan.php?id=" + glob_plan.id,
		data: {action: "getNodeServices", nodeId: nodeId},
		dataType: "json"
	})
	.done(function(data) {
		if(data.error) {
			// Error Message
			displayUserMessage(data.error, true, displayMsgId);
		} else {
			return data;
		}
	});
}

// Update the service selection options
function updSelectService(nodeId, select) {
	getNodeServices(nodeId).then(function(services) {
		$.each(services, function(key, service) {
			select.append("<option value='" + service.id + "'>" + service.name + "</option>");
		});
		select.trigger("change");
		select.trigger("chosen:updated");
	});
}

// Handle the click on resize button of the object actions menu
function actionResizeObject(objId, actionsMenu, resizableElt, fctOnResizeRefused=()=>{}, fctOnResizeAccepted=()=>{}) {
	var acceptButton = $("<button>", {"class": "obj-action accept-action", "type": "button", "title": glob_lang.PLAN_MODIFY_VALIDATE}),
		refuseButton = $("<button>", {"class": "obj-action refuse-action", "type": "button", "title": glob_lang.PLAN_MODIFY_ABORT});

	acceptRefuseActionMenu(actionsMenu, acceptButton, refuseButton);

	// Enable the resizableElt
	resizableElt.resizable("enable");

	// Validate object resize
	acceptButton.on("click", function() {
		var width = resizableElt.outerWidth(),
			height = resizableElt.outerHeight();

		fctOnResizeAccepted(objId);
		resizableElt.resizable("destroy");
		closeObjectEditMenu(resizableElt);
		glob_plan_has_changes = true;
	});

	// Cancel object resize
	refuseButton.on("click", function() {
		fctOnResizeRefused();
		resizableElt.resizable("destroy");
		closeObjectEditMenu(resizableElt);
	});
}

// Do something on element drag or on click without drag between mouse down & up
// Example of use:
// onEltClickOrDrag($('.plan-window'), function(isEltDrag, e) {
// 	if(isEltDrag && $(e.target).hasClass('objContainer')) {
// 		console.log("Drag object container");
// 	} else if(!isEltDrag) {
// 		console.log("Plan simple click without drag");
// 	}
// });
function onEltClickOrDrag(elt, callback) {
	var isEltDrag = false;

	elt.on('click', function(e) {
		callback(isEltDrag, e);
	});

	$(document).on('mousemove', function() {
		isEltDrag = true;
	}).on('mousedown', function() {
		isEltDrag = false;
	});
}

// Set resizable plan
function setPlanResizable() {
	$('#plan' + glob_plan.id).resizable({
		disabled: false,
		aspectRatio: false,
		minWidth: 200,
		minHeight: 200,
		resize: function() {
			// Keep the view to the bottom-right corner to make the resize fluid
			$(this).closest('.plan-window')[0].scrollTo($(this).width(), $(this).height());
		}
	});
}

// Save plan size into hidden input
function savePlanSize()
{
	const plan = $('#plan' + glob_plan.id);

	$("#plan_size").val(JSON.stringify({
		width: plan.width(),
		height: plan.height()
	}));
}

// Set plan size if size param exists
function setPlanSize()
{
	if(glob_plan?.params?.size) {
		$('#plan' + glob_plan.id).addClass('plan-fixed-size').css({
			width: glob_plan.params.size.width + 'px',
			height: glob_plan.params.size.height + 'px',
		});
	}
}

// Get plan service graph (link)
function getServiceGraph(id, title='')
{
	var dialog = $('#dialog-info');
	if(glob_action == 'dash_plan') {
		dialog = window.parent.$('#dialog-info-' + glob_plan.id);
	}


	if(title == '' && glob_lang?.GRAPH != undefined) {
		title = glob_lang.GRAPH;
	}

	openDialogByAJAX(
		glob_apache_path + "dialog_plan.php?action=serviceGraph&id=" + glob_plan.id,
		{
			serviceId: id,
			isIframe: true
		},
		title,
		dialog,
		{}
	);
}

// Drag object on plan
function dragPlanObj(object)
{
	object.draggable({
		containment: $("#plan" + glob_plan.id),
		cursor: "move",
		drag: function(e, ui) {
			glob_plan_has_changes = true;
		},
	});
}

// Get a random obj id for a new object on plan
function getRandomObjId()
{
	return 'tmp-' + Date.now();
}

// Check if the user has the rights to modify a plan (Will throw an exception if not and return true if he can)
function isPlanWrittable()
{
	return $.ajax({
		type: "POST",
		url: glob_apache_path + "display_plan.php?id=" + glob_plan.id,
		data: { action: "isPlanWrittable" },
		dataType: "json"
	})
	.done(data => {
		if(data.error) {
			// Error returning by php
			throw new Error(data.error);
		} else {
			return data;
		}
	});
}

// Retrieve saved messages from another page in session var (js session var cannot be accessible by php because it's saved on the browser not on the server)
function getPlanSessionMessage()
{
	// Succces message
	const success = sessionStorage.getItem('plan_msg_success');
	if(success) {
		sessionStorage.removeItem('plan_msg_success');
		displayUserMessage(success, false);
	}
	// Error message
	const error = sessionStorage.getItem('plan_msg_error');
	if(error) {
		sessionStorage.removeItem('plan_msg_error');
		displayUserMessage(error, true);
	}
}

// Perform a Jquery ajax request with the same behaviour as a common Promise to use it with (async/await statements and error handeling with try catch)
// Usage:
// $.ajax({type: "POST",url: url, ... }).done(data => {return data;});
// ajaxPromise({type: "POST",url: url, ... }).then(data => {return data;});
function ajaxPromise(settings)
{
	return new Promise((resolve, reject) => {
		$.ajax(settings)
		.done(data => resolve(data))
		.fail((jqXHR, textStatus, errorThrown) => reject(errorThrown));
	});
}

// Perform an ajax request with response in json usable in try catch async/await statement. Error handeling is preconfigured for php error trow
function asyncSaveData(settings)
{
	return ajaxPromise({
		type: "POST",
		...settings,
		dataType: "json",
	})
	.then(data => {
		if(data.error) {
			// Error returned by php
			throw new Error(data.error);
		} else {
			return data;
		}
	});
}

// Delete objects on plan. Return a promise usable in async/await try catch
function asyncDeleteObjects(apachePath, planId, objIds)
{
	if(objIds.length > 0) {
		return asyncSaveData({
			url: apachePath + "display_plan.php?id=" + planId,
			data: {
				action: "deleteObjects",
				delete_objects: JSON.stringify(objIds),
			},
		});
	} else {
		return Promise.resolve(true);
	}
}
