<?php
/*
** @name: AbstractPlan.class.php
**
** @description: Parent class of Plan.class & Object.class
** @author: Quentin George <q.george@esia-sa.com>
** @date: 12/01/2022
*/
// namespace Plan;
namespace Plan;

abstract class AbstractPlan
{
	protected $id;
	protected $name;
	protected $type_id;
	protected $type_name;
	protected $params;
	protected $state;
	/**
	***
	*** @NAME: setId
	*** @DESC: Set identifier
	*** @IN: (INTEGER) $id => identifier
	*** @OUT: /
	*** @THROW: /
	**/
	public function setId($id)
	{
		$this->id = $id;
	}
	/**
	***
	*** @NAME: setName
	*** @DESC: Set name
	*** @IN: (STRING) $name
	*** @OUT: /
	*** @THROW: /
	**/
	public function setName($name)
	{
		$this->name = $name;
	}
	/**
	***
	*** @NAME: setTypeId
	*** @DESC: Set type identifier
	*** @IN: (INTEGER) $id => type identifier
	*** @OUT: /
	*** @THROW: /
	**/
	public function setTypeId($id)
	{
		$this->type_id = $id;
	}
	/**
	***
	*** @NAME: setTypeName
	*** @DESC: Set type name
	*** @IN: (STRING) $type => type name
	*** @OUT: /
	*** @THROW: /
	**/
	public function setTypeName($type)
	{
		$this->type_name = $type;
	}
	/**
	***
	*** @NAME: setParams
	*** @DESC: Set params. Inside the DB it's in JSON format
	*** @IN: (ARRAY or STRING) $params => string if json encoded
	*** @OUT: /
	*** @THROW: PlanException if JSON cannot be decoded or $params is not an array
	**/
	public function setParams($params)
	{
		if(empty($params))
		{
			$this->params = array();
		}
		else
		{
			// If string params are JSON encoded
			if(is_string($params))
			{
				$params = json_decode($params, true);

				if(json_last_error() !== JSON_ERROR_NONE)
				{
					throw new PlanException('JSON error: ' . json_last_error_msg());
				}
			}

			if(is_array($params))
			{
				$this->params = $params;
			}
			else
			{
				throw new PlanException('Wrong parameter type');
			}
		}
	}
	/**
	***
	*** @NAME: setParam
	*** @DESC: Add or Update a part of params
	*** @IN: (ARRAY) $keys => path to the value. $keys = array("person", "name") will set $params["person"]["name"]
	*** @IN: (ANY) $val => value to set
	*** @OUT: /
	*** @THROW: /
	**/
	public function setParam($keys, $val)
	{
		eval('
			$this->params["' . implode('"]["', $keys) . '"] = $val;
		');
	}
	/**
	***
	*** @NAME: deleteParam
	*** @DESC: Remove a part of params
	*** @IN: (ARRAY) $keys => path to the value. $keys = array("person", "name") will remove $params["person"]["name"]
	*** @OUT: /
	*** @THROW: /
	**/
	public function deleteParam($key)
	{
		eval('
			unset($this->params["' . implode('"]["', $keys) . '"]);
		');
	}
	/**
	***
	*** @NAME: setState
	*** @DESC: Set global state
	*** @IN: (STRING||INTEGER) $state
	*** @OUT: /
	*** @THROW: /
	**/
	public function setState($state)
	{
		$state = strtolower($state); // If state is an integer strtolower will return it as string
		switch($state)
		{
			case 'ok':
			case '0':
				$this->state = 'ok';
				break;
			case 'warning':
			case '1':
				$this->state = 'warning';
				break;
			case 'critical':
			case '2':
				$this->state = 'critical';
				break;
			case 'pending':
			case '4':
				$this->state = 'pending';
				break;
			default:
				$this->state = 'unknown';
		}
	}
	/**
	***
	*** @NAME: getId
	*** @DESC: Return identifier
	*** @IN: /
	*** @OUT: Identifier
	*** @THROW: /
	**/
	public function getId()
	{
		if(isset($this->id) && !empty($this->id))
		{
			return $this->id;
		}
		else
		{
			return null;
		}
	}
	/**
	***
	*** @NAME: getName
	*** @DESC: Return name
	*** @IN: /
	*** @OUT: Name
	*** @THROW: /
	**/
	public function getName()
	{
		return $this->name;
	}
	/**
	***
	*** @NAME: getTypeId
	*** @DESC: Return type identifier
	*** @IN: /
	*** @OUT: Type id
	*** @THROW: /
	**/
	public function getTypeId()
	{
		return $this->type_id;
	}
	/**
	***
	*** @NAME: getTypeName
	*** @DESC: Return type name
	*** @IN: /
	*** @OUT: Type name
	*** @THROW: /
	**/
	public function getTypeName()
	{
		return $this->type_name;
	}
	/**
	***
	*** @NAME: getParams
	*** @DESC: Return params
	*** @IN: /
	*** @OUT: Params array
	*** @THROW: /
	**/
	public function getParams($param='')
	{
		if(!empty($param))
		{
			return $this->params[$param];
		}
		else
		{
			return $this->params;
		}
	}
	/**
	***
	*** @NAME: getState
	*** @DESC: Return global state
	*** @IN: /
	*** @OUT: State as string
	*** @THROW: /
	**/
	public function getState()
	{
		return $this->state;
	}
	/**
	***
	*** @NAME: getStateCode
	*** @DESC: Return global state code integer
	*** @IN: /
	*** @OUT: State code
	*** @THROW: /
	**/
	public function getStateCode()
	{
		switch($this->state)
		{
			case 'ok':
				return 0;
			case 'warning':
				return 1;
			case 'critical':
				return 2;
			case 'pending':
				return 4;
			default:
				return 3; // unknown
		}
	}
}

class PlanException extends \Exception{}
?>
