<?php
/*
** @name: ObjectPlan.class.php
**
** @description: Class for objects plans of esia webp-plan
** @author: Quentin George <q.george@esia-sa.com>
** @date: 12/01/2022
*/
namespace Plan;

class ObjectPlan extends AbstractPlan
{
	protected $link_plan_id;
	protected $icon_id;
	protected $icon_name;
	protected $icon_type;
	protected $icon_path;
	/**
	***
	*** @NAME: __construct
	*** @DESC: Default ObjectPlan.class constructor
	*** @IN: (Optional) (OBJECT) $obj =>  if it's a 'ObjectPlan.class' object, the new object will be a copy of this one
	*** @OUT: /
	*** @THROW: PlanException if the object is not a 'ObjectPlan.class' object
	**/
	public function __construct($obj=NULL)
	{
		if(is_object($obj))
		{
			if(get_class($obj) == 'Plan\ObjectPlan')
			{
				$this->constructClone($obj);
			}
			else
			{
				throw new PlanException('$obj is not a ObjectPlan class');
			}
		}
		else
		{
			$this->params = array();
			$this->state = 'unknown';
		}
	}
	/**
	***
	*** @NAME: constructClone
	*** @DESC: Copy constructor
	*** @IN: (OBJECT) $obj => 'ObjectPlan.class' object
	*** @OUT: /
	*** @THROW: /
	**/
	public function constructClone(\Plan\ObjectPlan $obj)
	{
		$this->id = $obj->getId();
		$this->name = $obj->getName();
		$this->type_id = $obj->getTypeId();
		$this->type_name = $obj->getTypeName();
		$this->params = $obj->getParams();
		$this->state = $obj->getState();

		$this->link_plan_id = $obj->getLinkPlanId();
		$this->icon_id = $obj->getIconId();
		$this->icon_name = $obj->getIconName();
		$this->icon_type = $obj->getIconType();
		$this->icon_path = $obj->getIconPath();
	}
	/**
	***
	*** @NAME: toArray
	*** @DESC: return all class vars and methods results as an array
	*** @IN: (Optional) (STRING) $path => $_SERVER['DOCUMENT_ROOT'] If object icon file is required
	*** @OUT: array all methods converted to array
	*** @THROW: /
	**/
	public function toArray($path=NULL)
	{
		$arr = array(
			'id' => $this->id,
			'name' => $this->name,
			'type_id' => $this->type_id,
			'type_name' => $this->type_name,
			'params' => $this->params,
			'state' => $this->state,
			'state_code' => $this->getStateCode(),
			'link_plan_id' => $this->link_plan_id,
			'icon_id' => $this->icon_id,
			'icon_name' => $this->icon_name,
			'icon_type' => $this->icon_type,
			'icon_path' => $this->icon_path,
			'icon_file' => ''
		);

		if(!empty($path) && !empty($this->icon_id))
		{
			$arr['icon_file'] = $this->getIconFile($path);
		}

		return $arr;
	}
	/**
	***
	*** @NAME: setLinkPlanId
	*** @DESC: Set plan obj plan linked id
	*** @IN: (INTEGER) $id => id of the plan linked to the object
	*** @OUT: /
	*** @THROW: /
	**/
	public function setLinkPlanId($id)
	{
		$this->link_plan_id = $id;
	}
	/**
	***
	*** @NAME: setIconId
	*** @DESC: Set plan obj icon id
	*** @IN: (INTEGER) $id => id of the plan obj icon
	*** @OUT: /
	*** @THROW: /
	**/
	public function setIconId($id)
	{
		$this->icon_id = $id;
	}
	/**
	***
	*** @NAME: setIconName
	*** @DESC: Set plan obj icon name
	*** @IN: (STRING) $name => name of the plan obj icon
	*** @OUT: /
	*** @THROW: /
	**/
	public function setIconName($name)
	{
		$this->icon_name = $name;
	}
	/**
	***
	*** @NAME: setIconType
	*** @DESC: Set plan obj icon type name
	*** @IN: (STRING) $type => type name of the plan obj icon
	*** @OUT: /
	*** @THROW: /
	**/
	public function setIconType($type)
	{
		$this->icon_type = $type;
	}
	/**
	***
	*** @NAME: setIconPath
	*** @DESC: Set plan obj icon path
	*** @IN: (STRING) $path => path of the plan obj icon
	*** @OUT: /
	*** @THROW: /
	**/
	public function setIconPath($path)
	{
		$this->icon_path = $path;
	}
	/**
	***
	*** @NAME: getLinkPlanId
	*** @DESC: Return plan obj plan linked id
	*** @IN: /
	*** @OUT: Id of the plan linked to the object
	*** @THROW: /
	**/
	public function getLinkPlanId()
	{
		return $this->link_plan_id;
	}
	/**
	***
	*** @NAME: getIconId
	*** @DESC: Return plan obj icon id
	*** @IN: /
	*** @OUT: Id of the plan obj icon
	*** @THROW: /
	**/
	public function getIconId()
	{
		return $this->icon_id;
	}
	/**
	***
	*** @NAME: getIconName
	*** @DESC: Return plan obj icon name
	*** @IN: /
	*** @OUT: Name of the plan obj icon
	*** @THROW: /
	**/
	public function getIconName()
	{
		return $this->icon_name;
	}
	/**
	***
	*** @NAME: getIconType
	*** @DESC: Return plan obj icon type name
	*** @IN: /
	*** @OUT: Type name of the plan obj icon
	*** @THROW: /
	**/
	public function getIconType()
	{
		return $this->icon_type;
	}
	/**
	***
	*** @NAME: getIconPath
	*** @DESC: Return plan obj icon path
	*** @IN: /
	*** @OUT: Path of the plan obj icon
	*** @THROW: /
	**/
	public function getIconPath()
	{
		return $this->icon_path;
	}
	/**
	***
	*** @NAME: getIconFile
	*** @DESC: Return plan obj icon file to display in function of the current state. Set the state first by using setState($dbconnection, $userId)
	*** @IN: (STRING) $path => $_SERVER['DOCUMENT_ROOT']. Only used to check if the file exist
	*** @OUT: array [path => Path of the plan obj icon file, isUniq => true if there is only one icon for all states (display css rectangle in state color ibstead of colored icon)]
	*** @THROW: /
	**/
	public function getIconFile($path)
	{
		// Set icon as icon state
		$file = array(
			'path' => $this->icon_path . $this->icon_name . "_" . $this->state . "." . $this->icon_type,
			'isUniq' => FALSE,
			'isInDefaultState' => $this->isInDefaultState()
		);

		// If the icon for this state doesn't exist set the icon as the ok icon for this icon name (default if uploaded only one icon)
		if(!file_exists($path . $file['path']))
		{
			$file['path'] = $this->icon_path . $this->icon_name . "_ok." . $this->icon_type;
			$file['isUniq'] = TRUE;

			// If no icons are found for this icon name set the default icon
			if(!file_exists($path . $file['path']))
			{
				$file['path'] = "/data/plan/icons/default_" . $this->state . ".svg";
				$file['isUniq'] = FALSE;
			}
		}
		elseif($this->state == 'ok' && !file_exists($path . $this->icon_path . $this->icon_name . "_critical." . $this->icon_type))
		{
			// Check for other icons to know if it's an uniq icon because ok icon is always present
			$file['isUniq'] = TRUE;
		}

		return $file;
	}
	/**
	***
	*** @NAME: isInDefaultState
	*** @DESC: Check if the object state is in default (OK) or calculated by elements linked behind it. Only for objects of type_name container
	*** @OUT: boolean => true if the object is in ok state by default (no linked element checked)
	*** @THROW: /
	**/
	public function isInDefaultState()
	{
		// Set to false for other types than container to avoid issues due to parameters mismatches
		if($this->type_name != 'container')
		{
			return FALSE;
		}
		// Check if state check has been ignored by user
		if(!array_key_exists('getstate', $this->params) || $this->params['getstate'] != 'no')
		{
			// Plan linked
			if(!empty($this->link_plan_id))
			{
				return FALSE;
			}
			// Nodes
			if(!empty($this->params['nodes']))
			{
				return FALSE;
			}
			// Groups
			if(!empty($this->params['groups']))
			{
				return FALSE;
			}
			// Services
			if(!empty($this->params['services']))
			{
				return FALSE;
			}
		}

		return TRUE;
	}
}
?>
