CREATE TABLE public.plan_folders
(
   folder_id serial, 
   folder_name character varying(35), 
   folder_parent_id integer, 
   CONSTRAINT "plan_folders_primaryKey" PRIMARY KEY (folder_id)
) 
WITH (
  OIDS = FALSE
)
;

ALTER TABLE plan_plans
   ADD COLUMN plan_folder_id integer;

ALTER TABLE plan_plans
  ADD CONSTRAINT plan_foreignkey_folder FOREIGN KEY (plan_folder_id) REFERENCES plan_folders (folder_id)
   ON UPDATE NO ACTION ON DELETE NO ACTION;
CREATE INDEX fki_plan_foreignkey_folder
  ON plan_plans(plan_folder_id);



-- Table: plan_users_access

-- DROP TABLE plan_users_access;

CREATE TABLE plan_users_access
(
  plan_access_user_id integer NOT NULL,
  plan_access_plan_id integer NOT NULL,
  plan_access_privileges character varying(10) NOT NULL DEFAULT 'READ'::character varying,
  CONSTRAINT "plan_access_primaryKey" PRIMARY KEY (plan_access_user_id, plan_access_plan_id, plan_access_privileges),
  CONSTRAINT "plan_access_ForeignKey_user_id" FOREIGN KEY (plan_access_user_id)
      REFERENCES esia_users (user_id) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION,
  CONSTRAINT "plan_access_ForeingKey_plan_id" FOREIGN KEY (plan_access_plan_id)
      REFERENCES plan_plans (plan_id) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION
)
WITH (
  OIDS=FALSE
);
ALTER TABLE plan_users_access
  OWNER TO esia;




--DROP FUNCTION getallParentPlanFolder(integer[]);

CREATE OR REPLACE FUNCTION getallParentPlanFolder(IN folderid integer[])
  RETURNS TABLE(fold_id integer, fold_name character varying, fold_parent_id integer) AS
$BODY$
    BEGIN
IF folderid IS NULL
THEN
RETURN QUERY SELECT folder_id,folder_name,folder_parent_id
             FROM plan_folders AS f;
ELSE
	RETURN QUERY
		WITH RECURSIVE q AS (SELECT folder_id,folder_name,folder_parent_id
			FROM plan_folders
			WHERE folder_id = ANY (folderid)
			UNION ALL
			SELECT m.folder_id, m.folder_name,m.folder_parent_id
			FROM plan_folders m
			JOIN q ON q.folder_parent_id = m.folder_id)
		SELECT * FROM q;
END IF;
   END;
$BODY$
  LANGUAGE plpgsql VOLATILE
  COST 100
  ROWS 1000;
ALTER FUNCTION getallParentPlanFolder(integer[])
  OWNER TO esia;


CREATE OR REPLACE FUNCTION getallParentPlanFolder(IN folderid integer)
  RETURNS TABLE(fold_id integer, fold_name character varying, fold_parent_id integer) AS
$BODY$
    BEGIN
IF folderid IS NULL
THEN
RETURN QUERY SELECT folder_id,folder_name,folder_parent_id
             FROM plan_folders AS f;
ELSE
	RETURN QUERY
		WITH RECURSIVE q AS (SELECT folder_id,folder_name,folder_parent_id
			FROM plan_folders
			WHERE folder_id = folderid
			UNION ALL
			SELECT m.folder_id, m.folder_name,m.folder_parent_id
			FROM plan_folders m
			JOIN q ON q.folder_parent_id = m.folder_id)
		SELECT * FROM q;
END IF;
   END;
$BODY$
  LANGUAGE plpgsql VOLATILE
  COST 100
  ROWS 1000;
ALTER FUNCTION getallParentPlanFolder(integer)
  OWNER TO esia;

CREATE OR REPLACE FUNCTION getallchildplanfolder(IN folderid integer)
  RETURNS TABLE(fold_id integer, fold_name character varying, fold_parent_id integer) AS
$BODY$
    BEGIN
	IF folderid IS NULL
	THEN
	RETURN QUERY SELECT folder_id,folder_name,folder_parent_id
		     FROM plan_folders AS f;
	ELSE
		RETURN QUERY
			WITH RECURSIVE q AS (SELECT folder_id,folder_name,folder_parent_id
				FROM plan_folders
				WHERE folder_id = folderid
				UNION ALL
				SELECT m.folder_id, m.folder_name,m.folder_parent_id
				FROM plan_folders m
				JOIN q ON m.folder_parent_id = q.folder_id)
			SELECT * FROM q;
	END IF;
   END;
$BODY$
  LANGUAGE plpgsql VOLATILE
  COST 100
  ROWS 1000;
ALTER FUNCTION getallchildplanfolder(integer)
  OWNER TO esia;
INSERT INTO esia_privileges(privilege_id, privilege_name, privilege_site, privilege_description) VALUES (default,'map_create_folder' ,1,'Autorise la création de répertoire dans le plugin de plan');
INSERT INTO esia_privileges(privilege_id, privilege_name, privilege_site, privilege_description) VALUES (default,'map_create_plan' ,1,'Autorise la création de plan dans le plugin de plan');



INSERT INTO esia_privilege_grantedto( grant_group_id, grant_privilege_id, grant_acces) VALUES ((SELECT group_id FROM esia_groups WHERE group_name='root'), (SELECT privilege_id FROM esia_privileges WHERE privilege_name='map_create_folder'), 1);
INSERT INTO esia_privilege_grantedto( grant_group_id, grant_privilege_id, grant_acces) VALUES ((SELECT group_id FROM esia_groups WHERE group_name='root'), (SELECT privilege_id FROM esia_privileges WHERE privilege_name='map_create_plan'), 1);


--Suppression accès google map
DELETE FROM plan_types WHERE type_name='googlemap';
--Ajout des droit d'accès à tous les plan
INSERT INTO plan_users_access(plan_access_user_id, plan_access_plan_id, plan_access_privileges) 
SELECT user_id,plan_id,'WRITE' FROM esia_users,plan_plans WHERE user_id  IN ( SELECT user_id
			                                                                    FROM esia_users
			                                                                    JOIN esia_memberof ON member_user_id = user_id
			                                                                    JOIN esia_groups ON member_group_id = group_id
			                                                                    WHERE group_name='root');
