-- Table: plan_types

-- DROP TABLE plan_types;

CREATE TABLE plan_types
(
  type_id serial NOT NULL,
  type_name character varying NOT NULL,
  type_controller character varying,
  CONSTRAINT "plan_type_PrimaryKey" PRIMARY KEY (type_id),
  CONSTRAINT plan_type_name_unique UNIQUE (type_name)
)
WITH (
  OIDS=FALSE
);
ALTER TABLE plan_types
  OWNER TO esia;

-- Table: plan_plans

-- DROP TABLE plan_plans;

CREATE TABLE plan_plans
(
  plan_id serial NOT NULL,
  plan_name character varying NOT NULL,
  plan_params json,
  plan_type_id integer,
  CONSTRAINT "plan_PrimaryKey" PRIMARY KEY (plan_id),
  CONSTRAINT "plan_foreignKey_type_id" FOREIGN KEY (plan_type_id)
      REFERENCES plan_types (type_id) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION
)
WITH (
  OIDS=FALSE
);
ALTER TABLE plan_plans
  OWNER TO esia;
  
-- Table : plan_icon_groups

-- DROP TABLE plan_icon_groups

CREATE TABLE plan_icon_groups
(
  group_id serial NOT NULL,
  group_name character varying NOT NULL,
  CONSTRAINT "icon_group_PrimaryKey" PRIMARY KEY (group_id)
)
WITH (
  OIDS=FALSE
);
ALTER TABLE plan_icon_groups
  OWNER TO esia;

-- Table: plan_object_icons

-- DROP TABLE plan_object_icons;

CREATE TABLE plan_object_icons
(
  icon_id serial NOT NULL,
  icon_name character varying NOT NULL,
  icon_path character varying,
  icon_type character varying,
  icon_group_id integer,
  CONSTRAINT "icon_PrimaryKey" PRIMARY KEY (icon_id),
  CONSTRAINT icon_name_unique UNIQUE (icon_name),
  CONSTRAINT "icon_group_foreignKey_type_id" FOREIGN KEY (icon_group_id)
      REFERENCES plan_icon_groups (group_id) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION
)
WITH (
  OIDS=FALSE
);
ALTER TABLE plan_object_icons
  OWNER TO esia;

-- Table: plan_object_types

-- DROP TABLE plan_object_types;

CREATE TABLE plan_object_types
(
  type_id serial NOT NULL,
  type_name character varying NOT NULL,
  type_controller character varying,
  type_default_params json,
  CONSTRAINT "object_type_PrimaryKey" PRIMARY KEY (type_id),
  CONSTRAINT object_type_name_unique UNIQUE (type_name)
)
WITH (
  OIDS=FALSE
);
ALTER TABLE plan_object_types
  OWNER TO esia;

-- Table: plan_objects

-- DROP TABLE plan_objects;

CREATE TABLE plan_objects
(
  object_id serial NOT NULL,
  object_name character varying NOT NULL,
  object_on_plan_id INTEGER NOT NULL,
  object_link_plan_id INTEGER,
  object_icon_id INTEGER,
  object_type_id INTEGER,
  object_params json,
  CONSTRAINT "object_PrimaryKey" PRIMARY KEY (object_id),
  CONSTRAINT "object_foreignKey_on_plan_id" FOREIGN KEY (object_on_plan_id)
      REFERENCES plan_plans (plan_id) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION,
  CONSTRAINT "object_foreignKey_link_plan_id" FOREIGN KEY (object_link_plan_id)
      REFERENCES plan_plans (plan_id) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION,
  CONSTRAINT "object_foreignKey_icon_id" FOREIGN KEY (object_icon_id)
      REFERENCES plan_object_icons (icon_id) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION,
  CONSTRAINT "object_foreignKey_type_id" FOREIGN KEY (object_type_id)
      REFERENCES plan_object_types (type_id) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION
)
WITH (
  OIDS=FALSE
);
ALTER TABLE plan_objects
  OWNER TO esia;


INSERT INTO esia_web_plugin VALUES (DEFAULT,'WEBP_PLAN','lang/',NULL,'plan_control.php?action=plan',NULL,TRUE,'universal','/plugin/plan/',NULL,TRUE,'/plugin/plan/images/map3.png');

INSERT INTO plan_object_types VALUES (DEFAULT,'link',NULL,NULL);
INSERT INTO plan_object_types VALUES (DEFAULT,'plan',NULL,NULL);
INSERT INTO plan_object_types VALUES (DEFAULT,'container',NULL,NULL);
INSERT INTO plan_object_types VALUES (DEFAULT,'navigation',NULL,NULL);
INSERT INTO plan_object_types VALUES (DEFAULT,'text',NULL,NULL);
INSERT INTO plan_object_types VALUES (DEFAULT,'map',NULL,NULL);
INSERT INTO plan_object_types VALUES (DEFAULT,'zone',NULL,NULL);

INSERT INTO plan_types VALUES (DEFAULT,'image','plan_control.php?action=plan');
INSERT INTO plan_types VALUES (DEFAULT,'googlemap','plan_control.php?action=googlemap');

INSERT INTO plan_icon_groups VALUES (DEFAULT,'ESIA');

INSERT INTO plan_object_icons VALUES (DEFAULT,'default','/data/plan/icons/','svg',(SELECT group_id FROM plan_icon_groups WHERE group_name='ESIA'));
INSERT INTO plan_object_icons VALUES (DEFAULT,'switch','/data/plan/icons/','svg',(SELECT group_id FROM plan_icon_groups WHERE group_name='ESIA'));
INSERT INTO plan_object_icons VALUES (DEFAULT,'server','/data/plan/icons/','svg',(SELECT group_id FROM plan_icon_groups WHERE group_name='ESIA'));
INSERT INTO plan_object_icons VALUES (DEFAULT,'router','/data/plan/icons/','svg',(SELECT group_id FROM plan_icon_groups WHERE group_name='ESIA'));
INSERT INTO plan_object_icons VALUES (DEFAULT,'cluster','/data/plan/icons/','svg',(SELECT group_id FROM plan_icon_groups WHERE group_name='ESIA'));
INSERT INTO plan_object_icons VALUES (DEFAULT,'localisation','/data/plan/icons/','png',(SELECT group_id FROM plan_icon_groups WHERE group_name='ESIA'));
INSERT INTO plan_object_icons VALUES (DEFAULT,'firewall','/data/plan/icons/','svg',(SELECT group_id FROM plan_icon_groups WHERE group_name='ESIA'));
INSERT INTO plan_object_icons VALUES (DEFAULT,'backup','/data/plan/icons/','svg',(SELECT group_id FROM plan_icon_groups WHERE group_name='ESIA'));
INSERT INTO plan_object_icons VALUES (DEFAULT,'database','/data/plan/icons/','svg',(SELECT group_id FROM plan_icon_groups WHERE group_name='ESIA'));
INSERT INTO plan_object_icons VALUES (DEFAULT,'disk','/data/plan/icons/','svg',(SELECT group_id FROM plan_icon_groups WHERE group_name='ESIA'));
INSERT INTO plan_object_icons VALUES (DEFAULT,'file','/data/plan/icons/','svg',(SELECT group_id FROM plan_icon_groups WHERE group_name='ESIA'));
INSERT INTO plan_object_icons VALUES (DEFAULT,'interface','/data/plan/icons/','svg',(SELECT group_id FROM plan_icon_groups WHERE group_name='ESIA'));
INSERT INTO plan_object_icons VALUES (DEFAULT,'pc','/data/plan/icons/','svg',(SELECT group_id FROM plan_icon_groups WHERE group_name='ESIA'));
INSERT INTO plan_object_icons VALUES (DEFAULT,'phone','/data/plan/icons/','svg',(SELECT group_id FROM plan_icon_groups WHERE group_name='ESIA'));
INSERT INTO plan_object_icons VALUES (DEFAULT,'ping','/data/plan/icons/','svg',(SELECT group_id FROM plan_icon_groups WHERE group_name='ESIA'));
INSERT INTO plan_object_icons VALUES (DEFAULT,'printer','/data/plan/icons/','svg',(SELECT group_id FROM plan_icon_groups WHERE group_name='ESIA'));
INSERT INTO plan_object_icons VALUES (DEFAULT,'process','/data/plan/icons/','svg',(SELECT group_id FROM plan_icon_groups WHERE group_name='ESIA'));
INSERT INTO plan_object_icons VALUES (DEFAULT,'unity','/data/plan/icons/','svg',(SELECT group_id FROM plan_icon_groups WHERE group_name='ESIA'));
INSERT INTO plan_object_icons VALUES (DEFAULT,'rack','/data/plan/icons/','svg',(SELECT group_id FROM plan_icon_groups WHERE group_name='ESIA'));
INSERT INTO plan_object_icons VALUES (DEFAULT,'cloud','/data/plan/icons/','png',(SELECT group_id FROM plan_icon_groups WHERE group_name='ESIA'));
INSERT INTO plan_object_icons VALUES (DEFAULT,'legend_wm','/data/plan/icons/','png',(SELECT group_id FROM plan_icon_groups WHERE group_name='ESIA'));
INSERT INTO plan_object_icons VALUES (DEFAULT,'esia','/data/plan/icons/','png',(SELECT group_id FROM plan_icon_groups WHERE group_name='ESIA'));

INSERT INTO dashboard_view VALUES (DEFAULT,'DASHBOARD_PLAN','/dashboard/view_plan.php?id=:widgetid&type=plan',30,100,300,'{"title":{"style":"text-align: center; font-size:15px;color:#575759;"},"description":{"style":"text-align: center; margin:auto;font-size:12px;"},"plan":""}');
